@@ -21,6 +21,7 @@
 
 #include "ompi_config.h"
 #include "opal/class/opal_atomic_lifo.h"
+#include "opal/prefetch.h"
 #include "opal/threads/threads.h"
 #include "opal/threads/condition.h"
 #include "ompi/constants.h"
@@ -31,6 +32,10 @@ extern "C" {
 #endif
 
 struct mca_mem_pool_t;
+struct ompi_free_list_item_t;
+    
+typedef void (*ompi_free_list_item_init_fn_t) (
+        struct ompi_free_list_item_t*, void* ctx);
 
 struct ompi_free_list_t
 {
@@ -39,34 +44,31 @@ struct ompi_free_list_t
     size_t fl_num_allocated;
     size_t fl_num_per_alloc;
     size_t fl_num_waiting;
-    size_t fl_elem_size;
-    size_t fl_header_space;
-    size_t fl_alignment;
-    opal_class_t* fl_elem_class;
+    size_t fl_frag_size;                /* size of the fragment descriptor */
+    size_t fl_frag_alignment;           /* fragment descriptor alignment */
+    size_t fl_payload_buffer_size;      /* size of payload buffer */
+    size_t fl_payload_buffer_alignment; /* payload buffer alignment */
+    opal_class_t* fl_frag_class;
     struct mca_mpool_base_module_t* fl_mpool;
     opal_mutex_t fl_lock;
     opal_condition_t fl_condition; 
     opal_list_t fl_allocations;
+    ompi_free_list_item_init_fn_t item_init;
+    void* ctx;
 };
 typedef struct ompi_free_list_t ompi_free_list_t;
 OMPI_DECLSPEC OBJ_CLASS_DECLARATION(ompi_free_list_t);
 
+struct mca_mpool_base_registration_t;
 struct ompi_free_list_item_t
 { 
     opal_list_item_t super; 
-    void* user_data; 
+    struct mca_mpool_base_registration_t *registration;
+    void *ptr;
 }; 
 typedef struct ompi_free_list_item_t ompi_free_list_item_t; 
-
 OMPI_DECLSPEC OBJ_CLASS_DECLARATION(ompi_free_list_item_t);
 
-struct ompi_free_list_memory_t {
-    opal_list_item_t super;
-    struct mca_mpool_base_registration_t *registration;
-};
-typedef struct ompi_free_list_memory_t ompi_free_list_memory_t;
-OMPI_DECLSPEC OBJ_CLASS_DECLARATION(ompi_free_list_memory_t);
-
 /**
  * Initialize a free list.
  *
@@ -82,13 +84,15 @@ OMPI_DECLSPEC OBJ_CLASS_DECLARATION(ompi_free_list_memory_t);
 OMPI_DECLSPEC int ompi_free_list_init_ex(
     ompi_free_list_t *free_list, 
     size_t element_size,
-    size_t header_size,
     size_t alignment,
     opal_class_t* element_class,
     int num_elements_to_alloc,
     int max_elements_to_alloc,
     int num_elements_per_alloc,
-    struct mca_mpool_base_module_t*);
+    struct mca_mpool_base_module_t*,
+    ompi_free_list_item_init_fn_t item_init,
+    void *ctx
+    );
 
 static inline int ompi_free_list_init(
     ompi_free_list_t *free_list, 
@@ -99,11 +103,75 @@ static inline int ompi_free_list_init(
     int num_elements_per_alloc,
     struct mca_mpool_base_module_t* mpool)
 {
-    return ompi_free_list_init_ex(free_list, element_size, 0, CACHE_LINE_SIZE,
+    return ompi_free_list_init_ex(free_list, element_size, CACHE_LINE_SIZE,
             element_class, num_elements_to_alloc, max_elements_to_alloc,
-            num_elements_per_alloc, mpool);
+                                  num_elements_per_alloc, mpool, NULL, NULL);
 }
 
+/**
+ * Initialize a free list.  - this will replace ompi_free_list_init_ex
+ *
+ * @param free_list                (IN)  Free list.
+ * @param frag_size                (IN)  Size of each element - allocated by malloc.
+ * @param frag_alignment           (IN)  Fragment alignment.
+ * @param frag_class               (IN)  opal_class_t of element - used to initialize allocated elements.
+ * @param payload_buffer_size      (IN)  Size of payload buffer - allocated from mpool.
+ * @param payload_buffer_alignment (IN)  Payload buffer alignment.
+ * @param num_elements_to_alloc    (IN)  Initial number of elements to allocate.
+ * @param max_elements_to_alloc    (IN)  Maximum number of elements to allocate.
+ * @param num_elements_per_alloc   (IN)  Number of elements to grow by per allocation.
+ * @param mpool                    (IN)  Optional memory pool for allocation.s
+ * @param item_init                (IN)
+ * @param ctx                      (IN)
+ */
+ 
+OMPI_DECLSPEC int ompi_free_list_init_ex_new(
+    ompi_free_list_t *free_list, 
+    size_t frag_size,
+    size_t frag_alignment,
+    opal_class_t* frag_class,
+    size_t payload_buffer_size,
+    size_t payload_buffer_alignment,
+    int num_elements_to_alloc,
+    int max_elements_to_alloc,
+    int num_elements_per_alloc,
+    struct mca_mpool_base_module_t*,
+    ompi_free_list_item_init_fn_t item_init,
+    void *ctx
+    );
+
+/**
+ * Initialize a free list. - this will replace ompi_free_list_init
+ *
+ * @param free_list                (IN)  Free list.
+ * @param frag_size                (IN)  Size of each element - allocated by malloc.
+ * @param frag_alignment           (IN)  Fragment alignment.
+ * @param frag_class               (IN)  opal_class_t of element - used to initialize allocated elements.
+ * @param payload_buffer_size      (IN)  Size of payload buffer - allocated from mpool.
+ * @param payload_buffer_alignment (IN)  Payload buffer alignment.
+ * @param num_elements_to_alloc    (IN)  Initial number of elements to allocate.
+ * @param max_elements_to_alloc    (IN)  Maximum number of elements to allocate.
+ * @param num_elements_per_alloc   (IN)  Number of elements to grow by per allocation.
+ * @param mpool                    (IN)  Optional memory pool for allocation.s
+ */
+static inline int ompi_free_list_init_new(
+    ompi_free_list_t *free_list, 
+    size_t frag_size,
+    size_t frag_alignment,
+    opal_class_t* frag_class,
+    size_t payload_buffer_size,
+    size_t payload_buffer_alignment,
+    int num_elements_to_alloc,
+    int max_elements_to_alloc,
+    int num_elements_per_alloc,
+    struct mca_mpool_base_module_t* mpool)
+{
+    return ompi_free_list_init_ex_new(free_list,
+            frag_size, frag_alignment, frag_class,
+            payload_buffer_size, payload_buffer_alignment,
+            num_elements_to_alloc, max_elements_to_alloc,
+            num_elements_per_alloc, mpool, NULL, NULL);
+}
 
 OMPI_DECLSPEC int ompi_free_list_grow(ompi_free_list_t* flist, size_t num_elements);
 
@@ -122,7 +190,7 @@ typedef struct ompi_free_list_pos_t {
     unsigned char* last_memory;
     unsigned char* last_item;
 } ompi_free_list_pos_t;
-#define OMPI_FREE_LIST_POS_BEGINING {NULL, NULL}
+#define OMPI_FREE_LIST_POS_BEGINNING {NULL, NULL}
 
 OMPI_DECLSPEC int ompi_free_list_parse( ompi_free_list_t* list,
                                         ompi_free_list_pos_t* position,
@@ -142,22 +210,19 @@ OMPI_DECLSPEC int ompi_free_list_parse( ompi_free_list_t* list,
  
 #define OMPI_FREE_LIST_GET(fl, item, rc) \
 { \
-    if(opal_using_threads()) { \
-        item = (ompi_free_list_item_t*) opal_atomic_lifo_pop(&((fl)->super)); \
-        if(NULL == item) { \
+    rc = OMPI_SUCCESS; \
+    item = (ompi_free_list_item_t*) opal_atomic_lifo_pop(&((fl)->super)); \
+    if( OPAL_UNLIKELY(NULL == item) ) { \
+        if(opal_using_threads()) { \
             opal_mutex_lock(&((fl)->fl_lock)); \
             ompi_free_list_grow((fl), (fl)->fl_num_per_alloc); \
             opal_mutex_unlock(&((fl)->fl_lock)); \
-            item = (ompi_free_list_item_t*) opal_atomic_lifo_pop(&((fl)->super)); \
-        } \
-    } else { \
-        item = (ompi_free_list_item_t*) opal_atomic_lifo_pop(&((fl)->super)); \
-        if(NULL == item) { \
+        } else { \
             ompi_free_list_grow((fl), (fl)->fl_num_per_alloc); \
-            item =(ompi_free_list_item_t*) opal_atomic_lifo_pop(&((fl)->super)); \
         } \
+        item = (ompi_free_list_item_t*) opal_atomic_lifo_pop(&((fl)->super)); \
+        if( OPAL_UNLIKELY(NULL == item) ) rc = OMPI_ERR_TEMP_OUT_OF_RESOURCE; \
     }  \
-    rc = (NULL == item) ?  OMPI_ERR_TEMP_OUT_OF_RESOURCE : OMPI_SUCCESS; \
 } 
 
 /**
