@@ -50,9 +50,10 @@ void ompi_rb_tree_construct(opal_object_t * object)
     ompi_rb_tree_t * tree = (ompi_rb_tree_t *) object;
     tree->root_ptr = NULL;
     OBJ_CONSTRUCT(&(tree->free_list), ompi_free_list_t);
-    ompi_free_list_init(&(tree->free_list), sizeof(ompi_rb_tree_node_t),
-                        OBJ_CLASS(ompi_rb_tree_node_t),
-                        0, -1 , 128, NULL);
+    ompi_free_list_init_new(&(tree->free_list), sizeof(ompi_rb_tree_node_t),
+            CACHE_LINE_SIZE, OBJ_CLASS(ompi_rb_tree_node_t),
+            0,CACHE_LINE_SIZE,
+            0, -1 , 128, NULL);
 }
 
 /* the destructor function */
@@ -169,22 +170,22 @@ int ompi_rb_tree_insert(ompi_rb_tree_t *tree, void * key, void * value)
     return OMPI_SUCCESS;
 }
 
-
 /* Finds the node in the tree based on the key */
-void * ompi_rb_tree_find(ompi_rb_tree_t *tree, void *key)
+void * ompi_rb_tree_find_with(ompi_rb_tree_t *tree, void *key,
+        ompi_rb_tree_comp_fn_t compfn)
 {
     ompi_rb_tree_node_t * node;
     int compvalue;
 
     node = tree->root_ptr->left;
     while (node != tree->nill) {
-        compvalue = tree->comp(key, node->key);
+        compvalue = compfn(key, node->key);
         /* if the result of the comparison function is 0, we found it */
         if (compvalue == 0) {
             return(node->value);
         }
         /* else if it is less than 0, go left, else right */
-        (compvalue < 0) ? (node = node->left) : (node = node->right);
+        node = ((compvalue < 0) ? node->left : node->right);
     }
     /* if we didn't find anything, return NULL */
     return(NULL);
@@ -193,7 +194,7 @@ void * ompi_rb_tree_find(ompi_rb_tree_t *tree, void *key)
 /* Finds the node in the tree based on the key and returns a pointer
  * to the node. This is a bit a code duplication, but this has to be fast
  * so we go ahead with the duplication */
-ompi_rb_tree_node_t * ompi_rb_tree_find_node(ompi_rb_tree_t *tree, void *key)
+static ompi_rb_tree_node_t * ompi_rb_tree_find_node(ompi_rb_tree_t *tree, void *key)
 {
     ompi_rb_tree_node_t * node;
     int compvalue;
@@ -206,7 +207,7 @@ ompi_rb_tree_node_t * ompi_rb_tree_find_node(ompi_rb_tree_t *tree, void *key)
             return(node);
         }
         /* else if it is less than 0, go left, else right */
-        (compvalue < 0) ? (node = node->left) : (node = node->right);
+        node = ((compvalue < 0) ? node->left : node->right);
     }
     /* if we didn't find anything, return NULL */
     return(NULL);
@@ -284,7 +285,7 @@ int ompi_rb_tree_destroy(ompi_rb_tree_t *tree)
 
 /* Find the next inorder successor of a node    */
 
-ompi_rb_tree_node_t * btree_successor(ompi_rb_tree_t * tree, ompi_rb_tree_node_t * node)
+static ompi_rb_tree_node_t * btree_successor(ompi_rb_tree_t * tree, ompi_rb_tree_node_t * node)
 {
     ompi_rb_tree_node_t * p;
 
@@ -311,7 +312,7 @@ ompi_rb_tree_node_t * btree_successor(ompi_rb_tree_t * tree, ompi_rb_tree_node_t
 /* Insert an element in the normal binary search tree fashion    */
 /* this function goes through the tree and finds the leaf where
  * the node will be inserted   */
-void btree_insert(ompi_rb_tree_t *tree, ompi_rb_tree_node_t * node)
+static void btree_insert(ompi_rb_tree_t *tree, ompi_rb_tree_node_t * node)
 {
     ompi_rb_tree_node_t * parent = tree->root_ptr;
     ompi_rb_tree_node_t * n = parent->left; /* the real root of the tree */
@@ -325,7 +326,7 @@ void btree_insert(ompi_rb_tree_t *tree, ompi_rb_tree_node_t * node)
     /* find the leaf where we will insert the node */
     while (n != tree->nill) {
         parent = n;
-        (tree->comp(node->key, n->key) <= 0) ? (n = n->left) : (n = n->right);
+        n = ((tree->comp(node->key, n->key) <= 0) ? n->left : n->right);
     }
 
     /* place it on either the left or the right */
@@ -344,7 +345,7 @@ void btree_insert(ompi_rb_tree_t *tree, ompi_rb_tree_node_t * node)
 }
 
 /* Fixup the balance of the btree after deletion    */
-void btree_delete_fixup(ompi_rb_tree_t *tree, ompi_rb_tree_node_t * x)
+static void btree_delete_fixup(ompi_rb_tree_t *tree, ompi_rb_tree_node_t * x)
 {
     ompi_rb_tree_node_t * w;
     ompi_rb_tree_node_t * root = tree->root_ptr->left;
@@ -408,7 +409,7 @@ void btree_delete_fixup(ompi_rb_tree_t *tree, ompi_rb_tree_node_t * x)
 
 /* Free the nodes in inorder fashion    */
 
-void
+static void
 inorder_destroy(ompi_rb_tree_t *tree, ompi_rb_tree_node_t * node)
 {
     ompi_free_list_item_t * item;
@@ -470,7 +471,7 @@ static void inorder_traversal(ompi_rb_tree_t *tree,
 /* Left rotate the tree    */
 /* basically what we want to do is to make x be the left child
  * of its right child    */
-void left_rotate(ompi_rb_tree_t *tree, ompi_rb_tree_node_t * x)
+static void left_rotate(ompi_rb_tree_t *tree, ompi_rb_tree_node_t * x)
 {
     ompi_rb_tree_node_t * y;
 
@@ -501,7 +502,7 @@ void left_rotate(ompi_rb_tree_t *tree, ompi_rb_tree_node_t * x)
 /* Right rotate the tree    */
 /* basically what we want to do is to make x be the right child
  * of its left child */
-void right_rotate(ompi_rb_tree_t *tree, ompi_rb_tree_node_t * x)
+static void right_rotate(ompi_rb_tree_t *tree, ompi_rb_tree_node_t * x)
 {
     ompi_rb_tree_node_t * y;
 
