@@ -1239,7 +1239,11 @@ int fscanf(FILE *stream, const char *format, ...)
 }
 
 
+#if defined(HAVE___FPRINTF_CHK) && HAVE___FPRINTF_CHK
+int __fprintf_chk(FILE *stream, int flag, const char *format, ...)
+#else /* HAVE___FPRINTF_CHK */
 int fprintf(FILE *stream, const char *format, ...)
+#endif /* HAVE___FPRINTF_CHK */
 {
 #define VT_IOWRAP_THISFUNCNAME fprintf
 	int ret;
@@ -1257,7 +1261,11 @@ int fprintf(FILE *stream, const char *format, ...)
 	
 		vt_debug_msg(DBG_IO, "vfprintf\n");
 		va_start (arg, format);
+#if defined(HAVE___FPRINTF_CHK) && HAVE___FPRINTF_CHK
+		ret = __vfprintf_chk(stream, flag, format, arg);
+#else /* HAVE___FPRINTF_CHK */
 		ret = vfprintf(stream, format, arg);
+#endif /* HAVE___FPRINTF_CHK */
 		va_end (arg);
 	
 		num_bytes = (ssize_t)ret;
@@ -1268,7 +1276,11 @@ int fprintf(FILE *stream, const char *format, ...)
 	}
 	else {
 		va_start (arg, format);
+#if defined(HAVE___FPRINTF_CHK) && HAVE___FPRINTF_CHK
+		ret = __vfprintf_chk(stream, flag, format, arg);
+#else /* HAVE___FPRINTF_CHK */
 		ret = vfprintf(stream, format, arg);
+#endif /* HAVE___FPRINTF_CHK */
 		va_end (arg);
 		return ret;
 	}
