@@ -49,7 +49,7 @@ extern int ompi_position_debug;
 static inline void position_predefined_data( ompi_convertor_t* CONVERTOR,
                                              dt_elem_desc_t* ELEM,
                                              uint32_t* COUNT,
-                                             char** POINTER,
+                                             unsigned char** POINTER,
                                              size_t* SPACE )
 {
     uint32_t _copy_count = *(COUNT);
@@ -73,7 +73,7 @@ static inline void position_predefined_data( ompi_convertor_t* CONVERTOR,
 static inline void position_contiguous_loop( ompi_convertor_t* CONVERTOR,
                                              dt_elem_desc_t* ELEM,
                                              uint32_t* COUNT,
-                                             char** POINTER,
+                                             unsigned char** POINTER,
                                              size_t* SPACE )
 {
     ddt_loop_desc_t *_loop = (ddt_loop_desc_t*)(ELEM);
@@ -105,7 +105,7 @@ int ompi_convertor_generic_simple_position( ompi_convertor_t* pConvertor,
     uint16_t type;            /* type at current position */
     dt_elem_desc_t* description = pConvertor->use_desc->desc;
     dt_elem_desc_t* pElem;
-    char *base_pointer = pConvertor->pBaseBuf;
+    unsigned char *base_pointer = pConvertor->pBaseBuf;
     size_t iov_len_local;
     ptrdiff_t extent = pConvertor->pDesc->ub - pConvertor->pDesc->lb;
 
@@ -120,10 +120,10 @@ int ompi_convertor_generic_simple_position( ompi_convertor_t* pConvertor,
     if( iov_len_local > pConvertor->pDesc->size ) {
         pStack = pConvertor->pStack;  /* we're working with the full stack */
         count_desc = (uint32_t)(iov_len_local / pConvertor->pDesc->size);
-        DO_DEBUG( opal_output( 0, "position before %ld asked %ld data size %d"
-                               " iov_len_local %d count_desc %d\n",
-                               pConvertor->bConverted, *position, pConvertor->pDesc->size,
-                               iov_len_local, count_desc ); );
+        DO_DEBUG( opal_output( 0, "position before %lu asked %lu data size %lu"
+                               " iov_len_local %lu count_desc %d\n",
+                               (unsigned long)pConvertor->bConverted, (unsigned long)*position, (unsigned long)pConvertor->pDesc->size,
+                               (unsigned long)iov_len_local, count_desc ); );
         /**
          * Update all the stack except the last one which is supposed to be for
          * the last partial element description.
@@ -133,8 +133,8 @@ int ompi_convertor_generic_simple_position( ompi_convertor_t* pConvertor,
         pConvertor->bConverted += count_desc * pConvertor->pDesc->size;
         iov_len_local = *position - pConvertor->bConverted;
         pStack[0].count -= count_desc;
-        DO_DEBUG( opal_output( 0, "after bConverted %ld remaining count %d iov_len_local %d\n",
-                               pConvertor->bConverted, pStack[0].count, iov_len_local ); );
+        DO_DEBUG( opal_output( 0, "after bConverted %lu remaining count %lu iov_len_local %lu\n",
+                               (unsigned long)pConvertor->bConverted, (unsigned long)pStack[0].count, (unsigned long)iov_len_local ); );
     }
 
     pStack = pConvertor->pStack + pConvertor->stack_pos;
@@ -146,15 +146,16 @@ int ompi_convertor_generic_simple_position( ompi_convertor_t* pConvertor,
     pElem = &(description[pos_desc]); 
     base_pointer += pStack->disp;
     
-    DO_DEBUG( opal_output( 0, "position start pos_desc %d count_desc %d disp %ld\n"
-                           "stack_pos %d pos_desc %d count_desc %d disp %ld\n",
-                           pos_desc, count_desc, base_pointer - pConvertor->pBaseBuf,
-                           pConvertor->stack_pos, pStack->index, pStack->count, pStack->disp ); );
+    DO_DEBUG( opal_output( 0, "position start pos_desc %d count_desc %d disp %llx\n"
+                           "stack_pos %d pos_desc %d count_desc %d disp %llx\n",
+                           pos_desc, count_desc, (unsigned long long)(base_pointer - pConvertor->pBaseBuf),
+                           pConvertor->stack_pos, pStack->index, (int)pStack->count, (unsigned long long)pStack->disp ); );
 
     while( 1 ) {
         if( DT_END_LOOP == pElem->elem.common.type ) { /* end of the current loop */
-            DO_DEBUG( opal_output( 0, "position end_loop count %d stack_pos %d pos_desc %d disp %ld space %d\n",
-                                   pStack->count, pConvertor->stack_pos, pos_desc, pStack->disp, iov_len_local ); );
+            DO_DEBUG( opal_output( 0, "position end_loop count %d stack_pos %d pos_desc %d disp %llx space %lu\n",
+                                   (int)pStack->count, pConvertor->stack_pos, pos_desc,
+                                   (unsigned long long)pStack->disp, (unsigned long)iov_len_local ); );
             if( --(pStack->count) == 0 ) { /* end of loop */
                 if( pConvertor->stack_pos == 0 ) {
                     pConvertor->flags |= CONVERTOR_COMPLETED;
@@ -175,8 +176,9 @@ int ompi_convertor_generic_simple_position( ompi_convertor_t* pConvertor,
             }
             base_pointer = pConvertor->pBaseBuf + pStack->disp;
             UPDATE_INTERNAL_COUNTERS( description, pos_desc, pElem, count_desc );
-            DO_DEBUG( opal_output( 0, "position new_loop count %d stack_pos %d pos_desc %d disp %ld space %d\n",
-                                   pStack->count, pConvertor->stack_pos, pos_desc, pStack->disp, iov_len_local ); );
+            DO_DEBUG( opal_output( 0, "position new_loop count %d stack_pos %d pos_desc %d disp %llx space %lu\n",
+                                   (int)pStack->count, pConvertor->stack_pos, pos_desc,
+                                   (unsigned long long)pStack->disp, (unsigned long)iov_len_local ); );
         }
         if( DT_LOOP == pElem->elem.common.type ) {
             ptrdiff_t local_disp = (ptrdiff_t)base_pointer;
@@ -197,6 +199,9 @@ int ompi_convertor_generic_simple_position( ompi_convertor_t* pConvertor,
             base_pointer = pConvertor->pBaseBuf + pStack->disp;
             UPDATE_INTERNAL_COUNTERS( description, pos_desc, pElem, count_desc );
             DDT_DUMP_STACK( pConvertor->pStack, pConvertor->stack_pos, pElem, "advance loop" );
+            DO_DEBUG( opal_output( 0, "position set loop count %d stack_pos %d pos_desc %d disp %llx space %lu\n",
+                                   (int)pStack->count, pConvertor->stack_pos, pos_desc,
+                                   (unsigned long long)pStack->disp, (unsigned long)iov_len_local ); );
             continue;
         }
         while( pElem->elem.common.flags & DT_FLAG_DATA ) {
@@ -211,6 +216,9 @@ int ompi_convertor_generic_simple_position( ompi_convertor_t* pConvertor,
             base_pointer = pConvertor->pBaseBuf + pStack->disp;
             pos_desc++;  /* advance to the next data */
             UPDATE_INTERNAL_COUNTERS( description, pos_desc, pElem, count_desc );
+            DO_DEBUG( opal_output( 0, "position set loop count %d stack_pos %d pos_desc %d disp %llx space %lu\n",
+                                   (int)pStack->count, pConvertor->stack_pos, pos_desc,
+                                   (unsigned long long)pStack->disp, (unsigned long)iov_len_local ); );
         }
     }
  complete_loop:
@@ -220,8 +228,8 @@ int ompi_convertor_generic_simple_position( ompi_convertor_t* pConvertor,
         /* I complete an element, next step I should go to the next one */
         PUSH_STACK( pStack, pConvertor->stack_pos, pos_desc, DT_BYTE, count_desc,
                     base_pointer - pStack->disp - pConvertor->pBaseBuf );
-        DO_DEBUG( opal_output( 0, "position save stack stack_pos %d pos_desc %d count_desc %d disp %ld\n",
-                               pConvertor->stack_pos, pStack->index, pStack->count, pStack->disp ); );
+        DO_DEBUG( opal_output( 0, "position save stack stack_pos %d pos_desc %d count_desc %d disp %llx\n",
+                               pConvertor->stack_pos, pStack->index, (int)pStack->count, (unsigned long long)pStack->disp ); );
         return 0;
     }
     return 1;
