@@ -2,13 +2,15 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006-2007 Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -22,11 +24,16 @@
 
 #include "opal/mca/mca.h"
 #include "opal/mca/base/base.h"
+#include "orte/util/show_help.h"
+
+#include "orte/util/show_help.h"
+
 #include "opal/mca/base/mca_base_param.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/btl/btl.h"
 #include "ompi/mca/btl/base/base.h"
 
+mca_btl_active_message_callback_t mca_btl_base_active_message_trigger[MCA_BTL_TAG_MAX];
 
 /*
  *  mca_btl_base_descriptor_t
@@ -69,10 +76,11 @@ OBJ_CLASS_INSTANCE(
 int mca_btl_base_output = -1;
 char* mca_btl_base_include = NULL;
 char* mca_btl_base_exclude = NULL;
+int mca_btl_base_warn_component_unused = 1;
 opal_list_t mca_btl_base_components_opened;
 opal_list_t mca_btl_base_modules_initialized;
-
-static bool already_opened = false;
+int mca_btl_base_already_opened = 0;
+bool mca_btl_base_thread_multiple_override = false;
 
 /**
  * Function for finding and opening either all MCA components, or the one
@@ -80,21 +88,33 @@ static bool already_opened = false;
  */
 int mca_btl_base_open(void)
 {
-    if (already_opened) return OMPI_SUCCESS;
-    already_opened = true;
+    int i;
+    if( ++mca_btl_base_already_opened > 1 ) return OMPI_SUCCESS;
 
-    mca_base_param_reg_int_name(
-                                "btl", 
-                                "base_debug", 
-                                "If btl_base_debug is 1 standard debug is output, if > 1 verbose debug is output", 
+    /* Verbose output */
+    mca_base_param_reg_int_name("btl", 
+                                "base_verbose", 
+                                "Verbosity level of the BTL framework", 
                                 false, false, 
                                 0, 
-                                &mca_btl_base_debug);
+                                &mca_btl_base_verbose);
+
+    mca_btl_base_output = opal_output_open(NULL);
+    opal_output_set_verbosity(mca_btl_base_output, mca_btl_base_verbose);
+
+    /* Override the per-BTL "don't run if THREAD_MULTIPLE selected"
+       embargo? */
+    mca_base_param_reg_int_name("btl", 
+                                "base_thread_multiple_override", 
+                                "Enable BTLs that are not normally enabled when MPI_THREAD_MULTIPLE is enabled (THIS IS FOR DEVELOPERS ONLY!  SHOULD NOT BE USED BY END USERS!)",
+                                true, false, 
+                                0, &i);
+    mca_btl_base_thread_multiple_override = OPAL_INT_TO_BOOL(i);
 
   /* Open up all available components */
     
   if (OMPI_SUCCESS != 
-      mca_base_components_open("btl", 0, mca_btl_base_static_components, 
+      mca_base_components_open("btl", mca_btl_base_output, mca_btl_base_static_components,
                                &mca_btl_base_components_opened, true)) {
     return OMPI_ERROR;
   }
@@ -110,6 +130,9 @@ int mca_btl_base_open(void)
       mca_base_param_register_string("btl","base","include",NULL,NULL), &mca_btl_base_include);
   mca_base_param_lookup_string(
       mca_base_param_register_string("btl","base","exclude",NULL,NULL), &mca_btl_base_exclude);
+  mca_base_param_reg_int_name("btl", "base_warn_component_unused",
+      "This parameter is used to turn on warning messages when certain NICs are not used",
+      false, false, 1, &mca_btl_base_warn_component_unused);
 
   /* All done */
   return OMPI_SUCCESS;
