@@ -21,11 +21,6 @@
 #include <sys/time.h>
 #include <time.h>
 #include "ompi/types.h"
-#include "orte/mca/ns/base/base.h"
-#include "orte/mca/oob/base/base.h"
-#include "orte/mca/rml/rml.h"
-#include "orte/mca/errmgr/errmgr.h"
-#include "orte/dss/dss.h"
 #include "btl_gm.h"
 #include "btl_gm_endpoint.h" 
 #include "btl_gm_proc.h"
