@@ -1,9 +1,8 @@
-
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2008 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -29,28 +28,33 @@
 /* Open MPI includes */
 #include "ompi/class/ompi_free_list.h"
 #include "ompi/class/ompi_bitmap.h"
+#include "orte/util/show_help.h"
 #include "opal/event/event.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/btl/btl.h"
 #include "ompi/mca/btl/base/base.h"
-#include "opal/util/output.h"
 #include "ompi/mca/mpool/mpool.h" 
-#include "ompi/mca/btl/btl.h"
 
-#include "myriexpress.h"
+#include <myriexpress.h>
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+#ifdef HAVE_MX_EXTENSIONS_H
+#include <mx_extensions.h>
+#endif  /* HAVE_MX_EXTENSIONS_H */
 
-#define MCA_BTL_HAS_MPOOL 0
+BEGIN_C_DECLS
+
+/**
+ * The mask used for receive and for the PUT protocol
+ */
+#define BTL_MX_RECV_MASK 0x00000000000000ffULL
+#define BTL_MX_PUT_MASK  0xffffffffffffffffULL
 
 /**
  * MX BTL component.
  */
 
 struct mca_btl_mx_component_t {
-    mca_btl_base_component_1_0_0_t          super;  /**< base BTL component */ 
+    mca_btl_base_component_2_0_0_t          super;  /**< base BTL component */ 
     
     int32_t                                 mx_num_btls;
     int32_t                                 mx_max_btls;
@@ -73,6 +77,12 @@ struct mca_btl_mx_component_t {
 
     int32_t                                 mx_support_sharedmem;
     /**< true if we want to activate the MX support for shared memory */
+    int32_t                                 mx_support_self;
+    /**< true if we want to activate the MX support for self communications */
+    int32_t                                 mx_bonding;
+    /**< true if MX is in charge of doing the device bonding */
+    int32_t                                 mx_use_unexpected;
+    /**< true if Open MPI is allowed to register an unexpected handler with the MX library */
 
     opal_list_t mx_procs;  /**< list of mx proc structures */
 
@@ -83,40 +93,43 @@ struct mca_btl_mx_component_t {
     ompi_free_list_t mx_send_eager_frags;      /**< free list of mx eager send fragments */
     ompi_free_list_t mx_send_user_frags;       /**< free list of mx user send fragments */
 
-    ompi_free_list_t mx_recv_frags;            /**< free list of mx recv fragments */
-
-    opal_list_t      mx_pending_acks;          /**< queue of pending sends */
-
     opal_mutex_t     mx_lock;                  /**< lock for accessing module state */
 
-    char* mx_mpool_name;                       /**< name of memory pool */ 
+#if MX_HAVE_MAPPER_STATE
+    char* mx_if_include;                       /**< include the following NICs */
+    char* mx_if_exclude;                       /**< Exclude the following NICs. These
+                                                 *   values are based on the last 6
+                                                 *   digits in hexadecimal of the MAC
+                                                 *   address of the mapper.
+                                                 */
+#endif  /* MX_HAVE_MAPPER_STATE */
 }; 
 typedef struct mca_btl_mx_component_t mca_btl_mx_component_t;
 
-extern mca_btl_mx_component_t mca_btl_mx_component;
+OMPI_MODULE_DECLSPEC extern mca_btl_mx_component_t mca_btl_mx_component;
 
 /**
- * BTL Module Interface
+ * BTL Module Interface.
+ * Each BTL correspond to a high level vision of a network interface. The 
+ * current version of the MX BTL is not able to handle stripping of the
+ * messages by itself. Therefore, it rely on the PML layer for that.
  */
 struct mca_btl_mx_module_t {
-    mca_btl_base_module_t  super;  /**< base BTL interface */
-    mca_btl_base_recv_reg_t mx_reg[MCA_BTL_TAG_MAX]; 
+    mca_btl_base_module_t   super;                   /**< base BTL interface */
+    mx_endpoint_t           mx_endpoint;             /**< local MX endpoint */
+    mx_endpoint_addr_t      mx_endpoint_addr;        /**< local MX endpoint address */
+    uint32_t                mx_unique_network_id;    /**< unique identifier for this BTL,
+                                                      *   based on the MAC address of the
+                                                      *   mapper used to route messages.
+                                                      */
+    opal_list_t             mx_peers;                /**<  list of peers */
 
-    mx_endpoint_t      mx_endpoint;        /**<  */
-    mx_endpoint_addr_t mx_endpoint_addr;   /**<  */
-    opal_list_t        mx_peers;           /**<  */
-
-    int32_t            mx_posted_request;  /**< number of posted MX request */
-    opal_mutex_t       mx_lock;            /**< lock for accessing module state */
-
-#if MCA_BTL_HAS_MPOOL
-    struct mca_mpool_base_module_t* mx_mpool;
-#endif
+    int32_t                 mx_posted_request;       /**< number of posted MX request */
+    opal_mutex_t            mx_lock;                 /**< lock for accessing module state */
 }; 
 typedef struct mca_btl_mx_module_t mca_btl_mx_module_t;
 extern mca_btl_mx_module_t mca_btl_mx_module;
 
-
 /**
  * Register MX component parameters with the MCA framework
  */
@@ -216,35 +229,6 @@ extern int mca_btl_mx_send(
 );
 
 
-/**
- * Initiate an asynchronous put.
- *
- * @param btl (IN)         BTL module
- * @param endpoint (IN)    BTL addressing information
- * @param descriptor (IN)  Description of the data to be transferred
- */
-                                                                                                    
-extern int mca_btl_mx_put(
-    struct mca_btl_base_module_t* btl,
-    struct mca_btl_base_endpoint_t* btl_peer,
-    struct mca_btl_base_descriptor_t* decriptor
-);
-
-
-/**
- * Initiate an asynchronous get.
- *
- * @param btl (IN)         BTL module
- * @param endpoint (IN)    BTL addressing information
- * @param descriptor (IN)  Description of the data to be transferred
- */
-                                                                                                    
-extern int mca_btl_mx_get(
-    struct mca_btl_base_module_t* btl,
-    struct mca_btl_base_endpoint_t* btl_peer,
-    struct mca_btl_base_descriptor_t* decriptor
-);
-
 /**
  * Register a callback function that is called on receipt
  * of a fragment.
@@ -269,9 +253,11 @@ extern int mca_btl_mx_register(
  * @param size (IN)     Request segment size.
  */
 
-extern mca_btl_base_descriptor_t* mca_btl_mx_alloc(
-    struct mca_btl_base_module_t* btl, 
-    size_t size); 
+mca_btl_base_descriptor_t* mca_btl_mx_alloc( struct mca_btl_base_module_t* btl,
+                                             struct mca_btl_base_endpoint_t* endpoint,
+                                             uint8_t order,
+                                             size_t size,
+                                             uint32_t flags);
 
 
 /**
@@ -281,9 +267,8 @@ extern mca_btl_base_descriptor_t* mca_btl_mx_alloc(
  * @param descriptor (IN)  Allocated descriptor.
  */
 
-extern int mca_btl_mx_free(
-    struct mca_btl_base_module_t* btl, 
-    mca_btl_base_descriptor_t* des); 
+int mca_btl_mx_free( struct mca_btl_base_module_t* btl, 
+                     mca_btl_base_descriptor_t* des );
     
 
 /**
@@ -298,27 +283,34 @@ extern int mca_btl_mx_free(
  * @param convertor (IN)    Data type convertor
  * @param reserve (IN)      Additional bytes requested by upper layer to precede user data
  * @param size (IN/OUT)     Number of bytes to prepare (IN), number of bytes actually prepared (OUT) 
-*/
+ */
+mca_btl_base_descriptor_t*
+mca_btl_mx_prepare_src( struct mca_btl_base_module_t* btl,
+                        struct mca_btl_base_endpoint_t* peer,
+                        struct mca_mpool_base_registration_t*,
+                        struct ompi_convertor_t* convertor,
+                        uint8_t order,
+                        size_t reserve,
+                        size_t* size,
+                        uint32_t flags);
 
-mca_btl_base_descriptor_t* mca_btl_mx_prepare_src(
-    struct mca_btl_base_module_t* btl,
-    struct mca_btl_base_endpoint_t* peer,
-    struct mca_mpool_base_registration_t*,
-    struct ompi_convertor_t* convertor,
-    size_t reserve,
-    size_t* size
-);
+mca_btl_base_descriptor_t*
+mca_btl_mx_prepare_dst( struct mca_btl_base_module_t* btl, 
+                        struct mca_btl_base_endpoint_t* peer,
+                        struct mca_mpool_base_registration_t*,
+                        struct ompi_convertor_t* convertor,
+                        uint8_t order,
+                        size_t reserve,
+                        size_t* size,
+                        uint32_t flags);
 
-extern mca_btl_base_descriptor_t* mca_btl_mx_prepare_dst( 
-    struct mca_btl_base_module_t* btl, 
-    struct mca_btl_base_endpoint_t* peer,
-    struct mca_mpool_base_registration_t*,
-    struct ompi_convertor_t* convertor,
-    size_t reserve,
-    size_t* size); 
+/**
+ * Fault Tolerance Event Notification Function
+ * @param state Checkpoint Stae
+ * @return OMPI_SUCCESS or failure status
+ */
+int mca_btl_mx_ft_event(int state);
 
+END_C_DECLS
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
 #endif
