@@ -11,7 +11,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2008 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2006-2008 Mellanox Technologies. All rights reserved.
+ * Copyright (c) 2006-2009 Mellanox Technologies. All rights reserved.
  * Copyright (c) 2006-2007 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2006-2007 Voltaire All rights reserved.
@@ -75,6 +75,14 @@ BEGIN_C_DECLS
  * Infiniband (IB) BTL component.
  */
 
+typedef enum {
+    MCA_BTL_OPENIB_TRANSPORT_IB,
+    MCA_BTL_OPENIB_TRANSPORT_IWARP,
+    MCA_BTL_OPENIB_TRANSPORT_RDMAOE,
+    MCA_BTL_OPENIB_TRANSPORT_UNKNOWN,
+    MCA_BTL_OPENIB_TRANSPORT_SIZE
+} mca_btl_openib_transport_type_t;
+
 typedef enum {
     MCA_BTL_OPENIB_PP_QP,
     MCA_BTL_OPENIB_SRQ_QP,
@@ -95,8 +103,6 @@ struct mca_btl_openib_qp_info_t {
     size_t size;
     int32_t rd_num;
     int32_t rd_low;
-    ompi_free_list_t send_free;     /**< free lists of send buffer descriptors */
-    ompi_free_list_t recv_free;     /**< free lists of receive buffer descriptors */
     union {
         mca_btl_openib_pp_qp_info_t pp_qp;
         mca_btl_openib_srq_qp_info_t srq_qp;
@@ -255,6 +261,8 @@ struct mca_btl_openib_component_t {
     ompi_free_list_t recv_user_free;
     /**< frags for coalesced massages */
     ompi_free_list_t send_free_coalesced;
+    /** Default receive queues */
+    char* default_recv_qps;
 }; typedef struct mca_btl_openib_component_t mca_btl_openib_component_t;
 
 OMPI_MODULE_DECLSPEC extern mca_btl_openib_component_t mca_btl_openib_component;
@@ -273,6 +281,12 @@ typedef struct mca_btl_openib_modex_message_t {
     uint16_t apm_lid;
     /** The MTU used by this port */
     uint8_t mtu;
+    /** vendor id define device type and tuning */
+    uint32_t vendor_id;
+    /** vendor part id define device type and tuning */
+    uint32_t vendor_part_id;
+    /** Transport type of remote port */
+    uint8_t transport_type;
     /** Dummy field used to calculate the real length */
     uint8_t end;
 } mca_btl_openib_modex_message_t;
@@ -488,6 +502,32 @@ extern int mca_btl_openib_send(
     mca_btl_base_tag_t tag
 );
 
+/**
+ * PML->BTL Initiate a immediate send of the specified size.
+ *
+ * @param btl (IN)               BTL instance
+ * @param ep (IN)                Endpoint
+ * @param convertor (IN)         Datatypes converter
+ * @param header (IN)            PML header
+ * @param header_size (IN)       PML header size
+ * @param payload_size (IN)      Payload size
+ * @param order (IN)             Order
+ * @param flags (IN)             Flags
+ * @param tag (IN)               Tag
+ * @param descriptor (OUT)       Messages descriptor
+ */
+extern int mca_btl_openib_sendi( struct mca_btl_base_module_t* btl,
+    struct mca_btl_base_endpoint_t* ep,
+    struct ompi_convertor_t* convertor,
+    void* header,
+    size_t header_size,
+    size_t payload_size,
+    uint8_t order,
+    uint32_t flags,
+    mca_btl_base_tag_t tag,
+    mca_btl_base_descriptor_t** descriptor
+); 
+
 /**
  * PML->BTL Initiate a put of the specified size.
  *
@@ -586,6 +626,13 @@ extern void mca_btl_openib_frag_progress_pending_put_get(
 extern int mca_btl_openib_ft_event(int state);
 
 
+/**
+ * Show an error during init, particularly when running out of
+ * registered memory.
+ */
+void mca_btl_openib_show_init_error(const char *file, int line,
+                                    const char *func, const char *dev);
+
 #define BTL_OPENIB_HP_CQ 0
 #define BTL_OPENIB_LP_CQ 1
 
@@ -601,6 +648,18 @@ extern int mca_btl_openib_ft_event(int state);
 
 int mca_btl_openib_post_srr(mca_btl_openib_module_t* openib_btl, const int qp);
 
+/**
+ * Get a transport name of btl by its transport type.
+ */
+
+const char* btl_openib_get_transport_name(mca_btl_openib_transport_type_t transport_type);
+
+/**
+ * Get a transport type of btl.
+ */
+
+mca_btl_openib_transport_type_t mca_btl_openib_get_transport_type(mca_btl_openib_module_t* openib_btl);
+
 static inline int qp_cq_prio(const int qp)
 {
     if(0 == qp)
