@@ -241,9 +241,10 @@ int btl_openib_register_mca_params(void)
                   REGINT_GE_ONE));
 
     CHECK(reg_int("cq_size", "ib_cq_size",
-                  "Size of the OpenFabrics completion "
-                  "queue (will automatically be set to a minimum of "
-                  "(2 * number_of_peers * btl_openib_rd_num))",
+                  "Minimum size of the OpenFabrics completion queue "
+                  "(CQs are automatically sized based on the number "
+                  "of peer MPI processes; this value determines the "
+                  "*minimum* size of all CQs)",
                   1000, &ival, REGINT_GE_ONE));
     mca_btl_openib_component.ib_cq_size[BTL_OPENIB_LP_CQ] =
         mca_btl_openib_component.ib_cq_size[BTL_OPENIB_HP_CQ] = (uint32_t) ival;
