@@ -13,7 +13,7 @@
  * Copyright (c) 2006      Los Alamos National Security, LLC.  All rights
  *                         reserved. 
  * Copyright (c) 2008      Mellanox Technologies.  All rights reserved.
- * Copyright (c) 2009      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2009-2011 IBM Corporation.  All rights reserved.
  *
  * $COPYRIGHT$
  * 
@@ -259,15 +259,14 @@ static int reply_start_connect(mca_btl_openib_endpoint_t *endpoint,
 static int set_remote_info(mca_btl_base_endpoint_t* endpoint,
                            mca_btl_openib_rem_info_t* rem_info)
 {
+    /* Free up the memory pointed to by rem_qps before overwriting the pointer
+       in the following memcpy */
+    free(endpoint->rem_info.rem_qps);
+
     /* copy the rem_info stuff */
-    memcpy(&((mca_btl_openib_endpoint_t*) endpoint)->rem_info, 
-           rem_info, sizeof(mca_btl_openib_rem_info_t)); 
-    
-    /* copy over the rem qp info */
-    memcpy(endpoint->rem_info.rem_qps,
-           rem_info->rem_qps, sizeof(mca_btl_openib_rem_qp_info_t) * 
-           mca_btl_openib_component.num_qps);
-    
+    memcpy(&((mca_btl_openib_endpoint_t*) endpoint)->rem_info,
+           rem_info, sizeof(mca_btl_openib_rem_info_t));
+
     BTL_VERBOSE(("Setting QP info,  LID = %d", endpoint->rem_info.rem_lid));
     return ORTE_SUCCESS;
 
@@ -635,7 +634,12 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
     mca_btl_openib_rem_info_t rem_info;
     uint8_t message_type;
     bool master;
-    
+
+    /* We later memcpy this whole structure. Make sure
+       that all the parameters are initialized, especially
+       the pointers */
+    memset(&rem_info,0, sizeof(rem_info));
+
     /* start by unpacking data first so we know who is knocking at 
        our door */ 
     BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT8));
@@ -782,7 +786,7 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
                just ignore this connection request */
             if (found && !master &&
                 MCA_BTL_IB_CLOSED != ib_endpoint->endpoint_state) {
-		OPAL_THREAD_UNLOCK(&mca_btl_openib_component.ib_lock);
+                OPAL_THREAD_UNLOCK(&mca_btl_openib_component.ib_lock);
                 return;
             }
         }
@@ -790,7 +794,7 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
         if (!found) {
             BTL_ERROR(("can't find suitable endpoint for this peer\n")); 
             mca_btl_openib_endpoint_invoke_error(NULL);
-	    OPAL_THREAD_UNLOCK(&mca_btl_openib_component.ib_lock);
+            OPAL_THREAD_UNLOCK(&mca_btl_openib_component.ib_lock);
             return; 
         }
         
@@ -805,6 +809,7 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
                to CONNECTING, and then reply with our QP
                information */
             if (master) {
+                assert(rem_info.rem_qps != NULL);
                 rc = reply_start_connect(ib_endpoint, &rem_info);
             } else {
                 rc = oob_module_start_connect(ib_endpoint->endpoint_local_cpc, 
@@ -827,6 +832,7 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
             break;
              
         case MCA_BTL_IB_CONNECTING :
+            assert(rem_info.rem_qps != NULL);
             set_remote_info(ib_endpoint, &rem_info);
             if (OMPI_SUCCESS != (rc = qp_connect_all(ib_endpoint))) {
                 BTL_ERROR(("endpoint connect error: %d", rc)); 
