@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      UT-Battelle, LLC. All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -28,7 +29,6 @@
 #include "ompi/mca/btl/base/base.h"
 #include "ompi/class/ompi_bitmap.h"
 #include "ompi/class/ompi_free_list.h"
-#include "orte/class/orte_proc_table.h"
 
 #include "btl_portals_endpoint.h"
 #include "btl_portals_frag.h"
@@ -38,7 +38,7 @@
  */
 struct mca_btl_portals_component_t {
     /* base BTL component */
-    mca_btl_base_component_1_0_1_t super;
+    mca_btl_base_component_2_0_0_t super;
 
     /* output channel for debugging.  Value settings when using
      * output_verbose:
@@ -61,6 +61,13 @@ struct mca_btl_portals_component_t {
 
     /* number of eager fragments */
     int portals_free_list_eager_max_num;
+    
+    /* shall I use portals to send to thyself? */
+    int portals_support_self;
+
+    /* do I need a portals ACK? */
+    int portals_need_ack;
+
 };
 typedef struct mca_btl_portals_component_t mca_btl_portals_component_t;
 
@@ -73,9 +80,6 @@ struct mca_btl_portals_module_t {
     /* base BTL module interface */
     mca_btl_base_module_t super;
 
-    /* registered callbacks */
-    mca_btl_base_recv_reg_t portals_reg[MCA_BTL_TAG_MAX];
-
     /* number of processes we're actively connected to.  Needed to
        know when to do activation / shutdown */
     int32_t portals_num_procs;
@@ -165,14 +169,12 @@ int mca_btl_portals_del_procs(struct mca_btl_base_module_t* btl_base,
                               struct ompi_proc_t **procs,
                               struct mca_btl_base_endpoint_t** peers);
 
-int mca_btl_portals_register(struct mca_btl_base_module_t* btl_base,
-                             mca_btl_base_tag_t tag,
-                             mca_btl_base_module_recv_cb_fn_t cbfunc,
-                             void* cbdata);
-
 mca_btl_base_descriptor_t* 
 mca_btl_portals_alloc(struct mca_btl_base_module_t* btl_base, 
-                      size_t size); 
+                      struct mca_btl_base_endpoint_t* endpoint,
+                      uint8_t order,
+                      size_t size,
+                      uint32_t flags); 
 
 int mca_btl_portals_free(struct mca_btl_base_module_t* btl_base, 
                          mca_btl_base_descriptor_t* des); 
@@ -182,22 +184,38 @@ mca_btl_portals_prepare_src(struct mca_btl_base_module_t* btl_base,
                             struct mca_btl_base_endpoint_t* peer,
                             mca_mpool_base_registration_t* registration, 
                             struct ompi_convertor_t* convertor,
+                            uint8_t order,
                             size_t reserve,
-                            size_t* size);
+                            size_t* size,
+                            uint32_t flags);
 
 mca_btl_base_descriptor_t* 
 mca_btl_portals_prepare_dst(struct mca_btl_base_module_t* btl_base, 
                             struct mca_btl_base_endpoint_t* peer,
                             mca_mpool_base_registration_t* registration, 
                             struct ompi_convertor_t* convertor,
+                            uint8_t order,
                             size_t reserve,
-                            size_t* size); 
+                            size_t* size,
+                            uint32_t flags); 
 
 int mca_btl_portals_send(struct mca_btl_base_module_t* btl_base,
                          struct mca_btl_base_endpoint_t* btl_peer,
                          struct mca_btl_base_descriptor_t* descriptor, 
                          mca_btl_base_tag_t tag);
 
+
+int mca_btl_portals_sendi(struct mca_btl_base_module_t* btl_base,
+                          struct mca_btl_base_endpoint_t* endpoint,
+                          struct ompi_convertor_t* convertor,
+                          void* header,
+                          size_t header_size,
+                          size_t payload_size,
+                          uint8_t order,
+                          uint32_t flags,
+                          mca_btl_base_tag_t tag, 
+                          mca_btl_base_descriptor_t** des);
+
 int mca_btl_portals_put(struct mca_btl_base_module_t* btl_base,
                         struct mca_btl_base_endpoint_t* btl_peer,
                         struct mca_btl_base_descriptor_t* decriptor);
