@@ -5,6 +5,30 @@
 #include "btl_sm_endpoint.h"
 
 
+static void
+add_pending(struct mca_btl_base_endpoint_t *ep, void *data, bool resend)
+{
+    int rc;
+    mca_btl_sm_pending_send_item_t *si;
+    opal_free_list_item_t *i;
+    OPAL_FREE_LIST_GET(&mca_btl_sm_component.pending_send_fl, i, rc);
+
+    /* don't handle error for now */
+    assert(i != NULL && rc == OMPI_SUCCESS);
+
+    si = (mca_btl_sm_pending_send_item_t*)i;
+    si->data = data;
+
+    OPAL_THREAD_ADD32(&mca_btl_sm_component.num_pending_sends, +1);
+
+    /* if data was on pending send list then prepend it to the list to
+     * minimize reordering */
+    if(resend)
+        opal_list_prepend(&ep->pending_sends, (opal_list_item_t*)si);
+    else
+        opal_list_append(&ep->pending_sends, (opal_list_item_t*)si);
+}
+
 /*
  * FIFO_MAP(x) defines which FIFO on the receiver should be used
  * by sender rank x.  The map is some many-to-one hash.
@@ -37,14 +61,20 @@
 
 
 #define MCA_BTL_SM_FIFO_WRITE(endpoint_peer, my_smp_rank,               \
-                              peer_smp_rank, hdr, resend, rc)           \
+                              peer_smp_rank, hdr, resend, retry_pending_sends, rc) \
 do {                                                                    \
-    sm_fifo_t* fifo = &(mca_btl_sm_component.fifo[peer_smp_rank][FIFO_MAP(my_smp_rank)]); \
+    mca_btl_sm_fifo_t* fifo = &(mca_btl_sm_component.fifo[peer_smp_rank][FIFO_MAP(my_smp_rank)]); \
+                                                                        \
+    if ( retry_pending_sends ) {                                        \
+        if ( 0 < opal_list_get_size(&endpoint_peer->pending_sends) ) {  \
+            mca_btl_sm_process_pending_sends(endpoint_peer);            \
+        }                                                               \
+    }                                                                   \
                                                                         \
     opal_atomic_lock(&(fifo->head_lock));                               \
     /* post fragment */                                                 \
     if(sm_fifo_write(hdr, fifo) != OMPI_SUCCESS) {                      \
-        btl_sm_add_pending(endpoint_peer, hdr, resend);                 \
+        add_pending(endpoint_peer, hdr, resend);                        \
         rc = OMPI_ERR_RESOURCE_BUSY;                                    \
     } else {                                                            \
         MCA_BTL_SM_SIGNAL_PEER(endpoint_peer);                          \
