@@ -44,6 +44,7 @@
 #  include <netdb.h>
 #  endif
 #endif
+#include <limits.h>
 
 #include "ompi/constants.h"
 #include "opal/event/event.h"
@@ -168,6 +169,7 @@ static void mca_btl_tcp_component_accept_handler(int, short, void*);
 
 int mca_btl_tcp_component_open(void)
 {
+    char* message;
 #ifdef __WINDOWS__
     WSADATA win_sock_data;
     if( WSAStartup(MAKEWORD(2,2), &win_sock_data) != 0 ) {
@@ -221,23 +223,41 @@ int mca_btl_tcp_component_open(void)
         !mca_btl_tcp_param_register_int ("use_nagle", "Whether to use Nagle's algorithm or not (using Nagle's algorithm may increase short message latency)", 0);
     mca_btl_tcp_component.tcp_port_min =
         mca_btl_tcp_param_register_int( "port_min_v4",
-            "The minimum port where the TCP BTL will try to bind (default 0)", 0 );
+            "The minimum port where the TCP BTL will try to bind (default 1024)", 1024 );
+    if( mca_btl_tcp_component.tcp_port_min > USHRT_MAX ) {
+        orte_show_help("help-mpi-btl-tcp.txt", "invalid minimum port",
+                       true, "v4", orte_process_info.nodename,
+                       mca_btl_tcp_component.tcp_port_min );
+        mca_btl_tcp_component.tcp_port_min = 1024;
+    }
+    asprintf( &message, 
+              "The number of ports where the TCP BTL will try to bind (default %d)."
+              " This parameter together with the port min, define a range of ports"
+              " where Open MPI will open sockets.",
+              (0x1 << 16) - mca_btl_tcp_component.tcp_port_min - 1 );
     mca_btl_tcp_component.tcp_port_range =
-        mca_btl_tcp_param_register_int( "port_range_v4",
-            "The number of ports where the TCP BTL will try to bind (default 64K)."
-            " This parameter together with the port min, define a range of ports"
-            " where Open MPI will open sockets.",
-            64*1024 - mca_btl_tcp_component.tcp_port_min - 1);
+        mca_btl_tcp_param_register_int( "port_range_v4", message,
+                                        (0x1 << 16) - mca_btl_tcp_component.tcp_port_min - 1);
+    free(message);
 #if OPAL_WANT_IPV6
     mca_btl_tcp_component.tcp6_port_min =
         mca_btl_tcp_param_register_int( "port_min_v6",
-            "The minimum port where the TCP BTL will try to bind (default 0)", 0 );
+            "The minimum port where the TCP BTL will try to bind (default 1024)", 1024 );
+    if( mca_btl_tcp_component.tcp6_port_min > USHRT_MAX ) {
+        orte_show_help("help-mpi-btl-tcp.txt", "invalid minimum port",
+                       true, "v6", orte_process_info.nodename,
+                       mca_btl_tcp_component.tcp6_port_min );
+        mca_btl_tcp_component.tcp6_port_min = 1024;
+    }
+    asprintf( &message, 
+              "The number of ports where the TCP BTL will try to bind (default %d)."
+              " This parameter together with the port min, define a range of ports"
+              " where Open MPI will open sockets.",
+              (0x1 << 16) - mca_btl_tcp_component.tcp6_port_min - 1 );
     mca_btl_tcp_component.tcp6_port_range =
-        mca_btl_tcp_param_register_int( "port_range_v6",
-            "The number of ports where the TCP BTL will try to bind (default 64K)."
-            " This parameter together with the port min, define a range of ports"
-            " where Open MPI will open sockets.",
-            64*1024 - mca_btl_tcp_component.tcp6_port_min - 1);
+        mca_btl_tcp_param_register_int( "port_range_v6", message,
+                                        (0x1 << 16) - mca_btl_tcp_component.tcp6_port_min - 1);
+    free(message);
 #endif
     mca_btl_tcp_module.super.btl_exclusivity =  MCA_BTL_EXCLUSIVITY_LOW + 100;
     mca_btl_tcp_module.super.btl_eager_limit = 64*1024;
@@ -541,12 +561,12 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
         addrlen = res->ai_addrlen;
         freeaddrinfo (res);
 
-#ifdef IPV6_V6ONLY
+#if defined(IPV6_V6ONLY) && !defined(__OpenBSD__)
         /* in case of AF_INET6, disable v4-mapped addresses */
         if (AF_INET6 == af_family) {
-            int flg = 0;
+            int flg = 1;
             if (setsockopt (sd, IPPROTO_IPV6, IPV6_V6ONLY,
-                            &flg, sizeof (flg)) < 0) {
+                            (char *) &flg, sizeof (flg)) < 0) {
                 opal_output(0,
                     "mca_btl_tcp_create_listen: unable to disable v4-mapped addresses\n");
             }
@@ -561,7 +581,7 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
 
     {  /* Don't reuse ports */
         int flg = 0;
-        if (setsockopt (sd, SOL_SOCKET, SO_REUSEADDR, (void*)&flg, sizeof (flg)) < 0) {
+        if (setsockopt (sd, SOL_SOCKET, SO_REUSEADDR, (const char *)&flg, sizeof (flg)) < 0) {
             BTL_ERROR((0, "mca_btl_tcp_create_listen: unable to unset the "
                            "SO_REUSEADDR option (%s:%d)\n",
                            strerror(opal_socket_errno), opal_socket_errno));
@@ -584,9 +604,9 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
 
         for( index = 0;  index < range; index++ ) {
 #if OPAL_WANT_IPV6
-            ((struct sockaddr_in6*) &inaddr)->sin6_port = port + index;
+            ((struct sockaddr_in6*) &inaddr)->sin6_port = htons(port + index);
 #else
-            ((struct sockaddr_in*) &inaddr)->sin_port = port + index;
+            ((struct sockaddr_in*) &inaddr)->sin_port = htons(port + index);
 #endif  /* OPAL_WANT_IPV6 */
             if(bind(sd, (struct sockaddr*)&inaddr, addrlen) < 0) {
                 if( (EADDRINUSE == opal_socket_errno) || (EADDRNOTAVAIL == opal_socket_errno) ) {
