@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -40,8 +40,7 @@ typedef enum {
     MCA_BTL_TCP_CONNECT_ACK,
     MCA_BTL_TCP_CLOSED,
     MCA_BTL_TCP_FAILED,
-    MCA_BTL_TCP_CONNECTED,
-    MCA_BTL_TCP_SHUTDOWN
+    MCA_BTL_TCP_CONNECTED
 } mca_btl_tcp_state_t;
 
 /**
@@ -81,7 +80,7 @@ OBJ_CLASS_DECLARATION(mca_btl_tcp_endpoint_t);
 void mca_btl_tcp_set_socket_options(int sd);
 void mca_btl_tcp_endpoint_close(mca_btl_base_endpoint_t*);
 int  mca_btl_tcp_endpoint_send(mca_btl_base_endpoint_t*, struct mca_btl_tcp_frag_t*);
-bool mca_btl_tcp_endpoint_accept(mca_btl_base_endpoint_t*, struct sockaddr_in*, int);
+bool mca_btl_tcp_endpoint_accept(mca_btl_base_endpoint_t*, struct sockaddr*, int);
 void mca_btl_tcp_endpoint_shutdown(mca_btl_base_endpoint_t*);
 
 #if defined(c_plusplus) || defined(__cplusplus)
