@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -29,6 +30,9 @@
 #ifdef HAVE_SYS_UIO_H
 #include <sys/uio.h>
 #endif
+#ifdef HAVE_NET_UIO_H
+#include <net/uio.h>
+#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif  /* HAVE_UNISTD_H */
@@ -115,7 +119,9 @@ bool mca_btl_tcp_frag_send(mca_btl_tcp_frag_t* frag, int sd)
                 mca_btl_tcp_endpoint_close(frag->endpoint);
                 return false;
             default:
-                BTL_ERROR(("mca_btl_tcp_frag_send: writev failed with errno=%d", opal_socket_errno));
+                BTL_ERROR(("mca_btl_tcp_frag_send: writev failed: %s (%d)", 
+                           strerror(opal_socket_errno),
+                           opal_socket_errno));
                 mca_btl_tcp_endpoint_close(frag->endpoint);
                 return false;
             }
@@ -142,7 +148,7 @@ bool mca_btl_tcp_frag_send(mca_btl_tcp_frag_t* frag, int sd)
 
 bool mca_btl_tcp_frag_recv(mca_btl_tcp_frag_t* frag, int sd)
 {
-    int cnt;
+    int cnt, dont_copy_data = 0;
     size_t i, num_vecs;
     mca_btl_base_endpoint_t* btl_endpoint = frag->endpoint;
 
@@ -152,29 +158,35 @@ bool mca_btl_tcp_frag_recv(mca_btl_tcp_frag_t* frag, int sd)
     if( 0 != btl_endpoint->endpoint_cache_length ) {
         size_t length;
         /* It's strange at the first look but cnt have to be set to the full amount of data
-	 * available. After going to advance_iov_position we will use cnt to detect if there
-	 * is still some data pending.
+         * available. After going to advance_iov_position we will use cnt to detect if there
+         * is still some data pending.
          */
         cnt = length = btl_endpoint->endpoint_cache_length;
         for( i = 0; i < frag->iov_cnt; i++ ) {
             if( length > frag->iov_ptr[i].iov_len )
-                length = frag->iov_ptr[0].iov_len;
-            memcpy( frag->iov_ptr[i].iov_base, btl_endpoint->endpoint_cache_pos, length );
+                length = frag->iov_ptr[i].iov_len;
+            if( (0 == dont_copy_data) || (length < frag->iov_ptr[i].iov_len) ) {
+                memcpy( frag->iov_ptr[i].iov_base, btl_endpoint->endpoint_cache_pos, length );
+            } else {
+                frag->segments[0].seg_addr.pval = btl_endpoint->endpoint_cache_pos;
+                frag->iov_ptr[i].iov_base = btl_endpoint->endpoint_cache_pos;
+            }
             btl_endpoint->endpoint_cache_pos += length;
             btl_endpoint->endpoint_cache_length -= length;
             length = btl_endpoint->endpoint_cache_length;
             if( 0 == length ) {
-		btl_endpoint->endpoint_cache_pos = btl_endpoint->endpoint_cache;
-	        break;
-	    }
+                btl_endpoint->endpoint_cache_pos = btl_endpoint->endpoint_cache;
+                break;
+            }
         }
         goto advance_iov_position;
     }
     /* What's happens if all iovecs are used by the fragment ? It still work, as we reserve one
      * iovec for the caching in the fragment structure (the +1).
      */
-    frag->iov_ptr[num_vecs].iov_base = btl_endpoint->endpoint_cache;
-    frag->iov_ptr[num_vecs].iov_len  = mca_btl_tcp_component.tcp_endpoint_cache;
+    frag->iov_ptr[num_vecs].iov_base = btl_endpoint->endpoint_cache_pos;
+    frag->iov_ptr[num_vecs].iov_len  = 
+        mca_btl_tcp_component.tcp_endpoint_cache - btl_endpoint->endpoint_cache_length;
     num_vecs++;
 #endif  /* MCA_BTL_TCP_ENDPOINT_CACHE */
 
@@ -199,14 +211,16 @@ bool mca_btl_tcp_frag_recv(mca_btl_tcp_frag_t* frag, int sd)
 	    mca_btl_tcp_endpoint_close(btl_endpoint);
 	    return false;
 	default:
-            BTL_ERROR(("mca_btl_tcp_frag_recv: readv failed with errno=%d", opal_socket_errno));
+            BTL_ERROR(("mca_btl_tcp_frag_recv: readv failed: %s (%d)", 
+                       strerror(opal_socket_errno),
+                       opal_socket_errno));
 	    mca_btl_tcp_endpoint_close(btl_endpoint);
 	    return false;
 	}
     };
 
  advance_iov_position:
-    /* if the write didn't complete - update the iovec state */
+    /* if the read didn't complete - update the iovec state */
     num_vecs = frag->iov_cnt;
     for( i = 0; i < num_vecs; i++ ) {
         if( cnt < (int)frag->iov_ptr->iov_len ) {
@@ -227,15 +241,22 @@ bool mca_btl_tcp_frag_recv(mca_btl_tcp_frag_t* frag, int sd)
 
     /* read header */
     if(frag->iov_cnt == 0) {
-        if (btl_endpoint->endpoint_nbo) MCA_BTL_TCP_HDR_NTOH(frag->hdr);
+        if (btl_endpoint->endpoint_nbo && frag->iov_idx == 1) MCA_BTL_TCP_HDR_NTOH(frag->hdr);
         switch(frag->hdr.type) {
         case MCA_BTL_TCP_HDR_TYPE_SEND:
             if(frag->iov_idx == 1 && frag->hdr.size) {
-                frag->iov[1].iov_base = (IOVBASE_TYPE*)(frag+1);
-                frag->iov[1].iov_len = frag->hdr.size;
                 frag->segments[0].seg_addr.pval = frag+1;
                 frag->segments[0].seg_len = frag->hdr.size;
+                frag->iov[1].iov_base = (IOVBASE_TYPE*)(frag->segments[0].seg_addr.pval);
+                frag->iov[1].iov_len = frag->hdr.size;
                 frag->iov_cnt++;
+#ifndef __sparc
+                /* The following cannot be done for sparc code 
+                 * because it causes alignment errors when accessing
+                 * structures later on in the btl and pml code.
+                 */
+                dont_copy_data = 1;
+#endif
                 goto repeat;
             }
             break;
@@ -246,11 +267,11 @@ bool mca_btl_tcp_frag_recv(mca_btl_tcp_frag_t* frag, int sd)
                 frag->iov_cnt++;
                 goto repeat;
             } else if (frag->iov_idx == 2) {
-                for(i=0; i<frag->hdr.count; i++) {
+                for( i = 0; i < frag->hdr.count; i++ ) {
                     frag->iov[i+2].iov_base = (IOVBASE_TYPE*)ompi_ptr_ltop(frag->segments[i].seg_addr.lval);
                     frag->iov[i+2].iov_len = frag->segments[i].seg_len;
-                    frag->iov_cnt++;
                 }
+                frag->iov_cnt += frag->hdr.count;
                 goto repeat;
             }
             break;
