@@ -39,7 +39,8 @@ mca_coll_basic_allgatherv_intra(void *sbuf, int scount,
                                 struct ompi_datatype_t *sdtype,
                                 void *rbuf, int *rcounts, int *disps,
                                 struct ompi_datatype_t *rdtype,
-                                struct ompi_communicator_t *comm)
+                                struct ompi_communicator_t *comm,
+                                mca_coll_base_module_t *module)
 {
     int i, size, rank ;
     int err;
@@ -70,8 +71,8 @@ mca_coll_basic_allgatherv_intra(void *sbuf, int scount,
     err = comm->c_coll.coll_gatherv(send_buf,
                                     rcounts[rank], send_type,rbuf,
                                     rcounts, disps, rdtype, 0,
-                                    comm);
-    
+                                    comm, comm->c_coll.coll_gatherv_module);
+
     if (MPI_SUCCESS != err) {
         return err;
     }
@@ -98,7 +99,8 @@ mca_coll_basic_allgatherv_intra(void *sbuf, int scount,
        return err;
     }
 
-    comm->c_coll.coll_bcast( rbuf, 1 ,newtype,0,comm);
+    comm->c_coll.coll_bcast( rbuf, 1 ,newtype,0,comm,
+            comm->c_coll.coll_bcast_module);
 
     ompi_ddt_destroy (&newtype);
 
@@ -118,15 +120,13 @@ mca_coll_basic_allgatherv_inter(void *sbuf, int scount,
                                 struct ompi_datatype_t *sdtype,
                                 void *rbuf, int *rcounts, int *disps,
                                 struct ompi_datatype_t *rdtype,
-                                struct ompi_communicator_t *comm)
+                                struct ompi_communicator_t *comm,
+                                mca_coll_base_module_t *module)
 {
-    int size, rsize;
-    int err, i;
-    int *scounts = NULL;
-    int *sdisps = NULL;
+    int rsize, err, i;
+    int *scounts = NULL, *sdisps = NULL;
 
     rsize = ompi_comm_remote_size(comm);
-    size = ompi_comm_size(comm);
 
     scounts = (int *) malloc(rsize * sizeof(int));
     sdisps = (int *) calloc(rsize, sizeof(int));
@@ -139,7 +139,8 @@ mca_coll_basic_allgatherv_inter(void *sbuf, int scount,
     }
 
     err = comm->c_coll.coll_alltoallv(sbuf, scounts, sdisps, sdtype,
-                                      rbuf, rcounts, disps, rdtype, comm);
+                                      rbuf, rcounts, disps, rdtype, comm,
+                                      comm->c_coll.coll_alltoallv_module);
 
     if (NULL != sdisps) {
         free(sdisps);
