@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -38,13 +38,11 @@ int
 mca_coll_basic_reduce_lin_intra(void *sbuf, void *rbuf, int count,
                                 struct ompi_datatype_t *dtype,
                                 struct ompi_op_t *op,
-                                int root, struct ompi_communicator_t *comm)
+                                int root, struct ompi_communicator_t *comm,
+                                mca_coll_base_module_t *module)
 {
-    int i;
-    int rank;
-    int err;
-    int size;
-    long true_lb, true_extent, lb, extent;
+    int i, rank, err, size;
+    ptrdiff_t true_lb, true_extent, lb, extent;
     char *free_buffer = NULL;
     char *pml_buffer = NULL;
     char *inplace_temp = NULL;
@@ -194,7 +192,7 @@ mca_coll_basic_reduce_lin_intra(void *sbuf, void *rbuf, int count,
 
     if (MPI_IN_PLACE == sbuf) {
         sbuf = rbuf;
-        inplace_temp = malloc(true_extent + (count - 1) * extent);
+        inplace_temp = (char*)malloc(true_extent + (count - 1) * extent);
         if (NULL == inplace_temp) {
             return OMPI_ERR_OUT_OF_RESOURCE;
         }
@@ -202,7 +200,7 @@ mca_coll_basic_reduce_lin_intra(void *sbuf, void *rbuf, int count,
     }
 
     if (size > 1) {
-        free_buffer = malloc(true_extent + (count - 1) * extent);
+        free_buffer = (char*)malloc(true_extent + (count - 1) * extent);
         if (NULL == free_buffer) {
             return OMPI_ERR_OUT_OF_RESOURCE;
         }
@@ -212,7 +210,7 @@ mca_coll_basic_reduce_lin_intra(void *sbuf, void *rbuf, int count,
     /* Initialize the receive buffer. */
 
     if (rank == (size - 1)) {
-        err = ompi_ddt_copy_content_same_ddt(dtype, count, rbuf, sbuf);
+        err = ompi_ddt_copy_content_same_ddt(dtype, count, (char*)rbuf, (char*)sbuf);
     } else {
         err = MCA_PML_CALL(recv(rbuf, count, dtype, size - 1,
                                 MCA_COLL_BASE_TAG_REDUCE, comm,
@@ -229,7 +227,7 @@ mca_coll_basic_reduce_lin_intra(void *sbuf, void *rbuf, int count,
 
     for (i = size - 2; i >= 0; --i) {
         if (rank == i) {
-            inbuf = sbuf;
+            inbuf = (char*)sbuf;
         } else {
             err = MCA_PML_CALL(recv(pml_buffer, count, dtype, i,
                                     MCA_COLL_BASE_TAG_REDUCE, comm,
@@ -250,7 +248,7 @@ mca_coll_basic_reduce_lin_intra(void *sbuf, void *rbuf, int count,
     }
 
     if (NULL != inplace_temp) {
-        err = ompi_ddt_copy_content_same_ddt(dtype, count, sbuf, inplace_temp);
+        err = ompi_ddt_copy_content_same_ddt(dtype, count, (char*)sbuf, inplace_temp);
         free(inplace_temp);
     }
     if (NULL != free_buffer) {
@@ -269,27 +267,67 @@ mca_coll_basic_reduce_lin_intra(void *sbuf, void *rbuf, int count,
  *	Function:	- reduction using O(log N) algorithm
  *	Accepts:	- same as MPI_Reduce()
  *	Returns:	- MPI_SUCCESS or error code
+ *
+ * 
+ *      Performing reduction on each dimension of the hypercube.
+ *	An example for 8 procs (dimensions = 3):
+ *
+ *      Stage 1, reduce on X dimension,  1 -> 0, 3 -> 2, 5 -> 4, 7 -> 6
+ *
+ *          6----<---7		proc_0: 0+1
+ *         /|       /|		proc_1: 1
+ *        / |      / |		proc_2: 2+3
+ *       /  |     /  |		proc_3: 3
+ *      4----<---5   |		proc_4: 4+5
+ *      |   2--< |---3		proc_5: 5
+ *      |  /     |  /		proc_6: 6+7
+ *      | /      | /		proc_7: 7
+ *      |/       |/
+ *      0----<---1
+ *
+ *	Stage 2, reduce on Y dimension, 2 -> 0, 6 -> 4
+ *
+ *          6--------7		proc_0: 0+1+2+3
+ *         /|       /|		proc_1: 1
+ *        v |      / |		proc_2: 2+3
+ *       /  |     /  |		proc_3: 3
+ *      4--------5   |		proc_4: 4+5+6+7
+ *      |   2--- |---3		proc_5: 5
+ *      |  /     |  /		proc_6: 6+7
+ *      | v      | /		proc_7: 7
+ *      |/       |/
+ *      0--------1
+ *
+ *	Stage 3, reduce on Z dimension, 4 -> 0
+ *
+ *          6--------7		proc_0: 0+1+2+3+4+5+6+7
+ *         /|       /|		proc_1: 1
+ *        / |      / |		proc_2: 2+3
+ *       /  |     /  |		proc_3: 3
+ *      4--------5   |		proc_4: 4+5+6+7
+ *      |   2--- |---3		proc_5: 5
+ *      v  /     |  /		proc_6: 6+7
+ *      | /      | /		proc_7: 7
+ *      |/       |/
+ *      0--------1
+ *
+ *
  */
 int
 mca_coll_basic_reduce_log_intra(void *sbuf, void *rbuf, int count,
                                 struct ompi_datatype_t *dtype,
                                 struct ompi_op_t *op,
-                                int root, struct ompi_communicator_t *comm)
+                                int root, struct ompi_communicator_t *comm,
+                                mca_coll_base_module_t *module)
 {
-    int i;
-    int size;
-    int rank;
-    int vrank;
-    int err;
-    int peer;
-    int dim;
-    int mask;
-    long true_lb, true_extent, lb, extent;
+    int i, size, rank, vrank;
+    int err, peer, dim, mask;
+    ptrdiff_t true_lb, true_extent, lb, extent;
     char *free_buffer = NULL;
     char *free_rbuf = NULL;
     char *pml_buffer = NULL;
     char *snd_buffer = NULL;
-    char *rcv_buffer = rbuf;
+    char *rcv_buffer = (char*)rbuf;
     char *inplace_temp = NULL;
 
     /* JMS Codearound for now -- if the operations is not communative,
@@ -299,7 +337,7 @@ mca_coll_basic_reduce_log_intra(void *sbuf, void *rbuf, int count,
 
     if (!ompi_op_is_commute(op)) {
         return mca_coll_basic_reduce_lin_intra(sbuf, rbuf, count, dtype,
-                                               op, root, comm);
+                                               op, root, comm, module);
     }
 
     /* Some variables */
@@ -316,7 +354,7 @@ mca_coll_basic_reduce_log_intra(void *sbuf, void *rbuf, int count,
     ompi_ddt_get_extent(dtype, &lb, &extent);
     ompi_ddt_get_true_extent(dtype, &true_lb, &true_extent);
     
-    free_buffer = malloc(true_extent + (count - 1) * extent);
+    free_buffer = (char*)malloc(true_extent + (count - 1) * extent);
     if (NULL == free_buffer) {
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
@@ -332,20 +370,20 @@ mca_coll_basic_reduce_log_intra(void *sbuf, void *rbuf, int count,
      * rationale above. */
 
     if (MPI_IN_PLACE == sbuf) {
-        inplace_temp = malloc(true_extent + (count - 1) * extent);
+        inplace_temp = (char*)malloc(true_extent + (count - 1) * extent);
         if (NULL == inplace_temp) {
             return OMPI_ERR_OUT_OF_RESOURCE;
         }
         sbuf = inplace_temp - lb;
-	err = ompi_ddt_copy_content_same_ddt(dtype, count, sbuf, rbuf);
+        err = ompi_ddt_copy_content_same_ddt(dtype, count, (char*)sbuf, (char*)rbuf);
     }
-    snd_buffer = sbuf;
+    snd_buffer = (char*)sbuf;
 
     if (rank != root && 0 == (vrank & 1)) {
         /* root is the only one required to provide a valid rbuf.
          * Assume rbuf is invalid for all other ranks, so fix it up
          * here to be valid on all non-leaf ranks */
-        free_rbuf = malloc(true_extent + (count - 1) * extent);
+        free_rbuf = (char*)malloc(true_extent + (count - 1) * extent);
         if (NULL == free_rbuf) {
             free(free_buffer);
             return OMPI_ERR_OUT_OF_RESOURCE;
@@ -377,7 +415,7 @@ mca_coll_basic_reduce_log_intra(void *sbuf, void *rbuf, int count,
                 }
                 return err;
             }
-            snd_buffer = rbuf;
+            snd_buffer = (char*)rbuf;
             break;
         }
 
@@ -428,7 +466,7 @@ mca_coll_basic_reduce_log_intra(void *sbuf, void *rbuf, int count,
                  * what we just received against it. */
                 if (!ompi_op_is_commute(op)) {
                     ompi_ddt_copy_content_same_ddt(dtype, count, pml_buffer,
-                                                   sbuf);
+                                                   (char*)sbuf);
                     ompi_op_reduce(op, rbuf, pml_buffer, count, dtype);
                 } else {
                     ompi_op_reduce(op, sbuf, pml_buffer, count, dtype);
@@ -437,7 +475,7 @@ mca_coll_basic_reduce_log_intra(void *sbuf, void *rbuf, int count,
                 snd_buffer = pml_buffer;
                 /* starting from now we always receive in the user
                  * provided buffer */
-                rcv_buffer = rbuf;
+                rcv_buffer = (char*)rbuf;
             }
         }
     }
@@ -446,7 +484,7 @@ mca_coll_basic_reduce_log_intra(void *sbuf, void *rbuf, int count,
     err = MPI_SUCCESS;
     if (0 == vrank) {
         if (root == rank) {
-            ompi_ddt_copy_content_same_ddt(dtype, count, rbuf, snd_buffer);
+            ompi_ddt_copy_content_same_ddt(dtype, count, (char*)rbuf, snd_buffer);
         } else {
             err = MCA_PML_CALL(send(snd_buffer, count,
                                     dtype, root, MCA_COLL_BASE_TAG_REDUCE,
@@ -488,18 +526,15 @@ int
 mca_coll_basic_reduce_lin_inter(void *sbuf, void *rbuf, int count,
                                 struct ompi_datatype_t *dtype,
                                 struct ompi_op_t *op,
-                                int root, struct ompi_communicator_t *comm)
+                                int root, struct ompi_communicator_t *comm,
+                                mca_coll_base_module_t *module)
 {
-    int i;
-    int rank;
-    int err;
-    int size;
-    long true_lb, true_extent, lb, extent;
+    int i, err, size;
+    ptrdiff_t true_lb, true_extent, lb, extent;
     char *free_buffer = NULL;
     char *pml_buffer = NULL;
 
     /* Initialize */
-    rank = ompi_comm_rank(comm);
     size = ompi_comm_remote_size(comm);
 
     if (MPI_PROC_NULL == root) {
@@ -515,7 +550,7 @@ mca_coll_basic_reduce_lin_inter(void *sbuf, void *rbuf, int count,
         ompi_ddt_get_extent(dtype, &lb, &extent);
         ompi_ddt_get_true_extent(dtype, &true_lb, &true_extent);
 
-        free_buffer = malloc(true_extent + (count - 1) * extent);
+        free_buffer = (char*)malloc(true_extent + (count - 1) * extent);
         if (NULL == free_buffer) {
             return OMPI_ERR_OUT_OF_RESOURCE;
         }
@@ -570,7 +605,8 @@ int
 mca_coll_basic_reduce_log_inter(void *sbuf, void *rbuf, int count,
                                 struct ompi_datatype_t *dtype,
                                 struct ompi_op_t *op,
-                                int root, struct ompi_communicator_t *comm)
+                                int root, struct ompi_communicator_t *comm,
+                                mca_coll_base_module_t *module)
 {
     return OMPI_ERR_NOT_IMPLEMENTED;
 }
