@@ -33,7 +33,8 @@
 int mca_coll_self_reduce_intra(void *sbuf, void *rbuf, int count,
                                struct ompi_datatype_t *dtype, 
                                struct ompi_op_t *op,
-                               int root, struct ompi_communicator_t *comm)
+                               int root, struct ompi_communicator_t *comm,
+                               mca_coll_base_module_t *module)
 {
     if (MPI_IN_PLACE == sbuf) {
         return MPI_SUCCESS;
