@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008-2009 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -27,98 +27,23 @@
 #include "opal/mca/mca.h"
 #include "orte/types.h"
 #include "ompi/mca/coll/coll.h"
-#include "ompi/mca/mpool/mpool.h"
 #include "ompi/mca/common/sm/common_sm_mmap.h"
 
 BEGIN_C_DECLS
 
-/*
- * Horrid debugging macro
- */
-#if 0
-#include <stdio.h>
-#define D(foo) { printf foo ; fflush(stdout); }
-#else
-#define D(foo)
-#endif
-
-#ifdef HAVE_SCHED_YIELD
-#  include <sched.h>
-#  define SPIN sched_yield()
-#elif defined(__WINDOWS__)
-#  define SPIN SwitchToThread()
-#else  /* no switch available */
-#  define SPIN
-#endif
-
-    /**
-     * Structure used within bootstrap mmap file for setting up a coll
-     * sm component on a communicator
-     */
-    struct mca_coll_sm_bootstrap_comm_setup_t {
-        /** Offset to be used in the data mpool for this comm's
-            collective sm operations -- use this value plus the base
-            of the mpool to obtain the pointer to this comm's
-            mca_coll_sm_mpool_area_t */
-        volatile size_t smbcs_data_mpool_offset;
-
-        /** Number of processes in this communicator who have seen
-            this value already. */
-        volatile int smbcs_count;
-
-        /** Mechanism for letting multiple processes know whether this
-            allocation succeeded or not */
-        volatile bool smbcs_success;
-    };
-    /**
-     * Convenience typedef
-     */
-    typedef struct mca_coll_sm_bootstrap_comm_setup_t 
-        mca_coll_sm_bootstrap_comm_setup_t;
-
-    /**
-     * Structure that acts as a key for the bootstrap area -- a
-     * segment in the bootstrap mpool is uniquely identified by its
-     * CID and the process name of rank 0 in the group.
-     */
-    struct mca_coll_sm_bootstrap_comm_key_t {
-        /** CID of the communicator using a bootstrap segment */
-        uint32_t mcsbck_cid;
-        /** Process name of rank 0 in this communicator */
-        orte_process_name_t mcsbck_rank0_name;
-    };
-    /**
-     * Convenience typedef
-     */
-    typedef struct mca_coll_sm_bootstrap_comm_key_t
-        mca_coll_sm_bootstrap_comm_key_t;
-
-    /**
-     * Extension to the control structure in the bootstrap mmap file
-     */
-    struct mca_coll_sm_bootstrap_header_extension_t {
-        /** upper-level control structure */
-        mca_common_sm_file_header_t super;
-
-        /** Pointer to the start of the segments in the bootstrap area
-            (map->seg_data only points to just beyond the
-            mca_common_sm_file_header_t) */
-        mca_coll_sm_bootstrap_comm_setup_t *smbhe_segments;
-
-        /** Pointer to array containing
-            component.sm_bootstrap_num_segments (CID, rank 0 process
-            name) tuples for use in bootstrap phase -- will always
-            point immediately after the end of this struct (i.e.,
-            still within this header, but since it's variable size
-            (set by MCA param), we can't just have it here in the
-            struct.  Bonk). */
-        mca_coll_sm_bootstrap_comm_key_t *smbhe_keys;
-    };
-    /**
-     * Convenience typedef
-     */
-    typedef struct mca_coll_sm_bootstrap_header_extension_t
-        mca_coll_sm_bootstrap_header_extension_t;
+/* Attempt to give some sort of progress / fairness if we're blocked
+   in an sm collective for a long time: call opal_progress once in a
+   great while.  Use a "goto" label for expdiency to exit loops. */
+#define SPIN_CONDITION_MAX 100000
+#define SPIN_CONDITION(cond, exit_label) \
+  do { int i; \
+       if (cond) goto exit_label; \
+       for (i = 0; i < SPIN_CONDITION_MAX; ++i) { \
+           if (cond) { goto exit_label; } \
+       } \
+       opal_progress(); \
+  } while (1); \
+  exit_label: 
 
     /** 
      * Structure to hold the sm coll component.  First it holds the
@@ -126,7 +51,7 @@ BEGIN_C_DECLS
      * sm-coll-component-specific stuff (e.g., current MCA param
      * values). 
      */
-    struct mca_coll_sm_component_t {
+    typedef struct mca_coll_sm_component_t {
         /** Base coll component */
         mca_coll_base_component_2_0_0_t super;
 
@@ -136,19 +61,6 @@ BEGIN_C_DECLS
         /** MCA parameter: Length of a cache line or page (in bytes) */
         int sm_control_size;
 
-        /** MCA parameter: Name of shared memory control / bootstrap
-            mmap file */
-        char *sm_bootstrap_filename;
-
-        /** MCA parameter: Number of segments in the bootstrap file
-            (for use with setting up multiple comm's with sm
-            components simultaneously) */
-        int sm_bootstrap_num_segments;
-
-        /** MCA parameter: Name of the mpool that this component will
-            use */
-        char *sm_mpool_name;
-
         /** MCA parameter: Number of "in use" flags in each
             communicator's area in the data mpool */
         int sm_comm_num_in_use_flags;
@@ -169,54 +81,17 @@ BEGIN_C_DECLS
 
         /******* end of MCA params ********/
 
-        /** Size of the bootstrap area -- calculated in
-            coll_sm_component.c */
-        size_t sm_bootstrap_size;
-
-        /** Size of the data mpool area -- calculated in
-            coll_sm_component.c */
-        size_t sm_data_mpool_size;
-
-        /** Data mpool that will be used */
-        mca_mpool_base_module_t *sm_data_mpool;
-
-        /** Whether we ended up creating the sm mpool or whether
-            someone else created it and we just found it */
-        bool sm_data_mpool_created;
-
-        /** Meta struct containing information about the bootstrap area */
-        mca_common_sm_mmap_t *sm_bootstrap_meta;
-
         /** How many fragment segments are protected by a single
             in-use flags.  This is solely so that we can only perform
             the division once and then just use the value without
             having to re-calculate. */
         int sm_segs_per_inuse_flag;
-
-        /** Whether the component's shared memory has been [lazily]
-            initialized or not */
-        bool sm_component_setup;
-
-        /** Once the component has been lazily initialized, keep the
-            state of it around */
-        bool sm_component_setup_success;
-
-        /** A lock protecting the lazy initialzation of the component
-            (SINCE THERE IS NO STATIC INITIALIZER FOR
-            opal_atomic_lock_t, THIS *MUST* BE THE LAST MEMBER OF THE
-            STRUCT!) */
-        opal_atomic_lock_t sm_component_setup_lock;
-
-    };
-    /**
-     * Convenience typedef
-     */
-    typedef struct mca_coll_sm_component_t mca_coll_sm_component_t;
+    } mca_coll_sm_component_t;
 
     /**
      * Structure for representing a node in the tree
      */
-    struct mca_coll_sm_tree_node_t {
+    typedef struct mca_coll_sm_tree_node_t {
         /** Arbitrary ID number, starting from 0 */
         int mcstn_id;
         /** Pointer to parent, or NULL if root */
@@ -226,11 +101,7 @@ BEGIN_C_DECLS
         /** Pointer to an array of children, or NULL if 0 ==
             mcstn_num_children */
         struct mca_coll_sm_tree_node_t **mcstn_children;
-    };
-    /**
-     * Convenienve typedef
-     */
-    typedef struct mca_coll_sm_tree_node_t mca_coll_sm_tree_node_t;
+    } mca_coll_sm_tree_node_t;
 
     /**
      * Simple structure comprising the "in use" flags.  Contains two
@@ -238,54 +109,39 @@ BEGIN_C_DECLS
      * set of segments and the operation number of the current
      * operation.
      */
-    struct mca_coll_sm_in_use_flag_t {
+    typedef struct mca_coll_sm_in_use_flag_t {
         /** Number of processes currently using this set of
             segments */
         volatile uint32_t mcsiuf_num_procs_using;
         /** Must match data->mcb_count */
         volatile uint32_t mcsiuf_operation_count;
-    };
-    /**
-     * Convenienve typedef
-     */
-    typedef struct mca_coll_sm_in_use_flag_t mca_coll_sm_in_use_flag_t;
+    } mca_coll_sm_in_use_flag_t;
 
     /**
      * Structure containing pointers to various arrays of data in the
-     * data mpool area (one of these indexes a single segment in the
-     * data mpool).  Nothing is hard-coded because all the array
-     * lengths and displacements of the pointers all depend on how
-     * many processes are in the communicator.
+     * per-communicator shmem data segment (one of these indexes a
+     * single segment in the per-communicator shmem data segment).
+     * Nothing is hard-coded because all the array lengths and
+     * displacements of the pointers all depend on how many processes
+     * are in the communicator.
      */
-    struct mca_coll_base_mpool_index_t {
+    typedef struct mca_coll_sm_data_index_t {
         /** Pointer to beginning of control data */
         uint32_t volatile *mcbmi_control;
         /** Pointer to beginning of message fragment data */
         char *mcbmi_data;
-    };
-    typedef struct mca_coll_base_mpool_index_t mca_coll_base_mpool_index_t;
+    } mca_coll_sm_data_index_t;
 
     /**
      * Structure for the sm coll module to hang off the communicator.
      * Contains communicator-specific information, including pointers
-     * into the data mpool for this comm's sm collective operations
-     * area. 
+     * into the per-communicator shmem data data segment for this
+     * comm's sm collective operations area.
      */
-    struct mca_coll_sm_comm_t {
-        /** If this process is the one that invoked mpool_alloc() for
-            the data segment, the return value will be in here.
-            Otherwise, it will be NULL (i.e., only the allocating
-            process will call free). */
-        void *mcb_data_mpool_malloc_addr;
-        /** Base of the data mpool */
-        unsigned char *mcb_mpool_base;
-        /** Offset into the data mpool where this comm's operations
-            area is */
-        size_t mcb_mpool_offset;
-        /** Pointer in the data mpool to the beginning of this comm's
-            operations area (i.e., mcb_mpool_base +
-            mcb_mpool_offset) */
-        unsigned char *mcb_mpool_area;
+    typedef struct mca_coll_sm_comm_t {
+        /* Meta data that we get back from the common mmap allocation
+           function */
+        mca_common_sm_mmap_t *mcb_mmap;
 
         /** Pointer to my barrier control pages (odd index pages are
             "in", even index pages are "out") */
@@ -310,10 +166,10 @@ BEGIN_C_DECLS
         /** "In use" flags indicating which segments are available */
         mca_coll_sm_in_use_flag_t *mcb_in_use_flags;
 
-        /** Array of indexes into the mpool area for control and data
-            fragment passing (containing pointers to each segments
-            control and data areas). */
-        mca_coll_base_mpool_index_t *mcb_mpool_index;
+        /** Array of indexes into the per-communicator shmem data
+            segment for control and data fragment passing (containing
+            pointers to each segments control and data areas). */
+        mca_coll_sm_data_index_t *mcb_data_index;
 
         /** Array of graph nodes representing the tree used for
             communications */
@@ -321,23 +177,24 @@ BEGIN_C_DECLS
 
         /** Operation number (i.e., which segment number to use) */
         uint32_t mcb_operation_count;
-    };
-    /**
-     * Convenience typedef
-     */
-    typedef struct mca_coll_sm_comm_t mca_coll_sm_comm_t;
+    } mca_coll_sm_comm_t;
 
-
-    struct mca_coll_sm_module_t {
+    /** Coll sm module */
+    typedef struct mca_coll_sm_module_t {
+        /** Base module */
 	mca_coll_base_module_t super;
 
-	mca_coll_sm_comm_t *sm_data;
+        /* Whether this module has been lazily initialized or not yet */
+        bool enabled;
+
+        /* Data that hangs off the communicator */
+	mca_coll_sm_comm_t *sm_comm_data;
+
+        /* Underlying reduce function and module */
 	mca_coll_base_module_reduce_fn_t previous_reduce;
 	mca_coll_base_module_t *previous_reduce_module;
-    };
-    typedef struct mca_coll_sm_module_t mca_coll_sm_module_t;
+    } mca_coll_sm_module_t;
     OBJ_CLASS_DECLARATION(mca_coll_sm_module_t);
-
     
     /**
      * Global component instance
@@ -347,14 +204,16 @@ BEGIN_C_DECLS
     /*
      * coll module functions
      */
-
     int mca_coll_sm_init_query(bool enable_progress_threads,
 			       bool enable_mpi_threads);
 
     mca_coll_base_module_t *
     mca_coll_sm_comm_query(struct ompi_communicator_t *comm, int *priority);
-
-    int mca_coll_sm_bootstrap_finalize(void);
+    
+    /* Lazily enable a module (since it involves expensive/slow mmap
+       allocation, etc.) */
+    int ompi_coll_sm_lazy_enable(mca_coll_base_module_t *module,
+                                 struct ompi_communicator_t *comm);
 
     int mca_coll_sm_allgather_intra(void *sbuf, int scount, 
 				    struct ompi_datatype_t *sdtype, 
@@ -461,7 +320,7 @@ BEGIN_C_DECLS
  * Global variables used in the macros (essentially constants, so
  * these are thread safe)
  */
-extern uint32_t mca_coll_sm_iov_size;
+extern uint32_t mca_coll_sm_one;
 
 
 /**
@@ -475,16 +334,16 @@ extern uint32_t mca_coll_sm_iov_size;
 /**
  * Macro to wait for the in-use flag to become idle (used by the root)
  */
-#define FLAG_WAIT_FOR_IDLE(flag) \
-    while (0 != (flag)->mcsiuf_num_procs_using) SPIN
+#define FLAG_WAIT_FOR_IDLE(flag, label) \
+    SPIN_CONDITION(0 == (flag)->mcsiuf_num_procs_using, label)
 
 /**
  * Macro to wait for a flag to indicate that it's ready for this
  * operation (used by non-root processes to know when FLAG_SET() has
  * been called)
  */
-#define FLAG_WAIT_FOR_OP(flag, op) \
-    while ((op) != flag->mcsiuf_operation_count) SPIN
+#define FLAG_WAIT_FOR_OP(flag, op, label) \
+    SPIN_CONDITION((op) == flag->mcsiuf_operation_count, label)
 
 /**
  * Macro to set an in-use flag with relevant data to claim it
@@ -507,8 +366,8 @@ extern uint32_t mca_coll_sm_iov_size;
     (iov).iov_base = \
         (index)->mcbmi_data + \
         ((rank) * mca_coll_sm_component.sm_fragment_size); \
-    (max_data) = (iov).iov_len = mca_coll_sm_component.sm_fragment_size; \
-    ompi_convertor_pack(&(convertor), &(iov), &mca_coll_sm_iov_size, \
+    (iov).iov_len = (max_data); \
+    ompi_convertor_pack(&(convertor), &(iov), &mca_coll_sm_one, \
                         &(max_data) )
 
 /**
@@ -517,8 +376,9 @@ extern uint32_t mca_coll_sm_iov_size;
  */
 #define COPY_FRAGMENT_OUT(convertor, src_rank, index, iov, max_data) \
     (iov).iov_base = (((char*) (index)->mcbmi_data) + \
-                      ((src_rank) * mca_coll_sm_component.sm_fragment_size)); \
-    ompi_convertor_unpack(&(convertor), &(iov), &mca_coll_sm_iov_size, \
+                       ((src_rank) * (mca_coll_sm_component.sm_fragment_size))); \
+    (iov).iov_len = (max_data); \
+    ompi_convertor_unpack(&(convertor), &(iov), &mca_coll_sm_one, \
                           &(max_data) )
 
 /**
@@ -552,12 +412,12 @@ extern uint32_t mca_coll_sm_iov_size;
  * Save the value passed and then reset it when done.  Used in fan out
  * operations.
  */
-#define CHILD_WAIT_FOR_NOTIFY(rank, index, value) \
+#define CHILD_WAIT_FOR_NOTIFY(rank, index, value, label) \
     do { \
         uint32_t volatile *ptr = ((uint32_t*) \
                                   (((char*) index->mcbmi_control) + \
                                    ((rank) * mca_coll_sm_component.sm_control_size))); \
-        while (0 == *ptr) SPIN; \
+        SPIN_CONDITION(0 != *ptr, label); \
         (value) = *ptr; \
         *ptr = 0; \
     } while (0)
@@ -577,13 +437,13 @@ extern uint32_t mca_coll_sm_iov_size;
  * data is in the child's segment.  Save the value when done.  Used
  * for fan in operations.
  */
-#define PARENT_WAIT_FOR_NOTIFY_SPECIFIC(child_rank, parent_rank, index, value) \
+#define PARENT_WAIT_FOR_NOTIFY_SPECIFIC(child_rank, parent_rank, index, value, label) \
     do { \
         size_t volatile *ptr = ((size_t volatile *) \
                                 (((char*) index->mcbmi_control) + \
                                  (mca_coll_sm_component.sm_control_size * \
                                   (parent_rank)))) + child_rank; \
-        while (0 == *ptr) SPIN; \
+        SPIN_CONDITION(0 != *ptr, label); \
         (value) = *ptr; \
         *ptr = 0; \
     } while (0)
