@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -70,12 +71,12 @@ int mca_coll_sm_reduce_intra(void *sbuf, void* rbuf, int count,
 
     /* There are several possibilities:
      *
-
-     * 0. If the datatype is larger than a segment, fall back to basic
+     * 0. If the datatype is larger than a segment, fall back to
+     *    underlying module
      * 1. If the op is user-defined, use the strict order
      * 2. If the op is intrinsic:
      *    a. If the op is float-associative, use the unordered
-     *    b. If the op is not float-asociative:
+     *    b. If the op is not float-associative:
      *       i. if the data is floating point, use the strict order
      *       ii. if the data is not floating point, use the unordered
      */
@@ -87,15 +88,39 @@ int mca_coll_sm_reduce_intra(void *sbuf, void* rbuf, int count,
                                           sm_module->previous_reduce_module);
     } 
 #if WANT_REDUCE_NO_ORDER
-    else if (!ompi_op_is_intrinsic(op) ||
-        (ompi_op_is_intrinsic(op) && !ompi_op_is_float_assoc(op) &&
-         0 != (dtype->flags & DT_FLAG_DATA_FLOAT))) {
-        return reduce_inorder(sbuf, rbuf, count, dtype, op, root, comm, module);
-    } else {
-        return reduce_no_order(sbuf, rbuf, count, dtype, op, root, comm, module);
+    else {
+        /* Lazily enable the module the first time we invoke a
+           collective on it */
+        if (!sm_module->enabled) {
+            if (OMPI_SUCCESS != 
+                (ret = ompi_coll_sm_lazy_enable(module, comm))) {
+                return ret;
+            }
+        }
+        
+        if (!ompi_op_is_intrinsic(op) ||
+            (ompi_op_is_intrinsic(op) && !ompi_op_is_float_assoc(op) &&
+             0 != (dtype->flags & OMPI_DATATYPE_FLAG_DATA_FLOAT))) {
+            return reduce_inorder(sbuf, rbuf, count, dtype, op, 
+                                  root, comm, module);
+        } else {
+            return reduce_no_order(sbuf, rbuf, count, dtype, op, 
+                                   root, comm, module);
+        }
     }
 #else
     else {
+        /* Lazily enable the module the first time we invoke a
+           collective on it */
+        if (!sm_module->enabled) {
+            int ret;
+
+            if (OMPI_SUCCESS != 
+                (ret = ompi_coll_sm_lazy_enable(module, comm))) {
+                return ret;
+            }
+        }
+
         return reduce_inorder(sbuf, rbuf, count, dtype, op, root, comm, module);
     }
 #endif
@@ -142,6 +167,8 @@ int mca_coll_sm_reduce_intra(void *sbuf, void* rbuf, int count,
  * If we don't have a friendly datatype, then free the temporary
  * buffer at the end.
  */
+
+
 static int reduce_inorder(void *sbuf, void* rbuf, int count, 
                           struct ompi_datatype_t *dtype, 
                           struct ompi_op_t *op, 
@@ -150,15 +177,15 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
 {
     struct iovec iov;
     mca_coll_sm_module_t *sm_module = (mca_coll_sm_module_t*) module;
-    mca_coll_sm_comm_t *data = sm_module->sm_data;
+    mca_coll_sm_comm_t *data = sm_module->sm_comm_data;
     int ret, rank, size;
     int flag_num, segment_num, max_segment_num;
     size_t total_size, max_data, bytes;
     mca_coll_sm_in_use_flag_t *flag;
-    ompi_convertor_t convertor;
-    mca_coll_base_mpool_index_t *index;
+    mca_coll_sm_data_index_t *index;
     size_t ddt_size;
     size_t segment_ddt_count, segment_ddt_bytes, zero = 0;
+    ptrdiff_t true_lb, true_extent, lb, extent;
 
     /* Setup some identities */
 
@@ -171,7 +198,13 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
        already guaranteed above that the datatype is not larger than a
        segment, so we'll at least get 1). */
 
+    /* ddt_size is the packed size (e.g., MPI_SHORT_INT is 6) */
     ompi_ddt_type_size(dtype, &ddt_size);
+    /* extent is from lb to ub (e.g., MPI_SHORT_INT is 8) */
+    ompi_ddt_get_extent(dtype, &lb, &extent);
+    /* true_extent is extent of actual type map, ignoring lb and ub
+       (e.g., MPI_SHORT_INT is 8) */
+    ompi_ddt_get_true_extent(dtype, &true_lb, &true_extent);
     segment_ddt_count = mca_coll_sm_component.sm_fragment_size / ddt_size;
     iov.iov_len = segment_ddt_bytes = segment_ddt_count * ddt_size;
     total_size = ddt_size * count;
@@ -188,14 +221,15 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
      *********************************************************************/
     
     if (root == rank) {
+        ompi_convertor_t rtb_convertor, rbuf_convertor;
         char *reduce_temp_buffer, *free_buffer, *reduce_target;
-        ptrdiff_t true_lb, true_extent, lb, extent;
         char *inplace_temp;
         int peer;
         size_t count_left = (size_t)count;
         int frag_num = 0;
         bool first_operation = true;
-        
+        bool sbuf_copied_to_rbuf = false;
+
         /* If the datatype is the same packed as it is unpacked, we
            can save a memory copy and just do the reduction operation
            directly from the shared memory segment.  However, if the
@@ -208,7 +242,20 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
         if (ompi_ddt_is_contiguous_memory_layout(dtype, count)) {
             reduce_temp_buffer = free_buffer = NULL;
         } else {
-            OBJ_CONSTRUCT(&convertor, ompi_convertor_t);
+            /* When we have a non-contiguous datatype, we need one or
+             * two convertors:
+             *
+             * rtb_convertor: unpacking from the shmem to the
+             * reduce_temp_buffer (where we can then apply the
+             * reduction).
+             *
+             * rbuf_convertor: unpacking from the shmem directly to the
+             * rbuf (no need to go to the reduce_temp_buffer first and
+             * then apply the reduction -- just copy straight to the
+             * target buffer).
+             */
+            OBJ_CONSTRUCT(&rtb_convertor, ompi_convertor_t);
+            OBJ_CONSTRUCT(&rbuf_convertor, ompi_convertor_t);
 
             /* See lengthy comment in coll basic reduce about
                explanation for how to malloc the extra buffer.  Note
@@ -235,19 +282,34 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
                                                                 segment_ddt_count, 
                                                                 reduce_temp_buffer,
                                                                 0,
-                                                                &convertor))) {
+                                                                &rtb_convertor))) {
                 free(free_buffer);
                 return ret;
             }
+
+            /* See if we need the rbuf_convertor */
+            if (size - 1 != rank) {
+                if (OMPI_SUCCESS != 
+                    (ret = ompi_convertor_copy_and_prepare_for_recv(
+                                       ompi_mpi_local_convertor,
+				       dtype,
+                                       count, 
+                                       rbuf,
+                                       0,
+                                       &rbuf_convertor))) {
+                    free(free_buffer);
+                    return ret;
+                }
+            }
         }
 
         /* If we're a) doing MPI_IN_PLACE (which means we're the root
            -- wouldn't have gotten down here with MPI_IN_PLACE if we
-           weren't the root), and b) we're not rank 0, then we need to
-           copy the rbuf into a temporary buffer and use that as the
-           sbuf */
+           weren't the root), and b) we're not rank (size-1), then we
+           need to copy the rbuf into a temporary buffer and use that
+           as the sbuf */
 
-        if (MPI_IN_PLACE == sbuf && 0 != rank) {
+        if (MPI_IN_PLACE == sbuf && (size - 1) != rank) {
             inplace_temp = (char*)malloc(true_extent + (count - 1) * extent);
             if (NULL == inplace_temp) {
                 if (NULL != free_buffer) {
@@ -256,6 +318,7 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
                 return OMPI_ERR_OUT_OF_RESOURCE;
             }
             sbuf = inplace_temp - lb;
+            ompi_ddt_copy_content_same_ddt(dtype, count, (char *) sbuf, (char *) rbuf);
         } else {
             inplace_temp = NULL;
         }
@@ -266,7 +329,7 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
             flag_num = (data->mcb_operation_count % 
                         mca_coll_sm_component.sm_comm_num_in_use_flags);
             FLAG_SETUP(flag_num, flag, data);
-            FLAG_WAIT_FOR_IDLE(flag);
+            FLAG_WAIT_FOR_IDLE(flag, reduce_root_flag_label);
             FLAG_RETAIN(flag, size, data->mcb_operation_count);
             ++data->mcb_operation_count;
 
@@ -276,19 +339,21 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
                 flag_num * mca_coll_sm_component.sm_segs_per_inuse_flag;
             max_segment_num = 
                 (flag_num + 1) * mca_coll_sm_component.sm_segs_per_inuse_flag;
-            reduce_target = (((char*) rbuf) + (frag_num * segment_ddt_bytes));
+            reduce_target = (((char*) rbuf) + (frag_num * extent * segment_ddt_count));
             do {
 
-                /* Process 0 (special case) */
-
-                if (rank == 0) {
+                /* Note that all the other coll modules reduce from
+                   process (size-1) to 0, so that's the order we'll do
+                   it here. */
+                /* Process (size-1) is the root (special case) */
+                if (size - 1 == rank) {
                     /* If we're the root *and* the first process to be
                        combined *and* this is the first segment in the
-                       entire algorithm, then just copy the whole
-                       buffer.  That way, we never need to copy from
-                       this process again (i.e., do the copy all at
-                       once since all the data is local, and then
-                       don't worry about it for the rest of the
+                       entire algorithm, then just copy the whole sbuf
+                       to rbuf.  That way, we never need to copy from
+                       my sbuf again (i.e., do the copy all at once
+                       since all the data is local, and then don't
+                       worry about it for the rest of the
                        algorithm) */
                     if (first_operation) {
                         first_operation = false;
@@ -296,54 +361,50 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
                             ompi_ddt_copy_content_same_ddt(dtype,
                                                            count,
                                                            reduce_target, (char*)sbuf);
-                            D(("root copied entire buffer to rbuf (contig ddt, count %d) FIRST OPERATION\n", count));
                         }
+                        sbuf_copied_to_rbuf = true;
                     }
-                } else {
-                    index = &(data->mcb_mpool_index[segment_num]);
-                    PARENT_WAIT_FOR_NOTIFY_SPECIFIC(0, rank, index, max_data);
-                        
-                    /* If we don't need an extra buffer, memcpy the
-                       fragment straight to the output buffer.
-                       Otherwise, unpack. */
+                } 
+
+                /* Process (size-1) is not the root */
+                else {
+                    /* Wait for the data to be copied into shmem, just
+                       like any other non-root process */
+                    index = &(data->mcb_data_index[segment_num]);
+                    PARENT_WAIT_FOR_NOTIFY_SPECIFIC(size - 1, rank, index, max_data, reduce_root_parent_label1);
                         
+                    /* If the datatype is contiguous, just copy it
+                       straight to the reduce_target */
                     if (NULL == free_buffer) {
-                        D(("root: special case -- copy from rank 0 shemem to reduce_target (%d bytes)\n", max_data));
-                        memcpy(reduce_target, index->mcbmi_data, max_data);
-                    } else {
-                        /* This is somethat inefficient -- should be
-                           able to avoid one of the memory copies
-                           here, but doing so would violate an
-                           abstraction barrier in the convertor (i.e.,
-                           directly manipulate some of the private
-                           data on the convertor struct) */
-                        D(("root: special case -- unpack and copy from rank 0 to reduce_target\n"));
-                        COPY_FRAGMENT_OUT(convertor, 0, index, 
+                        memcpy(reduce_target, ((char*)index->mcbmi_data) +
+                               (size - 1) * mca_coll_sm_component.sm_fragment_size, max_data);
+                    } 
+                    /* If the datatype is noncontiguous, use the
+                       rbuf_convertor to unpack it straight to the
+                       rbuf */
+                    else {
+                        max_data = segment_ddt_bytes;
+                        COPY_FRAGMENT_OUT(rbuf_convertor, size - 1, index, 
                                           iov, max_data);
-                        ompi_convertor_set_position(&convertor, &zero);
-                        
-                        ompi_ddt_copy_content_same_ddt(dtype, 
-                                                       max_data / ddt_size,
-                                                       reduce_target,
-                                                       iov.iov_base);
                     }
                 }
 
                 /* Loop over all the remaining processes, receiving
                    and reducing them in order */
 
-                for (peer = 1; peer < size; ++peer) {
-
-                    /* Handle the case where the source is this process */
+                for (peer = size - 2; peer >= 0; --peer) {
 
+                    /* Handle the case where the source is this
+                       process (which, by definition, excludes the
+                       sbuf_copied_to_rbuf case because that can
+                       *only* happen when root==0).  In this case, we
+                       don't need to wait for the peer (i.e., me) to
+                       copy into shmem -- just reduce directly from my
+                       sbuf. */
                     if (rank == peer) {
-                        /* Otherwise, I'm not the first process, so
-                           instead of copying, combine in the next
-                           fragment */
-                        D(("root combining fragment from shmem (contig ddt): count %d (left %d, seg %d)\n", min(count_left, segment_ddt_count), count_left, segment_ddt_count));
                         ompi_op_reduce(op, 
                                        ((char *) sbuf) +
-                                       frag_num * segment_ddt_bytes,
+                                       frag_num * extent * segment_ddt_count,
                                        reduce_target,
                                        min(count_left, segment_ddt_count),
                                        dtype);
@@ -351,24 +412,22 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
 
                     /* Now handle the case where the source is not
                        this process.  Wait for the process to copy to
-                       the segment. */
-
+                       the segment into shmem. */
                     else {
-                        index = &(data->mcb_mpool_index[segment_num]);
+                        index = &(data->mcb_data_index[segment_num]);
                         PARENT_WAIT_FOR_NOTIFY_SPECIFIC(peer, rank, 
-                                                        index, max_data);
+                                                        index, max_data, reduce_root_parent_label2);
                         
                         /* If we don't need an extra buffer, then do the
                            reduction operation on the fragment straight
                            from the shmem. */
                         
                         if (NULL == free_buffer) {
-                            D(("root combining %d elements in shmem from peer %d\n",
-                               max_data / ddt_size, peer));
                             ompi_op_reduce(op,
                                            (index->mcbmi_data + 
                                             (peer * mca_coll_sm_component.sm_fragment_size)),
-                                           reduce_target, max_data / ddt_size,
+                                           reduce_target, 
+                                           min(count_left, segment_ddt_count),
                                            dtype);
                         }
                         
@@ -376,23 +435,22 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
                            buffer and then do the reduction from there */
                         
                         else {
-                            D(("root combining %d elements in copy out buffer from peer %d\n",
-                               max_data / ddt_size, peer));
                             /* Unpack the fragment into my temporary
                                buffer */
-                            COPY_FRAGMENT_OUT(convertor, peer, index, 
+                            max_data = segment_ddt_bytes;
+                            COPY_FRAGMENT_OUT(rtb_convertor, peer, index, 
                                               iov, max_data);
-                            ompi_convertor_set_position(&convertor, &zero);
+                            ompi_convertor_set_position(&rtb_convertor, &zero);
                             
                             /* Do the reduction on this fragment */
                             ompi_op_reduce(op, reduce_temp_buffer,
                                            reduce_target, 
-                                           max_data / ddt_size,
+                                           min(count_left, segment_ddt_count),
                                            dtype);
                         }
                     } /* whether this process was me or not */
                 } /* loop over all proceses */
-
+                
                 /* We've iterated through all the processes -- now we
                    move on to the next segment */
 
@@ -400,7 +458,7 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
                 bytes += segment_ddt_bytes;
                 ++segment_num;
                 ++frag_num;
-                reduce_target += segment_ddt_bytes;
+                reduce_target += extent * segment_ddt_count;
             } while (bytes < total_size && segment_num < max_segment_num);
 
             /* Root is now done with this set of segments */
@@ -410,7 +468,8 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
         /* Kill the convertor, if we had one */
         
         if (NULL != free_buffer) {
-            OBJ_DESTRUCT(&convertor);
+            OBJ_DESTRUCT(&rtb_convertor);
+            OBJ_DESTRUCT(&rbuf_convertor);
             free(free_buffer);
         }
         if (NULL != inplace_temp) {
@@ -423,11 +482,11 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
      *********************************************************************/
 
     else {
-
         /* Here we get a convertor for the full count that the user
            provided (as opposed to the convertor that the root got) */
 
-        OBJ_CONSTRUCT(&convertor, ompi_convertor_t);
+	ompi_convertor_t sbuf_convertor;
+        OBJ_CONSTRUCT(&sbuf_convertor, ompi_convertor_t);
         if (OMPI_SUCCESS != 
             (ret = 
              ompi_convertor_copy_and_prepare_for_send(ompi_mpi_local_convertor,
@@ -435,7 +494,7 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
                                                       count, 
                                                       sbuf,
                                                       0,
-                                                      &convertor))) {
+                                                      &sbuf_convertor))) {
             return ret;
         }
 
@@ -448,7 +507,7 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
             /* Wait for the root to mark this set of segments as
                ours */
             FLAG_SETUP(flag_num, flag, data);
-            FLAG_WAIT_FOR_OP(flag, data->mcb_operation_count);
+            FLAG_WAIT_FOR_OP(flag, data->mcb_operation_count, reduce_nonroot_flag_label);
             ++data->mcb_operation_count;
 
             /* Loop over all the segments in this set */
@@ -458,11 +517,12 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
             max_segment_num = 
                 (flag_num + 1) * mca_coll_sm_component.sm_segs_per_inuse_flag;
             do {
-                index = &(data->mcb_mpool_index[segment_num]);
+                index = &(data->mcb_data_index[segment_num]);
 
                 /* Copy from the user's buffer to my shared mem
                    segment */
-                COPY_FRAGMENT_IN(convertor, index, rank, iov, max_data);
+                max_data = segment_ddt_bytes;
+                COPY_FRAGMENT_IN(sbuf_convertor, index, rank, iov, max_data);
                 bytes += max_data;
 
                 /* Wait for the write to absolutely complete */
@@ -482,7 +542,7 @@ static int reduce_inorder(void *sbuf, void* rbuf, int count,
 
         /* Kill the convertor */
 
-        OBJ_DESTRUCT(&convertor);
+        OBJ_DESTRUCT(&sbuf_convertor);
     }
 
     /* All done */
