@@ -2,13 +2,14 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2009 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2009      University of Houston. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -83,7 +84,8 @@ int ompi_coll_tuned_allgather_intra_bruck(void *sbuf, int scount,
                                           struct ompi_datatype_t *sdtype,
                                           void* rbuf, int rcount,
                                           struct ompi_datatype_t *rdtype,
-                                          struct ompi_communicator_t *comm)
+                                          struct ompi_communicator_t *comm,
+					  mca_coll_base_module_t *module)
 {
    int line = -1;
    int rank, size;
@@ -172,7 +174,7 @@ int ompi_coll_tuned_allgather_intra_bruck(void *sbuf, int scount,
       err = ompi_ddt_get_true_extent(rdtype, &true_lb, &true_extent);
       if (MPI_SUCCESS != err) { line = __LINE__; goto err_hndl; }
 
-      free_buf = (char*) calloc(((true_extent + 
+      free_buf = (char*) calloc(((true_extent + true_lb +
                                   ((size - rank) * rcount - 1) * rext)),
                                 sizeof(char));
       if (NULL == free_buf) { 
@@ -249,7 +251,7 @@ int ompi_coll_tuned_allgather_intra_bruck(void *sbuf, int scount,
  *
  *  TODO: Modify the algorithm to work with any number of nodes.
  *        We can modify code to use identical implementation like MPICH-2:
- *        - using recursive-halving algorith, at the end of each step, 
+ *        - using recursive-halving algorithm, at the end of each step, 
  *          determine if there are nodes who did not exchange their data in that
  *          step, and send them appropriate messages.
  */
@@ -258,7 +260,8 @@ ompi_coll_tuned_allgather_intra_recursivedoubling(void *sbuf, int scount,
                                                   struct ompi_datatype_t *sdtype,
                                                   void* rbuf, int rcount,
                                                   struct ompi_datatype_t *rdtype,
-                                                  struct ompi_communicator_t *comm)
+                                                  struct ompi_communicator_t *comm,
+						  mca_coll_base_module_t *module)
 {
    int line = -1;
    int rank, size, pow2size;
@@ -283,7 +286,8 @@ ompi_coll_tuned_allgather_intra_recursivedoubling(void *sbuf, int scount,
                    size));
 
       return ompi_coll_tuned_allgather_intra_bruck(sbuf, scount, sdtype, 
-                                                   rbuf, rcount, rdtype, comm);
+                                                   rbuf, rcount, rdtype,
+						   comm, module);
    }
 
    OPAL_OUTPUT((ompi_coll_tuned_stream,
@@ -365,7 +369,8 @@ int ompi_coll_tuned_allgather_intra_ring(void *sbuf, int scount,
                                          struct ompi_datatype_t *sdtype,
                                          void* rbuf, int rcount,
                                          struct ompi_datatype_t *rdtype,
-                                         struct ompi_communicator_t *comm)
+                                         struct ompi_communicator_t *comm,
+					 mca_coll_base_module_t *module)
 {
    int line = -1;
    int rank, size;
@@ -495,7 +500,8 @@ ompi_coll_tuned_allgather_intra_neighborexchange(void *sbuf, int scount,
                                                  struct ompi_datatype_t *sdtype,
                                                  void* rbuf, int rcount,
                                                  struct ompi_datatype_t *rdtype,
-                                                 struct ompi_communicator_t *comm)
+                                                 struct ompi_communicator_t *comm,
+						 mca_coll_base_module_t *module)
 {
    int line = -1;
    int rank, size;
@@ -514,7 +520,7 @@ ompi_coll_tuned_allgather_intra_neighborexchange(void *sbuf, int scount,
                    size));
       return ompi_coll_tuned_allgather_intra_ring(sbuf, scount, sdtype,
                                                   rbuf, rcount, rdtype,
-                                                  comm);
+                                                  comm, module);
    }
 
    OPAL_OUTPUT((ompi_coll_tuned_stream,
@@ -613,7 +619,8 @@ int ompi_coll_tuned_allgather_intra_two_procs(void *sbuf, int scount,
                                               struct ompi_datatype_t *sdtype,
                                               void* rbuf, int rcount,
                                               struct ompi_datatype_t *rdtype,
-                                              struct ompi_communicator_t *comm)
+                                              struct ompi_communicator_t *comm,
+					      mca_coll_base_module_t *module)
 {
    int line = -1, err = 0;
    int rank;
@@ -696,49 +703,32 @@ ompi_coll_tuned_allgather_intra_basic_linear(void *sbuf, int scount,
                                              void *rbuf,
                                              int rcount, 
                                              struct ompi_datatype_t *rdtype,
-                                             struct ompi_communicator_t *comm)
+                                             struct ompi_communicator_t *comm,
+					     mca_coll_base_module_t *module)
 {
     int err;
-    char *rbuf_original = NULL, *inplace_temp = NULL;
-    ptrdiff_t true_lb, true_extent, lb, extent;
+    ptrdiff_t lb, extent;
 
     /* Handle MPI_IN_PLACE (see explanantion in reduce.c for how to
        allocate temp buffer) -- note that rank 0 can use IN_PLACE
-       natively, and we'll be ok (actually saves a little bit of
-       copying around) */
-
+       natively, and we can just alias the right position in rbuf
+       as sbuf and avoid using a temporary buffer if gather is
+       implemented correctly */
     if (MPI_IN_PLACE == sbuf && 0 != ompi_comm_rank(comm)) {
        ompi_ddt_get_extent(rdtype, &lb, &extent);
-       ompi_ddt_get_true_extent(rdtype, &true_lb, &true_extent);
-       
-       rbuf_original = (char*)rbuf;
        sbuf = ((char*) rbuf) + (ompi_comm_rank(comm) * extent * rcount);
        sdtype = rdtype;
        scount = rcount;
-       
-       inplace_temp = (char*)malloc((true_extent + (rcount - 1) * extent) * 
-                                    ompi_comm_size(comm));
-       if (NULL == inplace_temp) {
-          return OMPI_ERR_OUT_OF_RESOURCE;
-       }
-       rbuf = inplace_temp - lb;
     } 
 
     /* Gather and broadcast. */
 
-    err = comm->c_coll.coll_gather(sbuf, scount, sdtype, rbuf, rcount,
-                                   rdtype, 0, comm);
+    err = comm->c_coll.coll_gather(sbuf, scount, sdtype,
+				   rbuf, rcount, rdtype,
+				   0, comm, comm->c_coll.coll_gather_module);
     if (MPI_SUCCESS == err) {
-        err = comm->c_coll.coll_bcast(rbuf, rcount * ompi_comm_size(comm), 
-                                      rdtype, 0, comm);
-    }
-
-    /* If we've got a temp buffer, copy back out */
-
-    if (MPI_SUCCESS == err && NULL != inplace_temp) {
-        ompi_ddt_copy_content_same_ddt(rdtype, rcount * ompi_comm_size(comm),
-                                       rbuf_original, (char*)rbuf);
-        free(inplace_temp);
+        err = comm->c_coll.coll_bcast(rbuf, rcount * ompi_comm_size(comm), rdtype,
+				      0, comm, comm->c_coll.coll_bcast_module);
     }
 
     /* All done */
@@ -762,14 +752,14 @@ ompi_coll_tuned_allgather_intra_basic_linear(void *sbuf, int scount,
 int 
 ompi_coll_tuned_allgather_intra_check_forced_init(coll_tuned_force_algorithm_mca_param_indices_t *mca_param_indices)
 {
-   int rc, max_alg = 6, requested_alg;
+   int max_alg = 6, requested_alg;
    
    ompi_coll_tuned_forced_max_algorithms[ALLGATHER] = max_alg;
    
-   rc = mca_base_param_reg_int (&mca_coll_tuned_component.super.collm_version,
-                                "allgather_algorithm_count",
-                                "Number of allgather algorithms available",
-                                false, true, max_alg, NULL);
+   mca_base_param_reg_int (&mca_coll_tuned_component.super.collm_version,
+                           "allgather_algorithm_count",
+                           "Number of allgather algorithms available",
+                           false, true, max_alg, NULL);
    
     mca_param_indices->algorithm_param_index
        = mca_base_param_reg_int(&mca_coll_tuned_component.super.collm_version,
@@ -815,60 +805,63 @@ int ompi_coll_tuned_allgather_intra_do_forced(void *sbuf, int scount,
                                               struct ompi_datatype_t *sdtype,
                                               void* rbuf, int rcount,
                                               struct ompi_datatype_t *rdtype,
-                                              struct ompi_communicator_t *comm)
+                                              struct ompi_communicator_t *comm,
+					      mca_coll_base_module_t *module)
 {
-   OPAL_OUTPUT((ompi_coll_tuned_stream,
-                "coll:tuned:allgather_intra_do_forced selected algorithm %d",
-                comm->c_coll_selected_data->user_forced[ALLGATHER].algorithm));
-   
-   switch (comm->c_coll_selected_data->user_forced[ALLGATHER].algorithm) {
-   case (0):   
-      return ompi_coll_tuned_allgather_intra_dec_fixed (sbuf, scount, sdtype, 
-                                                        rbuf, rcount, rdtype, 
-                                                        comm);
-   case (1):   
-      return ompi_coll_tuned_allgather_intra_basic_linear (sbuf, scount, sdtype,
-                                                           rbuf, rcount, rdtype,
-                                                           comm);
-   case (2):   
-      return ompi_coll_tuned_allgather_intra_bruck (sbuf, scount, sdtype, 
-                                                    rbuf, rcount, rdtype, comm);
-   case (3):   
-      return ompi_coll_tuned_allgather_intra_recursivedoubling (sbuf, scount, 
-                                                                sdtype, 
-                                                                rbuf, rcount, 
-                                                                rdtype, 
-                                                                comm);
-   case (4):
-      return ompi_coll_tuned_allgather_intra_ring (sbuf, scount, sdtype, 
-                                                   rbuf, rcount, rdtype, comm);
-   case (5):
-      return ompi_coll_tuned_allgather_intra_neighborexchange (sbuf, scount, 
-                                                               sdtype, 
-                                                               rbuf, rcount, 
-                                                               rdtype, 
-                                                               comm);
-   case (6):
-      return ompi_coll_tuned_allgather_intra_two_procs (sbuf, scount, sdtype, 
-                                                        rbuf, rcount, rdtype, 
-                                                        comm);
-   default:
-      OPAL_OUTPUT((ompi_coll_tuned_stream,
-                   "coll:tuned:allgather_intra_do_forced attempt to select algorithm %d when only 0-%d is valid?", 
-                   comm->c_coll_selected_data->user_forced[ALLGATHER].algorithm,
-                   ompi_coll_tuned_forced_max_algorithms[ALLGATHER]));
-      return (MPI_ERR_ARG);
+    mca_coll_tuned_module_t *tuned_module = (mca_coll_tuned_module_t*) module;
+    mca_coll_tuned_comm_t *data = tuned_module->tuned_data;
+
+    OPAL_OUTPUT((ompi_coll_tuned_stream,
+		 "coll:tuned:allgather_intra_do_forced selected algorithm %d",
+		 data->user_forced[ALLGATHER].algorithm));
+
+    switch (data->user_forced[ALLGATHER].algorithm) {
+    case (0):   
+	return ompi_coll_tuned_allgather_intra_dec_fixed (sbuf, scount, sdtype, 
+							  rbuf, rcount, rdtype, 
+							  comm, module);
+    case (1):   
+	return ompi_coll_tuned_allgather_intra_basic_linear (sbuf, scount, sdtype,
+							     rbuf, rcount, rdtype,
+							     comm, module);
+    case (2):   
+	return ompi_coll_tuned_allgather_intra_bruck (sbuf, scount, sdtype, 
+						      rbuf, rcount, rdtype,
+						      comm, module);
+    case (3):   
+	return ompi_coll_tuned_allgather_intra_recursivedoubling (sbuf, scount, sdtype, 
+								  rbuf, rcount, rdtype, 
+								  comm, module);
+    case (4):
+	return ompi_coll_tuned_allgather_intra_ring (sbuf, scount, sdtype, 
+						     rbuf, rcount, rdtype,
+						     comm, module);
+    case (5):
+	return ompi_coll_tuned_allgather_intra_neighborexchange (sbuf, scount, sdtype, 
+								 rbuf, rcount, rdtype, 
+								 comm, module);
+    case (6):
+	return ompi_coll_tuned_allgather_intra_two_procs (sbuf, scount, sdtype, 
+							  rbuf, rcount, rdtype, 
+							  comm, module);
+    default:
+	OPAL_OUTPUT((ompi_coll_tuned_stream,
+		     "coll:tuned:allgather_intra_do_forced attempt to select algorithm %d when only 0-%d is valid?", 
+		     data->user_forced[ALLGATHER].algorithm,
+		     ompi_coll_tuned_forced_max_algorithms[ALLGATHER]));
+	return (MPI_ERR_ARG);
     } /* switch */
 
 }
 
 
 int ompi_coll_tuned_allgather_intra_do_this(void *sbuf, int scount,
-                                           struct ompi_datatype_t *sdtype,
-                                           void* rbuf, int rcount,
-                                           struct ompi_datatype_t *rdtype,
-                                           struct ompi_communicator_t *comm,
-                                           int algorithm, int faninout, int segsize)
+					    struct ompi_datatype_t *sdtype,
+					    void* rbuf, int rcount,
+					    struct ompi_datatype_t *rdtype,
+					    struct ompi_communicator_t *comm,
+					    mca_coll_base_module_t *module,
+					    int algorithm, int faninout, int segsize)
 {
    OPAL_OUTPUT((ompi_coll_tuned_stream,
                "coll:tuned:allgather_intra_do_this selected algorithm %d topo faninout %d segsize %d", 
@@ -878,33 +871,31 @@ int ompi_coll_tuned_allgather_intra_do_this(void *sbuf, int scount,
    case (0):   
       return ompi_coll_tuned_allgather_intra_dec_fixed(sbuf, scount, sdtype, 
                                                        rbuf, rcount, rdtype, 
-                                                       comm);
+                                                       comm, module);
    case (1):   
       return ompi_coll_tuned_allgather_intra_basic_linear(sbuf, scount, sdtype,
                                                           rbuf, rcount, rdtype,
-                                                          comm);
+                                                          comm, module);
    case (2): 
       return ompi_coll_tuned_allgather_intra_bruck(sbuf, scount, sdtype, 
-                                                   rbuf, rcount, rdtype, comm);
+                                                   rbuf, rcount, rdtype,
+						   comm, module);
    case (3): 
-      return ompi_coll_tuned_allgather_intra_recursivedoubling(sbuf, scount, 
-                                                               sdtype, 
-                                                               rbuf, rcount, 
-                                                               rdtype, 
-                                                               comm);
+      return ompi_coll_tuned_allgather_intra_recursivedoubling(sbuf, scount, sdtype, 
+                                                               rbuf, rcount, rdtype, 
+                                                               comm, module);
    case (4): 
       return ompi_coll_tuned_allgather_intra_ring(sbuf, scount, sdtype, 
-                                                  rbuf, rcount, rdtype, comm);
+                                                  rbuf, rcount, rdtype,
+						  comm, module);
    case (5): 
-      return ompi_coll_tuned_allgather_intra_neighborexchange(sbuf, scount, 
-                                                              sdtype, 
-                                                              rbuf, rcount,
-                                                              rdtype, 
-                                                              comm);
+      return ompi_coll_tuned_allgather_intra_neighborexchange(sbuf, scount, sdtype, 
+                                                              rbuf, rcount, rdtype, 
+                                                              comm, module);
    case (6):
       return ompi_coll_tuned_allgather_intra_two_procs (sbuf, scount, sdtype, 
                                                         rbuf, rcount, rdtype, 
-                                                        comm);
+                                                        comm, module);
    default:
       OPAL_OUTPUT((ompi_coll_tuned_stream,
                    "coll:tuned:allgather_intra_do_this attempt to select algorithm %d when only 0-%d is valid?", 
