@@ -5,12 +5,12 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2010 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007      Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2008-2009 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008-2010 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2010      Los Alamos National Security, LLC.
  *                         All rights reserved.
  * $COPYRIGHT$
@@ -44,8 +44,10 @@
 #include <sys/mman.h>
 #endif
 
+#include "opal/runtime/opal.h"
 #include "opal/util/basename.h"
 #include "orte/util/show_help.h"
+#include "opal/util/path.h"
 #include "opal/align.h"
 #include "opal/types.h"
 #include "opal/threads/mutex.h"
@@ -226,6 +228,11 @@ mca_common_sm_mmap_t* mca_common_sm_mmap_init(ompi_proc_t **procs,
     if (0 == orte_util_compare_name_fields(ORTE_NS_CMP_ALL,
                                            ORTE_PROC_MY_NAME,
                                            &(procs[0]->proc_name))) {
+        /* Check, whether the specified filename is on a network file system */ 
+        if (opal_mmap_on_nfs_warning && opal_path_nfs(file_name)) { 
+            orte_show_help("help-mpi-common-sm.txt", "mmap on nfs", 1, 
+                           orte_process_info.nodename, file_name); 
+        } 
         /* process initializing the file */
         fd = open(file_name, O_CREAT|O_RDWR, 0600);
         if (fd < 0) {
