@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -13,6 +13,9 @@
  * Copyright (c) 2006-2008 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2006      Voltaire. All rights reserved.
  * Copyright (c) 2007      Mellanox Technologies. All rights reserved.
+ * Copyright (c) 2010      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2011      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  *
  * $COPYRIGHT$
  *
@@ -65,7 +68,7 @@ void mca_mpool_rdma_module_init(mca_mpool_rdma_module_t* mpool)
             OBJ_CLASS(mca_mpool_base_registration_t), 
             0,CACHE_LINE_SIZE,
             0, -1, 32, NULL);
-    OBJ_CONSTRUCT(&mpool->mru_list, opal_list_t);
+    OBJ_CONSTRUCT(&mpool->lru_list, opal_list_t);
     OBJ_CONSTRUCT(&mpool->gc_list, opal_list_t);
     mpool->stat_cache_hit = mpool->stat_cache_miss = mpool->stat_evicted = 0;
     mpool->stat_cache_found = mpool->stat_cache_notfound = 0;
@@ -120,16 +123,25 @@ void* mca_mpool_rdma_alloc(mca_mpool_base_module_t *mpool, size_t size,
     return addr;
 }
 
+/* This function must be called with the rcache lock held */
 static void do_unregistration_gc(struct mca_mpool_base_module_t *mpool)
 {
     mca_mpool_rdma_module_t *mpool_rdma = (mca_mpool_rdma_module_t*)mpool;
     mca_mpool_base_registration_t *reg;
 
     do {
+        /* Remove registration from garbage collection list
+           before deregistering it */
         reg = (mca_mpool_base_registration_t *)
             opal_list_remove_first(&mpool_rdma->gc_list);
-        dereg_mem(mpool, reg);
         mpool->rcache->rcache_delete(mpool->rcache, reg);
+
+        /* Drop the rcache lock before calling dereg_mem as there
+           may be memory allocations */
+        OPAL_THREAD_UNLOCK(&mpool->rcache->lock);
+        dereg_mem(mpool, reg);
+        OPAL_THREAD_LOCK(&mpool->rcache->lock);
+
         OMPI_FREE_LIST_RETURN(&mpool_rdma->reg_list,
                 (ompi_free_list_item_t*)reg);
     } while(!opal_list_is_empty(&mpool_rdma->gc_list));
@@ -150,7 +162,6 @@ static int register_cache_bypass(mca_mpool_base_module_t *mpool,
              mca_mpool_base_page_size_log);
     OMPI_FREE_LIST_GET(&mpool_rdma->reg_list, item, rc);
     if(OMPI_SUCCESS != rc) {
-        OPAL_THREAD_UNLOCK(&mpool->rcache->lock);
         return rc;
     }
     rdma_reg = (mca_mpool_base_registration_t*)item;
@@ -173,6 +184,40 @@ static int register_cache_bypass(mca_mpool_base_module_t *mpool,
     return OMPI_SUCCESS;
 }
 
+static inline bool mca_mpool_rdma_deregister_lru (mca_mpool_base_module_t *mpool) {
+    mca_mpool_rdma_module_t *mpool_rdma = (mca_mpool_rdma_module_t *) mpool;
+    mca_mpool_base_registration_t *old_reg;
+    int rc;
+
+    /* Remove the registration from the cache and list before
+       deregistering the memory */
+    old_reg = (mca_mpool_base_registration_t*)
+        opal_list_remove_first (&mpool_rdma->lru_list);
+    if (NULL == old_reg) {
+        return false;
+    }
+
+    mpool->rcache->rcache_delete(mpool->rcache, old_reg);
+
+    /* Drop the rcache lock while we deregister the memory */
+    OPAL_THREAD_UNLOCK(&mpool->rcache->lock);
+    rc = dereg_mem(mpool, old_reg);
+    OPAL_THREAD_LOCK(&mpool->rcache->lock);
+
+    /* This introduces a potential leak of registrations if
+       the deregistration fails to occur as we no longer have
+       a reference to it. Is this possible? */
+    if (OMPI_SUCCESS != rc) {
+        return false;
+    }
+
+    OMPI_FREE_LIST_RETURN(&mpool_rdma->reg_list,
+                          (ompi_free_list_item_t*)old_reg);
+    mpool_rdma->stat_evicted++;
+
+    return true;
+}
+
 /*
  * register memory
  */
@@ -209,7 +254,7 @@ int mca_mpool_rdma_register(mca_mpool_base_module_t *mpool, void *addr,
                  ((*reg)->base == base && (*reg)->bound == bound))) {
             if(0 == (*reg)->ref_count &&
                     mca_mpool_rdma_component.leave_pinned) {
-                opal_list_remove_item(&mpool_rdma->mru_list,
+                opal_list_remove_item(&mpool_rdma->lru_list,
                         (opal_list_item_t*)(*reg));
             }
             mpool_rdma->stat_cache_hit++;
@@ -250,24 +295,10 @@ int mca_mpool_rdma_register(mca_mpool_base_module_t *mpool, void *addr,
     while((rc = mpool->rcache->rcache_insert(mpool->rcache, rdma_reg,
              mca_mpool_rdma_component.rcache_size_limit)) ==
             OMPI_ERR_TEMP_OUT_OF_RESOURCE) {
-        mca_mpool_base_registration_t *old_reg;
         /* try to remove one unused reg and retry */
-        old_reg = (mca_mpool_base_registration_t*)
-            opal_list_get_last(&mpool_rdma->mru_list);
-        if(opal_list_get_end(&mpool_rdma->mru_list) !=
-                (opal_list_item_t*)old_reg) {
-            rc = dereg_mem(mpool, old_reg);
-            if(MPI_SUCCESS == rc) {
-                mpool->rcache->rcache_delete(mpool->rcache, old_reg);
-                opal_list_remove_item(&mpool_rdma->mru_list,
-                        (opal_list_item_t*)old_reg);
-                OMPI_FREE_LIST_RETURN(&mpool_rdma->reg_list,
-                        (ompi_free_list_item_t*)old_reg);
-                mpool_rdma->stat_evicted++;
-            } else
-                break;
-        } else
+        if (!mca_mpool_rdma_deregister_lru (mpool)) {
             break;
+        }        
     }
 
     if(rc != OMPI_SUCCESS) {
@@ -276,8 +307,14 @@ int mca_mpool_rdma_register(mca_mpool_base_module_t *mpool, void *addr,
         return rc;
     }
 
-    rc = mpool_rdma->resources.register_mem(mpool_rdma->resources.reg_data,
-            base, bound - base + 1, rdma_reg);
+    while (OMPI_ERR_OUT_OF_RESOURCE ==
+           (rc = mpool_rdma->resources.register_mem(mpool_rdma->resources.reg_data,
+                                                    base, bound - base + 1, rdma_reg))) {
+        /* try to remove one unused reg and retry */
+        if (!mca_mpool_rdma_deregister_lru (mpool)) {
+            break;
+        }        
+    }
 
     if(rc != OMPI_SUCCESS) {
         mpool->rcache->rcache_delete(mpool->rcache, rdma_reg);
@@ -289,6 +326,9 @@ int mca_mpool_rdma_register(mca_mpool_base_module_t *mpool, void *addr,
     *reg = rdma_reg;
     (*reg)->ref_count++;
     OPAL_THREAD_UNLOCK(&mpool->rcache->lock);
+
+    /* Cleanup any vmas that we have deferred deletion on */
+    mpool->rcache->rcache_clean(mpool->rcache);
     return OMPI_SUCCESS;
 }
 
@@ -338,7 +378,7 @@ int mca_mpool_rdma_find(struct mca_mpool_base_module_t *mpool, void *addr,
         assert(((void*)(*reg)->bound) >= addr);
         if(0 == (*reg)->ref_count &&
                 mca_mpool_rdma_component.leave_pinned) {
-            opal_list_remove_item(&mpool_rdma->mru_list,
+            opal_list_remove_item(&mpool_rdma->lru_list,
                     (opal_list_item_t*)(*reg));
         }
         mpool_rdma->stat_cache_found++;
@@ -375,19 +415,28 @@ int mca_mpool_rdma_deregister(struct mca_mpool_base_module_t *mpool,
     if(mca_mpool_rdma_component.leave_pinned && registration_is_cachebale(reg))
     {
         /* if leave_pinned is set don't deregister memory, but put it
-         * on MRU list for future use */
-        opal_list_prepend(&mpool_rdma->mru_list, (opal_list_item_t*)reg);
+         * on LRU list for future use */
+        opal_list_append(&mpool_rdma->lru_list, (opal_list_item_t*)reg);
     } else {
+	/* Remove from rcache first */
+	if(!(reg->flags & MCA_MPOOL_FLAGS_CACHE_BYPASS))
+	    mpool->rcache->rcache_delete(mpool->rcache, reg);
+
+	/* Drop the rcache lock before deregistring the memory */
+	OPAL_THREAD_UNLOCK(&mpool->rcache->lock);
         rc = dereg_mem(mpool, reg);
+	OPAL_THREAD_LOCK(&mpool->rcache->lock);
+
         if(OMPI_SUCCESS == rc) {
-            if(!(reg->flags & MCA_MPOOL_FLAGS_CACHE_BYPASS))
-                mpool->rcache->rcache_delete(mpool->rcache, reg);
             OMPI_FREE_LIST_RETURN(&mpool_rdma->reg_list,
                     (ompi_free_list_item_t*)reg);
         }
     }
     OPAL_THREAD_UNLOCK(&mpool->rcache->lock);
 
+    /* Cleanup any vmas that we have deferred deletion on */
+    mpool->rcache->rcache_clean(mpool->rcache);
+
     return rc;
 }
 
@@ -420,7 +469,7 @@ int mca_mpool_rdma_release_memory(struct mca_mpool_base_module_t *mpool,
                 continue;
             }
 
-            opal_list_remove_item(&mpool_rdma->mru_list,(opal_list_item_t*)reg);
+            opal_list_remove_item(&mpool_rdma->lru_list,(opal_list_item_t*)reg);
             opal_list_append(&mpool_rdma->gc_list, (opal_list_item_t*)reg);
         }
     } while(reg_cnt == RDMA_MPOOL_NREGS);
@@ -436,6 +485,7 @@ void mca_mpool_rdma_finalize(struct mca_mpool_base_module_t *mpool)
     mca_mpool_base_registration_t *reg;
     mca_mpool_base_registration_t *regs[RDMA_MPOOL_NREGS];
     int reg_cnt, i;
+    int rc;
 
     /* Statistic */
     if(true == mca_mpool_rdma_component.print_stats) {
@@ -460,24 +510,37 @@ void mca_mpool_rdma_finalize(struct mca_mpool_base_module_t *mpool)
             if(reg->ref_count) {
                 reg->ref_count = 0; /* otherway dereg will fail on assert */
             } else if (mca_mpool_rdma_component.leave_pinned) {
-                opal_list_remove_item(&mpool_rdma->mru_list,
+                opal_list_remove_item(&mpool_rdma->lru_list,
                         (opal_list_item_t*)reg);
             }
 
-            if(dereg_mem(mpool, reg) != OMPI_SUCCESS) {
+	    /* Remove from rcache first */
+            mpool->rcache->rcache_delete(mpool->rcache, reg);
+
+	    /* Drop lock before deregistering memory */
+	    OPAL_THREAD_UNLOCK(&mpool->rcache->lock);
+	    rc = dereg_mem(mpool, reg);
+	    OPAL_THREAD_LOCK(&mpool->rcache->lock);
+
+            if(rc != OMPI_SUCCESS) {
+		/* Potentially lose track of registrations
+		   do we have to put it back? */
                 continue;
             }
 
-            mpool->rcache->rcache_delete(mpool->rcache, reg);
             OMPI_FREE_LIST_RETURN(&mpool_rdma->reg_list,
                     (ompi_free_list_item_t*)reg);
         }
     } while(reg_cnt == RDMA_MPOOL_NREGS);
 
-    OBJ_DESTRUCT(&mpool_rdma->mru_list);
+    OBJ_DESTRUCT(&mpool_rdma->lru_list);
     OBJ_DESTRUCT(&mpool_rdma->gc_list);
     OBJ_DESTRUCT(&mpool_rdma->reg_list);
     OPAL_THREAD_UNLOCK(&mpool->rcache->lock);
+
+    /* Cleanup any vmas that we have deferred deletion on */
+    mpool->rcache->rcache_clean(mpool->rcache);
+
 }
 
 int mca_mpool_rdma_ft_event(int state) {
