@@ -46,7 +46,18 @@ extern "C" {
 struct mca_mtl_psm_module_t { 
     mca_mtl_base_module_t super; /**< base MTL interface */
 
-    int32_t connect_timeout;
+    int32_t      connect_timeout;
+  
+    uint32_t     debug_level;
+    int32_t      ib_unit;
+    int32_t      ib_port;
+    int32_t      ib_service_level;
+    uint64_t     ib_pkey;
+
+#if PSM_VERNO >= 0x010d
+    uint64_t     ib_service_id;
+    psm_path_res_t path_res_type;
+#endif
 
     psm_ep_t	 ep;
     psm_mq_t	 mq;
