@@ -7,6 +7,9 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
+ * Copyright (c) 2010      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -21,14 +24,12 @@
 #include "opal/class/opal_free_list.h"
 #include "opal/class/opal_hash_table.h"
 
-#include "ompi/mca/osc/osc.h"
 #include "ompi/win/win.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/request/request.h"
+#include "ompi/mca/osc/osc.h"
 
-#if defined(__cplusplus) || defined(c_plusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
 
 #define CONTROL_MSG_TAG (-200)
 
@@ -42,14 +43,18 @@ struct ompi_osc_pt2pt_component_t {
     /** lock access to datastructures in the component structure */ 
     opal_mutex_t p2p_c_lock; 
 
-    /** List of ompi_osc_pt2pt_module_ts currently in existance. 
-        Needed so that received fragments can be dispatched to the 
-        correct module */ 
+    /** List of active modules (windows) */
     opal_hash_table_t p2p_c_modules; 
 
     /** max size of eager message */
     size_t p2p_c_eager_size;
 
+    /** Lock for request management */
+    opal_mutex_t p2p_c_request_lock;
+
+    /** Condition variable for request management */
+    opal_condition_t p2p_c_request_cond;
+
     /** free list of ompi_osc_pt2pt_sendreq_t structures */
     opal_free_list_t p2p_c_sendreqs;
     /** free list of ompi_osc_pt2pt_replyreq_t structures */
@@ -58,6 +63,14 @@ struct ompi_osc_pt2pt_component_t {
     opal_free_list_t p2p_c_longreqs;
     /** free list for eager / control meessages */
     opal_free_list_t p2p_c_buffers;
+
+    /** list of outstanding requests, of type ompi_osc_pt2pt_mpireq_t */
+    opal_list_t p2p_c_pending_requests;
+
+#if OMPI_ENABLE_PROGRESS_THREADS
+    opal_thread_t p2p_c_thread;
+    bool p2p_c_thread_run;
+#endif
 };
 typedef struct ompi_osc_pt2pt_component_t ompi_osc_pt2pt_component_t;
 
@@ -69,6 +82,9 @@ struct ompi_osc_pt2pt_module_t {
     /** lock access to data structures in the current module */
     opal_mutex_t p2p_lock;
 
+    /** condition variable for access to current module */
+    opal_condition_t p2p_cond;
+
     /** lock for "atomic" window updates from reductions */
     opal_mutex_t p2p_acc_lock;
 
@@ -78,11 +94,6 @@ struct ompi_osc_pt2pt_module_t {
     /** communicator created with this window */
     ompi_communicator_t *p2p_comm;
 
-    /** control message receive request */
-    struct ompi_request_t *p2p_cb_request;
-
-    opal_list_t p2p_pending_control_sends;
-
     /** list of ompi_osc_pt2pt_sendreq_t structures, and includes all
         requests for this access epoch that have not already been
         started.  p2p_lock must be held when modifying this field. */
@@ -98,32 +109,28 @@ struct ompi_osc_pt2pt_module_t {
         Complete.  For lock, the number of
         messages waiting for completion on on the origin side.  Not
         protected by p2p_lock - must use atomic counter operations. */
-    volatile int32_t p2p_num_pending_out;
+    int32_t p2p_num_pending_out;
 
     /** For MPI_Fence synchronization, the number of expected incoming
         messages.  For Post/Wait, the number of expected updates from
         complete. For lock, the number of messages on the passive side
         we are waiting for.  Not protected by p2p_lock - must use
         atomic counter operations. */
-    volatile int32_t p2p_num_pending_in;
+    int32_t p2p_num_pending_in;
 
     /** Number of "ping" messages from the remote post group we've
         received */
-    volatile int32_t p2p_num_post_msgs;
+    int32_t p2p_num_post_msgs;
 
     /** Number of "count" messages from the remote complete group
         we've received */
-    volatile int32_t p2p_num_complete_msgs;
+    int32_t p2p_num_complete_msgs;
 
     /** cyclic counter for a unique tag for long messages.  Not
         protected by the p2p_lock - must use create_send_tag() to
         create a send tag */
     volatile int32_t p2p_tag_counter;
 
-    /** list of outstanding long messages that must be processes
-        (ompi_osc_pt2pt_request_long).  Protected by p2p_lock. */
-    opal_list_t p2p_long_msgs;
-
     opal_list_t p2p_copy_pending_sendreqs;
     unsigned int *p2p_copy_num_pending_sendreqs;
 
@@ -131,12 +138,8 @@ struct ompi_osc_pt2pt_module_t {
     /* an array of <sizeof(p2p_comm)> ints, each containing the value
        1. */
     int *p2p_fence_coll_counts;
-    /* an array of <sizeof(p2p_comm)> unsigned ints, for use in experimenting
-       with different synchronization costs */
-    unsigned int *p2p_fence_coll_results;
 
     /* ********************* PWSC data ************************ */
-
     struct ompi_group_t *p2p_pw_group;
     struct ompi_group_t *p2p_sc_group;
     bool *p2p_sc_remote_active_ranks;
@@ -146,37 +149,24 @@ struct ompi_osc_pt2pt_module_t {
     int32_t p2p_lock_status; /* one of 0, MPI_LOCK_EXCLUSIVE, MPI_LOCK_SHARED */
     int32_t p2p_shared_count;
     opal_list_t p2p_locks_pending;
+    opal_list_t p2p_unlocks_pending;
     int32_t p2p_lock_received_ack;
 };
 typedef struct ompi_osc_pt2pt_module_t ompi_osc_pt2pt_module_t;
-
 OMPI_MODULE_DECLSPEC extern ompi_osc_pt2pt_component_t mca_osc_pt2pt_component;
 
-/*
+
+/**
  * Helper macro for grabbing the module structure from a window instance
  */
-#if OMPI_ENABLE_DEBUG
-
-static inline ompi_osc_pt2pt_module_t* P2P_MODULE(struct ompi_win_t* win) 
-{
-    ompi_osc_pt2pt_module_t *module = 
-        (ompi_osc_pt2pt_module_t*) win->w_osc_module;
-
-    assert(module->p2p_win == win);
-
-    return module;
-}
-
-#else
 #define P2P_MODULE(win) ((ompi_osc_pt2pt_module_t*) win->w_osc_module)
-#endif
 
 /*
  * Component functions 
  */
 
 int ompi_osc_pt2pt_component_init(bool enable_progress_threads,
-                                 bool enable_mpi_threads);
+                                  bool enable_mpi_threads);
 
 int ompi_osc_pt2pt_component_finalize(void);
 
@@ -188,11 +178,7 @@ int ompi_osc_pt2pt_component_select(struct ompi_win_t *win,
                                    struct ompi_info_t *info,
                                    struct ompi_communicator_t *comm);
 
-int ompi_osc_pt2pt_progress(void);
-
-int ompi_osc_pt2pt_request_test(ompi_request_t ** rptr,
-                                int *completed,
-                                ompi_status_public_t * status );
+int ompi_osc_pt2pt_component_progress(void);
 
 /*
  * Module interface function types 
@@ -203,7 +189,7 @@ int ompi_osc_pt2pt_module_put(void *origin_addr,
                              int origin_count,
                              struct ompi_datatype_t *origin_dt,
                              int target,
-                             int target_disp,
+                             OMPI_PTRDIFF_TYPE target_disp,
                              int target_count,
                              struct ompi_datatype_t *target_dt,
                              struct ompi_win_t *win);
@@ -212,7 +198,7 @@ int ompi_osc_pt2pt_module_accumulate(void *origin_addr,
                                     int origin_count,
                                     struct ompi_datatype_t *origin_dt,
                                     int target,
-                                    int target_disp,
+                                    OMPI_PTRDIFF_TYPE target_disp,
                                     int target_count,
                                     struct ompi_datatype_t *target_dt,
                                     struct ompi_op_t *op,
@@ -222,7 +208,7 @@ int ompi_osc_pt2pt_module_get(void *origin_addr,
                              int origin_count,
                              struct ompi_datatype_t *origin_dt,
                              int target,
-                             int target_disp,
+                             OMPI_PTRDIFF_TYPE target_disp,
                              int target_count,
                              struct ompi_datatype_t *target_dt,
                              struct ompi_win_t *win);
@@ -262,9 +248,8 @@ int ompi_osc_pt2pt_passive_unlock(ompi_osc_pt2pt_module_t *module,
                                   int32_t origin,
                                   int32_t count);
 
+int ompi_osc_pt2pt_passive_unlock_complete(ompi_osc_pt2pt_module_t *module);
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
 
 #endif /* OMPI_OSC_PT2PT_H */
