@@ -35,6 +35,7 @@
 #include "pml_ob1_hdr.h"
 #include "ompi/mca/bml/base/base.h"
 #include "ompi/proc/proc.h"
+#include "ompi/mca/allocator/base/base.h"
 
 BEGIN_C_DECLS
 
@@ -49,9 +50,11 @@ struct mca_pml_ob1_t {
     int free_list_num;      /* initial size of free list */
     int free_list_max;      /* maximum size of free list */
     int free_list_inc;      /* number of elements to grow free list */
-    size_t eager_limit;     /* maximum eager limit size - overrides btl setting */
     size_t send_pipeline_depth;
     size_t recv_pipeline_depth;
+    size_t rdma_put_retries_limit;
+    int max_rdma_per_request;
+    int max_send_per_range;
     bool leave_pinned; 
     int leave_pinned_pipeline;
     
@@ -63,13 +66,19 @@ struct mca_pml_ob1_t {
     ompi_free_list_t recv_frags;
     ompi_free_list_t pending_pckts;
     ompi_free_list_t buffers;
+    ompi_free_list_t send_ranges;
 
     /* list of pending operations */
     opal_list_t pckt_pending;
     opal_list_t send_pending;
     opal_list_t recv_pending;
     opal_list_t rdma_pending;
+    /* List of pending fragments without a matching communicator */
+    opal_list_t non_existing_communicator_pending;
     bool enabled; 
+    char* allocator_name;
+    mca_allocator_base_module_t* allocator; 
+    uint32_t unexpected_limit;
 };
 typedef struct mca_pml_ob1_t mca_pml_ob1_t; 
 
@@ -170,29 +179,14 @@ extern int mca_pml_ob1_start( size_t count,
 
 extern int mca_pml_ob1_ft_event( int state );
 
-END_C_DECLS              
-
-#define MCA_PML_OB1_DES_ALLOC(bml_btl, des, size) \
-MCA_BML_BASE_BTL_DES_ALLOC(bml_btl, des,  \
-   sizeof(mca_pml_ob1_hdr_t) + (sizeof(mca_btl_base_segment_t) << 4), size)
-
-/**
- * structure to associate rdma btl with a registration
- */
-
-struct mca_pml_ob1_rdma_reg_t {
-    struct mca_bml_base_btl_t* bml_btl;
-    struct mca_mpool_base_registration_t* btl_reg;
-};
-typedef struct mca_pml_ob1_rdma_reg_t mca_pml_ob1_rdma_reg_t;
-
-#define MCA_PML_OB1_MAX_REGISTRATIONS 4
+END_C_DECLS
 
 struct mca_pml_ob1_pckt_pending_t {
     ompi_free_list_item_t super;
     ompi_proc_t* proc;
     mca_pml_ob1_hdr_t hdr;
     struct mca_bml_base_btl_t *bml_btl;
+    uint8_t order;
 };
 typedef struct mca_pml_ob1_pckt_pending_t mca_pml_ob1_pckt_pending_t;
 OBJ_CLASS_DECLARATION(mca_pml_ob1_pckt_pending_t);
@@ -211,16 +205,18 @@ do {                                                            \
         (ompi_free_list_item_t*)pckt);                          \
 } while(0)
 
-#define MCA_PML_OB1_ADD_FIN_TO_PENDING(P, D, B)                     \
+#define MCA_PML_OB1_ADD_FIN_TO_PENDING(P, D, B, O, S)               \
     do {                                                            \
         mca_pml_ob1_pckt_pending_t *_pckt;                          \
         int _rc;                                                    \
                                                                     \
         MCA_PML_OB1_PCKT_PENDING_ALLOC(_pckt,_rc);                  \
         _pckt->hdr.hdr_common.hdr_type = MCA_PML_OB1_HDR_TYPE_FIN;  \
-        _pckt->hdr.hdr_fin.hdr_des.pval = (D);                      \
+        _pckt->hdr.hdr_fin.hdr_des = (D);                           \
+        _pckt->hdr.hdr_fin.hdr_fail = (S);                          \
         _pckt->proc = (P);                                          \
         _pckt->bml_btl = (B);                                       \
+        _pckt->order = (O);                                         \
         OPAL_THREAD_LOCK(&mca_pml_ob1.lock);                        \
         opal_list_append(&mca_pml_ob1.pckt_pending,                 \
                 (opal_list_item_t*)_pckt);                          \
@@ -228,42 +224,9 @@ do {                                                            \
     } while(0)
 
 
-int mca_pml_ob1_send_fin_btl(ompi_proc_t* proc, mca_bml_base_btl_t* bml_btl, 
-        void *hdr_des);
-static inline int mca_pml_ob1_send_fin(ompi_proc_t* proc, void *hdr_des,
-        mca_bml_base_btl_t* bml_btl)
-{
-    size_t i;
-    mca_bml_base_endpoint_t* endpoint =
-        (mca_bml_base_endpoint_t*)proc->proc_bml;
+int mca_pml_ob1_send_fin(ompi_proc_t* proc, mca_bml_base_btl_t* bml_btl, 
+        ompi_ptr_t hdr_des, uint8_t order, uint32_t status);
 
-    /* Some BTLs (e.g TCP) can report put/get completion before data actually
-     * hits the buffer on the other side. For this kind of BTLs we need to send
-     * FIN through the same BTL PUT was performed with so network will handle
-     * ordering for us. If we will use another BTL, receiver can get FIN before
-     * data will hit the buffer and complete request prematurely. We mark such
-     * problematic BTLs with MCA_BTL_FLAGS_FAKE_RDMA flag (this kind of RDMA
-     * is really fake, because the real one guaranties that sender will see the
-     * completion only after receiver's NIC confirmed that all the data was
-     * received)
-     */
-    if(bml_btl->btl_flags & MCA_BTL_FLAGS_FAKE_RDMA) {
-        if(mca_pml_ob1_send_fin_btl(proc, bml_btl, hdr_des) == OMPI_SUCCESS)
-            return OMPI_SUCCESS;
-    } else {
-        for(i = 0; i < mca_bml_base_btl_array_get_size(&endpoint->btl_eager);
-                i++) {
-            bml_btl = mca_bml_base_btl_array_get_next(&endpoint->btl_eager);
-            if(mca_pml_ob1_send_fin_btl(proc, bml_btl, hdr_des) == OMPI_SUCCESS)
-                return OMPI_SUCCESS;
-        }
-        bml_btl = NULL;
-    }
-
-    MCA_PML_OB1_ADD_FIN_TO_PENDING(proc, hdr_des, bml_btl);
-
-    return OMPI_ERR_OUT_OF_RESOURCE;
-}
 /* This function tries to resend FIN/ACK packets from pckt_pending queue.
  * Packets are added to the queue when sending of FIN or ACK is failed due to
  * resource unavailability. bml_btl passed to the function doesn't represents
@@ -288,6 +251,7 @@ void mca_pml_ob1_process_pending_rdma(void);
         if(opal_list_get_size(&mca_pml_ob1.rdma_pending))       \
             mca_pml_ob1_process_pending_rdma();                 \
     } while (0)
+
 /*
  * Compute the total number of bytes on supplied descriptor
  */
@@ -301,4 +265,53 @@ do {                                                                        \
    length -= hdrlen;                                                        \
 } while(0)
 
+/* represent BTL chosen for sending request */
+struct mca_pml_ob1_com_btl_t {
+    mca_bml_base_btl_t *bml_btl;
+    struct mca_mpool_base_registration_t* btl_reg;
+    size_t length;
+};
+typedef struct mca_pml_ob1_com_btl_t mca_pml_ob1_com_btl_t;
+
+int mca_pml_ob1_com_btl_comp(const void *v1, const void *v2);
+
+/* Calculate what percentage of a message to send through each BTL according to
+ * relative weight */
+static inline void
+mca_pml_ob1_calc_weighted_length( mca_pml_ob1_com_btl_t *btls, int num_btls, size_t size,
+                                  double weight_total )
+{
+    int i;
+    size_t length_left;
+
+    /* shortcut for common case for only one BTL */
+    if( OPAL_LIKELY(1 == num_btls) ) {
+        btls[0].length = size;
+        return;
+    }
+
+    /* sort BTLs according of their weights so BTLs with smaller weight will
+     * not hijack all of the traffic */
+    qsort( btls, num_btls, sizeof(mca_pml_ob1_com_btl_t),
+           mca_pml_ob1_com_btl_comp );
+
+    for(length_left = size, i = 0; i < num_btls; i++) {
+        mca_bml_base_btl_t* bml_btl = btls[i].bml_btl;
+        size_t length = 0;
+        if( OPAL_UNLIKELY(0 != length_left) ) {
+            length = (length_left > bml_btl->btl->btl_eager_limit)?
+                ((size_t)(size * (bml_btl->btl_weight / weight_total))) :
+                length_left;
+
+            if(length > length_left)
+                length = length_left;
+            length_left -= length;
+        }
+        btls[i].length = length;
+    }
+
+    /* account for rounding errors */
+    btls[0].length += length_left;
+}
+
 #endif
