@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2007 The University of Tennessee and The University
@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2008 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -33,6 +33,7 @@
 #include "pml_ob1_recvfrag.h"
 #include "ompi/mca/bml/base/base.h" 
 #include "pml_ob1_component.h"
+#include "ompi/mca/allocator/base/base.h"
 
 OBJ_CLASS_INSTANCE( mca_pml_ob1_pckt_pending_t,
                     ompi_free_list_item_t,
@@ -46,16 +47,13 @@ mca_pml_ob1_component_init( int* priority, bool enable_progress_threads,
                             bool enable_mpi_threads );
 static int mca_pml_ob1_component_fini(void);
 
-mca_pml_base_component_1_0_0_t mca_pml_ob1_component = {
+mca_pml_base_component_2_0_0_t mca_pml_ob1_component = {
 
     /* First, the mca_base_component_t struct containing meta
        information about the component itself */
 
     {
-      /* Indicate that we are a pml v1.0.0 component (which also implies
-         a specific MCA version) */
-
-      MCA_PML_BASE_VERSION_1_0_0,
+      MCA_PML_BASE_VERSION_2_0_0,
     
       "ob1", /* MCA component name */
       OMPI_MAJOR_VERSION,  /* MCA component major version */
@@ -64,19 +62,22 @@ mca_pml_base_component_1_0_0_t mca_pml_ob1_component = {
       mca_pml_ob1_component_open,  /* component open */
       mca_pml_ob1_component_close  /* component close */
     },
-
-    /* Next the MCA v1.0.0 component meta data */
-
     {
-      /* Whether the component is checkpointable or not */
-      false
+        /* The component is checkpoint ready */
+        MCA_BASE_METADATA_PARAM_CHECKPOINT
     },
 
     mca_pml_ob1_component_init,  /* component init */
     mca_pml_ob1_component_fini   /* component finalize */
+    
 };
 
-
+void *mca_pml_ob1_seg_alloc( struct mca_mpool_base_module_t* mpool,
+                             size_t* size,
+                             mca_mpool_base_registration_t** registration);
+ 
+void mca_pml_ob1_seg_free( struct mca_mpool_base_module_t* mpool,
+                           void* segment );
 
 static inline int mca_pml_ob1_param_register_int(
     const char* param_name,
@@ -90,6 +91,8 @@ static inline int mca_pml_ob1_param_register_int(
 
 static int mca_pml_ob1_component_open(void)
 {
+    mca_allocator_base_component_t* allocator_component;
+
     mca_pml_ob1.free_list_num =
         mca_pml_ob1_param_register_int("free_list_num", 4);
     mca_pml_ob1.free_list_max =
@@ -98,19 +101,42 @@ static int mca_pml_ob1_component_open(void)
         mca_pml_ob1_param_register_int("free_list_inc", 64);
     mca_pml_ob1.priority =
         mca_pml_ob1_param_register_int("priority", 20);
-    mca_pml_ob1.eager_limit =
-        mca_pml_ob1_param_register_int("eager_limit", 128 * 1024);
     mca_pml_ob1.send_pipeline_depth =
         mca_pml_ob1_param_register_int("send_pipeline_depth", 3);
     mca_pml_ob1.recv_pipeline_depth =
         mca_pml_ob1_param_register_int("recv_pipeline_depth", 4);
+    mca_pml_ob1.rdma_put_retries_limit =
+        mca_pml_ob1_param_register_int("rdma_put_retries_limit", 5);
+    mca_pml_ob1.max_rdma_per_request =
+        mca_pml_ob1_param_register_int("max_rdma_per_request", 4);
+    mca_pml_ob1.max_send_per_range =
+        mca_pml_ob1_param_register_int("max_send_per_range", 4);
 
-    mca_pml_ob1.leave_pinned = ompi_mpi_leave_pinned;
-    mca_pml_ob1.leave_pinned_pipeline = (int) ompi_mpi_leave_pinned_pipeline;
+    mca_pml_ob1.unexpected_limit =
+        mca_pml_ob1_param_register_int("unexpected_limit", 128);
+ 
+    mca_base_param_reg_string(&mca_pml_ob1_component.pmlm_version,
+                              "allocator",
+                              "Name of allocator component for unexpected messages",
+                              false, false,
+                              "bucket",
+                              &mca_pml_ob1.allocator_name);
+
+    allocator_component = mca_allocator_component_lookup( mca_pml_ob1.allocator_name );
+    if(NULL == allocator_component) {
+        opal_output(0, "mca_pml_ob1_component_open: can't find allocator: %s\n", mca_pml_ob1.allocator_name);
+        return OMPI_ERROR;
+    }
+    mca_pml_ob1.allocator = allocator_component->allocator_init(true,
+                                                                mca_pml_ob1_seg_alloc,
+                                                                mca_pml_ob1_seg_free, NULL);
+    if(NULL == mca_pml_ob1.allocator) {
+        opal_output(0, "mca_pml_ob1_component_open: unable to initialize allocator\n");
+        return OMPI_ERROR;
+    }
 
     mca_pml_ob1.enabled = false; 
     return mca_bml_base_open(); 
-    
 }
 
 
@@ -139,24 +165,16 @@ mca_pml_ob1_component_init( int* priority,
     }
     *priority = mca_pml_ob1.priority;
 
-    /* buffered send */
-    if(OMPI_SUCCESS != mca_pml_base_bsend_init(enable_mpi_threads)) {
-        opal_output(0, "mca_pml_ob1_component_init: mca_pml_bsend_init failed\n");
-        return NULL;
-    }
-
-    
     if(OMPI_SUCCESS != mca_bml_base_init( enable_progress_threads, 
                                           enable_mpi_threads)) {
         return NULL;
     }
 
-    /* As our own progress function does nothing except calling the BML
-     * progress, let's modify the progress function pointer in our structure
-     * to avoid useless functions calls. The event library will instead call
-     * directly the BML function.
-     */
-     mca_pml_ob1.super.pml_progress = mca_bml.bml_progress;
+    /* Set this here (vs in component_open()) because
+       ompi_mpi_leave_pinned* may have been set after MCA params were
+       read (e.g., by the openib btl) */
+    mca_pml_ob1.leave_pinned = (1 == ompi_mpi_leave_pinned);
+    mca_pml_ob1.leave_pinned_pipeline = (int) ompi_mpi_leave_pinned_pipeline;
 
     return &mca_pml_ob1.super;
 }
@@ -169,12 +187,6 @@ int mca_pml_ob1_component_fini(void)
     if(OMPI_SUCCESS != (rc = mca_bml.bml_finalize()))
         return rc;
 
-
-    /* Shutdown buffered send */
-    if(OMPI_SUCCESS != (rc = mca_pml_base_bsend_fini())) {
-        return rc;
-    }
-
     if(!mca_pml_ob1.enabled)
         return OMPI_SUCCESS; /* never selected.. return success.. */  
     mca_pml_ob1.enabled = false;  /* not anymore */
@@ -183,12 +195,17 @@ int mca_pml_ob1_component_fini(void)
     OBJ_DESTRUCT(&mca_pml_ob1.pckt_pending);
     OBJ_DESTRUCT(&mca_pml_ob1.recv_pending);
     OBJ_DESTRUCT(&mca_pml_ob1.send_pending);
+    OBJ_DESTRUCT(&mca_pml_ob1.non_existing_communicator_pending);
     OBJ_DESTRUCT(&mca_pml_ob1.buffers);
     OBJ_DESTRUCT(&mca_pml_ob1.pending_pckts);
     OBJ_DESTRUCT(&mca_pml_ob1.recv_frags);
     OBJ_DESTRUCT(&mca_pml_ob1.rdma_frags);
     OBJ_DESTRUCT(&mca_pml_ob1.lock);
 
+    if(OMPI_SUCCESS != (rc = mca_pml_ob1.allocator->alc_finalize(mca_pml_ob1.allocator))) {
+        return rc;
+    }
+
 #if 0
     if (mca_pml_base_send_requests.fl_num_allocated !=
         mca_pml_base_send_requests.super.opal_list_length) {
@@ -206,3 +223,14 @@ int mca_pml_ob1_component_fini(void)
 
     return OMPI_SUCCESS;
 }
+
+void *mca_pml_ob1_seg_alloc( struct mca_mpool_base_module_t* mpool,
+                             size_t* size,
+                             mca_mpool_base_registration_t** registration) { 
+    return malloc(*size);
+}
+
+void mca_pml_ob1_seg_free( struct mca_mpool_base_module_t* mpool,
+                           void* segment ) { 
+    free(segment);
+}
