@@ -69,7 +69,8 @@ int MPI_Allreduce(void *sendbuf, void *recvbuf, int count,
         } else if (MPI_OP_NULL == op) {
             err = MPI_ERR_OP;
         } else if (!ompi_op_is_valid(op, datatype, &msg, FUNC_NAME)) {
-            int ret = OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_OP, msg);
+            int ret;
+            ret = OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_OP, msg);
             free(msg);
             return ret;
         } else if( MPI_IN_PLACE == recvbuf ) {
