@@ -53,7 +53,7 @@ int MPI_Attr_put(MPI_Comm comm, int keyval, void *attribute_val)
     OPAL_CR_ENTER_LIBRARY();
 
     ret = ompi_attr_set_c(COMM_ATTR, comm, &comm->c_keyhash, 
-                          keyval, attribute_val, false, true);
+                          keyval, attribute_val, false);
     
     OMPI_ERRHANDLER_RETURN(ret, comm, MPI_ERR_OTHER, FUNC_NAME);  
 }
