@@ -1,11 +1,11 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -20,6 +20,7 @@
 
 #include "ompi/mpi/c/bindings.h"
 #include "ompi/datatype/datatype.h"
+#include "ompi/memchecker.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Bcast = PMPI_Bcast
@@ -37,6 +38,27 @@ int MPI_Bcast(void *buffer, int count, MPI_Datatype datatype,
 {
     int err;
 
+    MEMCHECKER(
+        memchecker_datatype(datatype);
+        memchecker_comm(comm);
+        if (OMPI_COMM_IS_INTRA(comm)) {
+            if (ompi_comm_rank(comm) == root) {
+                /* check whether root's send buffer is defined. */
+                memchecker_call(&opal_memchecker_base_isdefined, buffer, count, datatype);
+            }
+            /* check whether receive buffer is addressable. */
+            memchecker_call(&opal_memchecker_base_isaddressable, buffer, count, datatype);  
+        } else {
+            if (MPI_ROOT == root) {
+                /* check whether root's send buffer is defined. */
+                memchecker_call(&opal_memchecker_base_isdefined, buffer, count, datatype);
+            } else if (MPI_PROC_NULL != root) {
+                /* check whether receive buffer is addressable. */
+                memchecker_call(&opal_memchecker_base_isaddressable, buffer, count, datatype);
+            }
+        }
+    );
+
     if (MPI_PARAM_CHECK) {
       err = MPI_SUCCESS;
       OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
@@ -78,8 +100,11 @@ int MPI_Bcast(void *buffer, int count, MPI_Datatype datatype,
         return MPI_SUCCESS;
     } 
 
+    OPAL_CR_ENTER_LIBRARY();
+
     /* Invoke the coll component to perform the back-end operation */
 
-    err = comm->c_coll.coll_bcast(buffer, count, datatype, root, comm);
+    err = comm->c_coll.coll_bcast(buffer, count, datatype, root, comm,
+                                  comm->c_coll.coll_bcast_module);
     OMPI_ERRHANDLER_RETURN(err, comm, err, FUNC_NAME);
 }
