@@ -1,8 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; -*- */
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2007 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -38,6 +39,8 @@ MPI_File MPI_File_f2c(MPI_Fint file_f)
 {
     int file_index = OMPI_FINT_2_INT(file_f);
 
+    OPAL_CR_NOOP_PROGRESS();
+
     if (MPI_PARAM_CHECK) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
     }
@@ -48,9 +51,9 @@ MPI_File MPI_File_f2c(MPI_Fint file_f)
 
     if (file_index < 0 ||
         file_index >= 
-        ompi_pointer_array_get_size(&ompi_file_f_to_c_table)) {
+        opal_pointer_array_get_size(&ompi_file_f_to_c_table)) {
         return NULL;
     }
 
-    return ompi_pointer_array_get_item(&ompi_file_f_to_c_table, file_index);
+    return (MPI_File)opal_pointer_array_get_item(&ompi_file_f_to_c_table, file_index);
 }
