@@ -1,11 +1,11 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
@@ -57,8 +57,6 @@ int MPI_Get(void *origin_addr, int origin_count,
             rc = MPI_ERR_RMA_SYNC;
         } else if ( target_disp < 0 ) {
             rc = MPI_ERR_DISP;
-        } else if ( (origin_count < 0) || (target_count < 0) ) {
-            rc = MPI_ERR_COUNT;
         } else {
             OMPI_CHECK_DATATYPE_FOR_ONE_SIDED(rc, origin_datatype, origin_count);
             if (OMPI_SUCCESS == rc) {
@@ -70,6 +68,8 @@ int MPI_Get(void *origin_addr, int origin_count,
 
     if (MPI_PROC_NULL == target_rank) return MPI_SUCCESS;
 
+    OPAL_CR_ENTER_LIBRARY();
+
     rc = win->w_osc_module->osc_get(origin_addr, origin_count, origin_datatype,
                                     target_rank, target_disp, target_count,
                                     target_datatype, win);
