@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -52,7 +52,7 @@ int MPI_Info_get_nthkey(MPI_Info info, int n, char *key)
 
     /*
      * 1. Check if info is a valid handle
-     * 2. Check if there are atleast "n" elements
+     * 2. Check if there are at least (n+1) elements
      * 3. If so, give the nth defined key
      */
     if (MPI_PARAM_CHECK) {
@@ -72,9 +72,16 @@ int MPI_Info_get_nthkey(MPI_Info info, int n, char *key)
         }
     }
 
+    OPAL_CR_ENTER_LIBRARY();
+
+    /* Keys are indexed on 0, which makes the "n" parameter offset by
+       1 from the value returned by get_nkeys().  So be sure to
+       compare appropriately. */
+
     err = ompi_info_get_nkeys(info, &nkeys);
     OMPI_ERRHANDLER_CHECK(err, MPI_COMM_WORLD, err, FUNC_NAME);
-    if (nkeys < n) {
+    if (n > (nkeys - 1)) {
+        OPAL_CR_EXIT_LIBRARY();
         return OMPI_ERRHANDLER_INVOKE (MPI_COMM_WORLD, MPI_ERR_INFO_KEY,
                                        FUNC_NAME);
     }
