@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -77,14 +77,14 @@ int MPI_Info_set(MPI_Info info, char *key, char *value)
                                            FUNC_NAME);
         }
 
-        key_length = (key) ? strlen (key) : 0;
+        key_length = (key) ? (int)strlen (key) : 0;
         if ((NULL == key) || (0 == key_length) ||
             (MPI_MAX_INFO_KEY <= key_length)) {
             return OMPI_ERRHANDLER_INVOKE (MPI_COMM_WORLD, MPI_ERR_INFO_KEY,
                                            FUNC_NAME);
         }
 
-        value_length = (value) ? strlen (value) : 0;
+        value_length = (value) ? (int)strlen (value) : 0;
         if ((NULL == value) || (0 == value_length) || 
             (MPI_MAX_INFO_VAL <= value_length)) {
             return OMPI_ERRHANDLER_INVOKE (MPI_COMM_WORLD, MPI_ERR_INFO_VALUE,
@@ -92,6 +92,8 @@ int MPI_Info_set(MPI_Info info, char *key, char *value)
         }
     }
 
+    OPAL_CR_ENTER_LIBRARY();
+
     /*
      * If all is right with the arguments, then call the back-end
      * allocator.
