@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2007 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2006-2008 University of Houston.  All rights reserved.
+ * Copyright (c) 2006-2009 University of Houston.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -131,7 +131,13 @@ int MPI_Intercomm_merge(MPI_Comm intercomm, int high,
     }
 
     /* activate communicator and init coll-module */
-    rc = ompi_comm_activate( &newcomp );           /* new comm */ 
+    rc = ompi_comm_activate( &newcomp,             /* new comm */ 
+                             intercomm,            /* old comm */
+                             NULL,                 /* bridge comm */
+                             NULL,                 /* local leader */
+                             NULL,                 /* remote_leader */
+                             OMPI_COMM_CID_INTER,  /* mode */
+                             -1 );                 /* send_first */
     if ( OMPI_SUCCESS != rc ) {
         goto exit;
     }
