@@ -1,14 +1,15 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -21,6 +22,7 @@
 #include "ompi/mpi/c/bindings.h"
 #include "ompi/datatype/datatype.h"
 #include "ompi/datatype/convertor.h"
+#include "ompi/memchecker.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Pack_external = PMPI_Pack_external
@@ -43,6 +45,11 @@ int MPI_Pack_external(char *datarep, void *inbuf, int incount,
     unsigned int iov_count;
     size_t size;
 
+    MEMCHECKER(
+        memchecker_datatype(datatype);
+        memchecker_call(&opal_memchecker_base_isdefined, inbuf, incount, datatype);
+    );
+
     if (MPI_PARAM_CHECK) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
         if ((NULL == outbuf) || (NULL == position)) {  /* inbuf can be MPI_BOTTOM */
@@ -51,21 +58,29 @@ int MPI_Pack_external(char *datarep, void *inbuf, int incount,
             return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COUNT, FUNC_NAME);
         } else if (outsize < 0) {
             return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_ARG, FUNC_NAME);
-        } else if (MPI_DATATYPE_NULL == datatype) {
+        } else if (MPI_DATATYPE_NULL == datatype || NULL == datatype) {
             return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_TYPE, FUNC_NAME);
         }
     }
 
+    OPAL_CR_ENTER_LIBRARY();
+
     OBJ_CONSTRUCT(&local_convertor, ompi_convertor_t);
 
-    /* The resulting convertor will be set to the position zero */
+    /* The resulting convertor will be set to the position zero. We have to use
+     * CONVERTOR_SEND_CONVERSION in order to force the convertor to do anything
+     * more than just packing the data.
+     */
     ompi_convertor_copy_and_prepare_for_send( ompi_mpi_external32_convertor,
-                                              datatype, incount, inbuf, 0, &local_convertor );
+                                              datatype, incount, inbuf,
+                                              CONVERTOR_SEND_CONVERSION,
+                                              &local_convertor );
 
     /* Check for truncation */
     ompi_convertor_get_packed_size( &local_convertor, &size );
     if( (*position + size) > (size_t)outsize ) {  /* we can cast as we already checked for < 0 */
         OBJ_DESTRUCT( &local_convertor );
+        OPAL_CR_EXIT_LIBRARY();
         return OMPI_ERRHANDLER_INVOKE( MPI_COMM_WORLD, MPI_ERR_TRUNCATE, FUNC_NAME );
     }
 
