@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -21,6 +21,7 @@
 #include "ompi/mpi/c/bindings.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/request/request.h"
+#include "ompi/memchecker.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Recv_init = PMPI_Recv_init
@@ -38,6 +39,12 @@ int MPI_Recv_init(void *buf, int count, MPI_Datatype type, int source,
 {
     int rc = MPI_SUCCESS;
 
+    MEMCHECKER(
+        memchecker_datatype(type);
+        memchecker_call(&opal_memchecker_base_isaddressable, buf, count, type);
+        memchecker_comm(comm);
+    );
+
     if ( MPI_PARAM_CHECK ) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
         OMPI_CHECK_DATATYPE_FOR_RECV(rc, type, count);
@@ -51,6 +58,8 @@ int MPI_Recv_init(void *buf, int count, MPI_Datatype type, int source,
                    (MPI_PROC_NULL != source) &&
                    ompi_comm_peer_invalid(comm, source)) {
             rc = MPI_ERR_RANK;
+        } else if (NULL == request) {
+            rc = MPI_ERR_REQUEST;
         }
         
         OMPI_ERRHANDLER_CHECK(rc, comm, rc, FUNC_NAME);
@@ -69,6 +78,11 @@ int MPI_Recv_init(void *buf, int count, MPI_Datatype type, int source,
         return MPI_SUCCESS;
     }
 
+    OPAL_CR_ENTER_LIBRARY();
+
+    /*
+     * Here, we just initialize the request -- memchecker should set the buffer in MPI_Start.
+     */
     rc = MCA_PML_CALL(irecv_init(buf,count,type,source,tag,comm,request));
     OMPI_ERRHANDLER_RETURN(rc, comm, rc, FUNC_NAME);
 }
