@@ -43,7 +43,7 @@ int MPI_Rsend_init(void *buf, int count, MPI_Datatype type,
 
     MEMCHECKER(
         memchecker_datatype(type);
-        memchecker_call(&opal_memchecker_base_isdefined, buf, count, type);
+        memchecker_call(&opal_memchecker_base_isaddressable, buf, count, type);
         memchecker_comm(comm);
     );
 
