@@ -1,14 +1,15 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -21,6 +22,7 @@
 #include "ompi/mpi/c/bindings.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/request/request.h"
+#include "ompi/memchecker.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Test = PMPI_Test
@@ -36,6 +38,11 @@ static const char FUNC_NAME[] = "MPI_Test";
 int MPI_Test(MPI_Request *request, int *completed, MPI_Status *status) 
 {
     int rc;
+
+    MEMCHECKER(
+        memchecker_request (request);
+    );
+
     if ( MPI_PARAM_CHECK ) {
         rc = MPI_SUCCESS;
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
@@ -47,13 +54,21 @@ int MPI_Test(MPI_Request *request, int *completed, MPI_Status *status)
         OMPI_ERRHANDLER_CHECK(rc, MPI_COMM_WORLD, rc, FUNC_NAME);
     }
 
+    OPAL_CR_ENTER_LIBRARY();
+
     rc = ompi_request_test(request, completed, status);
-    if(*completed < 0) {
+    if (*completed < 0) {
         *completed = 0;
     }
 
-    /* JMS: Tim will fix to invoke on the communicator/window/file on
-       the request (i.e., not COMM_WORLD) */
-    OMPI_ERRHANDLER_RETURN(rc, MPI_COMM_WORLD, rc, FUNC_NAME);
-}
+    MEMCHECKER(
+        opal_memchecker_base_mem_undefined(&status->MPI_ERROR, sizeof(int));
+    );
 
+    OPAL_CR_EXIT_LIBRARY();
+
+    if (OMPI_SUCCESS == rc) {
+        return MPI_SUCCESS;
+    }
+    return ompi_errhandler_request_invoke(1, request, FUNC_NAME);
+}
