@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -19,7 +19,7 @@
 #include "ompi_config.h"
 
 #include "ompi/mpi/f77/bindings.h"
-#include "ompi/mpi/f77/strings.h"
+#include "ompi/mpi/f77/f77_strings.h"
 #include "ompi/file/file.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILE_LAYER
@@ -74,6 +74,6 @@ void mpi_file_get_view_f(MPI_Fint *fh, MPI_Offset *disp,
         *disp = (MPI_Fint) c_disp;
         *etype = MPI_Type_c2f(c_etype);
         *filetype = MPI_Type_c2f(c_filetype);
-        ompi_fortran_string_c2f(c_datarep, datarep, MPI_MAX_DATAREP_STRING);
+        ompi_fortran_string_c2f(c_datarep, datarep, datarep_len);
     }
 }
