@@ -11,7 +11,7 @@
  *                         All rights reserved.
  * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
  *                         reserved. 
- * Copyright (c) 2006-2007 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2008 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -21,9 +21,8 @@
 
 #ifndef OMPI_RUNTIME_PARAMS_H
 #define OMPI_RUNTIME_PARAMS_H
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+
+BEGIN_C_DECLS
 
 /*
  * Global variables
@@ -57,6 +56,16 @@ OMPI_DECLSPEC extern bool ompi_mpi_param_check;
  */
 OMPI_DECLSPEC extern bool ompi_debug_show_handle_leaks;
 
+/**
+ * If > 0, show that many MPI_ALLOC_MEM leaks during MPI_FINALIZE.  If
+ * enabled, memory that was returned via MPI_ALLOC_MEM but was never
+ * freed via MPI_FREE_MEM will be displayed during MPI_FINALIZE.
+ *
+ * This is good debugging for user applications to find out if they
+ * are inadvertantly orphaning MPI "special" memory.
+ */
+OMPI_DECLSPEC extern int ompi_debug_show_mpi_alloc_mem_leaks;
+
 /**
  * Whether or not to actually free MPI handles when their
  * corresponding destructor is invoked.  If enabled, Open MPI will not
@@ -85,87 +94,67 @@ OMPI_DECLSPEC extern bool ompi_mpi_show_mca_params;
 OMPI_DECLSPEC extern char * ompi_mpi_show_mca_params_file;
 
 /**
- * If this value is true, assume that this ORTE job is the only job
- * running on the nodes that have been allocated to it, and bind
- * processes to processors (starting with processor 0).
+ * Whether we should keep the string hostnames of all the MPI
+ * process peers around or not (eats up a good bit of memory).
  */
-OMPI_DECLSPEC extern bool ompi_mpi_paffinity_alone;
+OMPI_DECLSPEC extern bool ompi_mpi_keep_peer_hostnames;
 
-    /**
-     * Whether we should keep the string hostnames of all the MPI
-     * process peers around or not (eats up a good bit of memory).
-     */
-    OMPI_DECLSPEC extern bool ompi_mpi_keep_peer_hostnames;
+/**
+ * Whether an MPI_ABORT should print out a stack trace or not.
+ */
+OMPI_DECLSPEC extern bool ompi_mpi_abort_print_stack;
 
-    /**
-     * Whether an MPI_ABORT should print out a stack trace or not.
-     */
-    OMPI_DECLSPEC extern bool ompi_mpi_abort_print_stack;
+/**
+ * Whether  MPI_ABORT  should  print  out an  identifying  message
+ * (e.g., hostname  and PID)  and loop waiting  for a  debugger to
+ * attach.  The value of the integer is how many seconds to wait:
+ *
+ * 0 = do not print the message and do not loop
+ * negative value = print the message and loop forever
+ * positive value = print the message and delay for that many seconds
+ */
+OMPI_DECLSPEC extern int ompi_mpi_abort_delay;
 
-    /**
-     * Whether we should force all connections to be created during
-     * MPI_INIT (vs. potentially making all the connection lazily upon
-     * first communication with an MPI peer process).
-     */
-    OMPI_DECLSPEC extern bool ompi_mpi_preconnect_all;
+/**
+ * Whether to use the "leave pinned" protocol or not (0 = no, 1 = yes,
+ * -1 = determine at runtime).
+ */
+OMPI_DECLSPEC extern int ompi_mpi_leave_pinned;
 
-    /**
-     * should we wireup the oob completely during MPI_INIT?
-     */
-    OMPI_DECLSPEC extern bool ompi_mpi_preconnect_oob;
+/**
+ * Whether to use the "leave pinned pipeline" protocol or not.
+ */
+OMPI_DECLSPEC extern bool ompi_mpi_leave_pinned_pipeline;
 
-    /**
-     * Whether  MPI_ABORT  should  print  out an  identifying  message
-     * (e.g., hostname  and PID)  and loop waiting  for a  debugger to
-     * attach.  The value of the integer is how many seconds to wait:
-     *
-     * 0 = do not print the message and do not loop
-     * negative value = print the message and loop forever
-     * positive value = print the message and delay for that many seconds
-     */
-    OMPI_DECLSPEC extern int ompi_mpi_abort_delay;
+/**
+ * Whether sparse MPI group storage formats are supported or not.
+ */
+OMPI_DECLSPEC extern bool ompi_have_sparse_group_storage;
 
-    /**
-     * Whether to use the "leave pinned" protocol or not.
-     */
-    OMPI_DECLSPEC extern bool ompi_mpi_leave_pinned;
+/**
+ * Whether sparse MPI group storage formats should be used or not.
+ */
+OMPI_DECLSPEC extern bool ompi_use_sparse_group_storage;
 
-    /**
-     * Whether to use the "leave pinned pipeline" protocol or not.
-     */
-    OMPI_DECLSPEC extern bool ompi_mpi_leave_pinned_pipeline;
-
-    /** 
-     * Do we want a warning if MPI_THREAD_MULTIPLE is used? 
-     */
-    OMPI_DECLSPEC extern bool ompi_mpi_warn_if_thread_multiple;
-
-    /** 
-     * Do we want a warning if progress threads are used?
-     */
-    OMPI_DECLSPEC extern bool ompi_mpi_warn_if_progress_threads;
-
-   /**
-     * Register MCA parameters used by the MPI layer.
-     *
-     * @returns OMPI_SUCCESS
-     *
-     * Registers several MCA parameters and initializes corresponding
-     * global variables to the values obtained from the MCA system.
-     */
+/**
+ * Register MCA parameters used by the MPI layer.
+ *
+ * @returns OMPI_SUCCESS
+ *
+ * Registers several MCA parameters and initializes corresponding
+ * global variables to the values obtained from the MCA system.
+ */
 OMPI_DECLSPEC int ompi_mpi_register_params(void);
 
+/**
+ * Display all MCA parameters used 
+ * 
+ * @returns OMPI_SUCCESS
+ *
+ * Displays in key = value format
+ */
+int ompi_show_all_mca_params(int32_t, int, char *);
 
-    /**
-     * Display all MCA parameters used 
-     * 
-     * @returns OMPI_SUCCESS
-     *
-     * Displays in key = value format
-     */
-    int ompi_show_all_mca_params(int32_t, int, char *);
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
 
 #endif /* OMPI_RUNTIME_PARAMS_H */
