@@ -1,14 +1,15 @@
 //
-// Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+// Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
 //                         University Research and Technology
 //                         Corporation.  All rights reserved.
-// Copyright (c) 2004-2005 The University of Tennessee and The University
+// Copyright (c) 2004-2006 The University of Tennessee and The University
 //                         of Tennessee Research Foundation.  All rights
 //                         reserved.
-// Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+// Copyright (c) 2004-2007 High Performance Computing Center Stuttgart, 
 //                         University of Stuttgart.  All rights reserved.
 // Copyright (c) 2004-2005 The Regents of the University of California.
 //                         All rights reserved.
+// Copryight (c) 2007-2008 Cisco Systems, Inc.  All rights reserved.
 // $COPYRIGHT$
 // 
 // Additional copyrights may follow
@@ -25,24 +26,24 @@
 
 #include <stdio.h>
 #include <ctype.h>
-#if HAVE_UNISTD_H
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if HAVE_NETDB_H
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
-#if HAVE_SYS_PARAM_H
+#ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
 #include <errno.h>
 
-#include "opal/install_dirs.h"
+#include "opal/mca/installdirs/installdirs.h"
 #include "opal/class/opal_object.h"
+#include "opal/runtime/opal.h"
 #include "orte/runtime/runtime.h"
-#include "opal/util/output.h"
+#include "orte/util/show_help.h"
 #include "opal/util/cmd_line.h"
 #include "opal/util/argv.h"
-#include "opal/util/show_help.h"
 #include "ompi/communicator/communicator.h"
 #include "opal/mca/base/base.h"
 #include "ompi/tools/ompi_info/ompi_info.h"
@@ -78,40 +79,46 @@ int main(int argc, char *argv[])
   int i, len;
 
   // Initialize the argv parsing handle
+  if (OMPI_SUCCESS != opal_init_util()) {
+    orte_show_help("help-ompi_info.txt", "lib-call-fail", true, 
+                   "opal_init_util", __FILE__, __LINE__, NULL);
+    exit(ret);
+  }
 
   cmd_line = OBJ_NEW(opal_cmd_line_t);
   if (NULL == cmd_line) {
     ret = errno;
-    opal_show_help("help-ompi_info.txt", "lib-call-fail", true, 
+    orte_show_help("help-ompi_info.txt", "lib-call-fail", true, 
                    "opal_cmd_line_create", __FILE__, __LINE__, NULL);
+    opal_finalize_util();
     exit(ret);
   }
 
-  opal_cmd_line_make_opt(cmd_line, 'v', "version", 2, 
-                         "Show version of Open MPI or a component");
-  opal_cmd_line_make_opt(cmd_line, '\0', "param", 2, 
-                         "Show MCA parameters");
-  opal_cmd_line_make_opt(cmd_line, '\0', "internal", 0, 
-                         "Show internal MCA parameters (not meant to be modified by users)");
-  opal_cmd_line_make_opt(cmd_line, '\0', "path", 1, 
-                         "Show paths that Open MPI was configured with");
-  opal_cmd_line_make_opt(cmd_line, '\0', "arch", 0, 
-                         "Show architecture Open MPI was compiled on");
-  opal_cmd_line_make_opt(cmd_line, 'c', "config", 0, 
-                         "Show configuration options");
-  opal_cmd_line_make_opt(cmd_line, 'h', "help", 0, 
-                         "Show this help message");
-  opal_cmd_line_make_opt(cmd_line, '\0', "pretty", 0, 
-                         "Display output in 'prettyprint' format (default)");
-  opal_cmd_line_make_opt(cmd_line, '\0', "parsable", 0, 
-                         "Display output in parsable format");
-  opal_cmd_line_make_opt(cmd_line, '\0', "parseable", 0, 
-                         "Synonym for --parsable");
-  opal_cmd_line_make_opt(cmd_line, '\0', "hostname", 0, 
-                         "Show the hostname that Open MPI was configured "
-                         "and built on");
-  opal_cmd_line_make_opt(cmd_line, 'a', "all", 0, 
-                         "Show all configuration options and MCA parameters");
+  opal_cmd_line_make_opt3(cmd_line, 'v', NULL, "version", 2, 
+                          "Show version of Open MPI or a component.  The first parameter can be the keywords \"ompi\" or \"all\", a framework name (indicating all components in a framework), or a framework:component string (indicating a specific component).  The second parameter can be one of: full, major, minor, release, greek, svn.");
+  opal_cmd_line_make_opt3(cmd_line, '\0', NULL, "param", 2, 
+                          "Show MCA parameters.  The first parameter is the framework (or the keyword \"all\"); the second parameter is the specific component name (or the keyword \"all\").");
+  opal_cmd_line_make_opt3(cmd_line, '\0', NULL, "internal", 0, 
+                          "Show internal MCA parameters (not meant to be modified by users)");
+  opal_cmd_line_make_opt3(cmd_line, '\0', NULL, "path", 1, 
+                          "Show paths that Open MPI was configured with.  Accepts the following parameters: prefix, bindir, libdir, incdir, mandir, pkglibdir, sysconfdir");
+  opal_cmd_line_make_opt3(cmd_line, '\0', NULL, "arch", 0, 
+                          "Show architecture Open MPI was compiled on");
+  opal_cmd_line_make_opt3(cmd_line, 'c', NULL, "config", 0, 
+                          "Show configuration options");
+  opal_cmd_line_make_opt3(cmd_line, 'h', NULL, "help", 0, 
+                          "Show this help message");
+  opal_cmd_line_make_opt3(cmd_line, '\0', NULL, "pretty", 0, 
+                          "When used in conjunction with other parameters, the output is displayed in 'prettyprint' format (default)");
+  opal_cmd_line_make_opt3(cmd_line, '\0', NULL, "parsable", 0, 
+                          "When used in conjunction with other parameters, the output is displayed in a machine-parsable format");
+  opal_cmd_line_make_opt3(cmd_line, '\0', NULL, "parseable", 0, 
+                          "Synonym for --parsable");
+  opal_cmd_line_make_opt3(cmd_line, '\0', NULL, "hostname", 0, 
+                          "Show the hostname that Open MPI was configured "
+                          "and built on");
+  opal_cmd_line_make_opt3(cmd_line, 'a', NULL, "all", 0, 
+                          "Show all configuration options and MCA parameters");
 
   // Call some useless functions in order to guarantee to link in some
   // global variables.  Only check the return value so that the
@@ -129,7 +136,12 @@ int main(int argc, char *argv[])
 
   // Get MCA parameters, if any */
   
-  mca_base_open();
+  if( OMPI_SUCCESS != mca_base_open() ) {
+      orte_show_help("help-ompi_info.txt", "lib-call-fail", true, "mca_base_open", __FILE__, __LINE__ );
+      OBJ_RELEASE(cmd_line);
+      opal_finalize_util();
+      exit(1);
+  }
   mca_base_cmd_line_setup(cmd_line);
 
   // Do the parsing
@@ -137,14 +149,18 @@ int main(int argc, char *argv[])
   if (OMPI_SUCCESS != opal_cmd_line_parse(cmd_line, false, argc, argv)) {
       cmd_error = true;
   }
-  if (!cmd_error && opal_cmd_line_is_taken(cmd_line, "help") || 
-      opal_cmd_line_is_taken(cmd_line, "h")) {
+  if (!cmd_error && 
+        (opal_cmd_line_is_taken(cmd_line, "help") || 
+         opal_cmd_line_is_taken(cmd_line, "h"))) {
       want_help = true;
   }
   if (cmd_error || want_help) {
       char *usage = opal_cmd_line_get_usage_msg(cmd_line);
-      opal_show_help("help-ompi_info.txt", "usage", true, usage);
+      orte_show_help("help-ompi_info.txt", "usage", true, usage);
       free(usage);
+      mca_base_close();
+      OBJ_RELEASE(cmd_line);
+      opal_finalize_util();
       exit(cmd_error ? 1 : 0);
   }
 
@@ -169,11 +185,20 @@ int main(int argc, char *argv[])
   ompi_info::mca_types.push_back("orte");
   ompi_info::mca_types.push_back("opal");
 
+  ompi_info::mca_types.push_back("filter");
+  ompi_info::mca_types.push_back("backtrace");
+  ompi_info::mca_types.push_back("memchecker");
   ompi_info::mca_types.push_back("memory");
   ompi_info::mca_types.push_back("paffinity");
+  ompi_info::mca_types.push_back("carto");
   ompi_info::mca_types.push_back("maffinity");
   ompi_info::mca_types.push_back("timer");
-
+  ompi_info::mca_types.push_back("installdirs");
+#if OPAL_ENABLE_FT == 1
+  ompi_info::mca_types.push_back("crs");
+#endif
+  ompi_info::mca_types.push_back("dpm");
+  ompi_info::mca_types.push_back("pubsub");
   ompi_info::mca_types.push_back("allocator");
   ompi_info::mca_types.push_back("coll");
   ompi_info::mca_types.push_back("io");
@@ -182,23 +207,34 @@ int main(int argc, char *argv[])
   ompi_info::mca_types.push_back("bml");
   ompi_info::mca_types.push_back("rcache");
   ompi_info::mca_types.push_back("btl");
+  ompi_info::mca_types.push_back("mtl");
   ompi_info::mca_types.push_back("topo");
   ompi_info::mca_types.push_back("osc");
+  ompi_info::mca_types.push_back("common");
+#if OPAL_ENABLE_FT == 1
+  ompi_info::mca_types.push_back("crcp");
+#endif
 
-  ompi_info::mca_types.push_back("errmgr");
-  ompi_info::mca_types.push_back("gpr");
+#if !ORTE_DISABLE_FULL_SUPPORT
   ompi_info::mca_types.push_back("iof");
-  ompi_info::mca_types.push_back("ns");
   ompi_info::mca_types.push_back("oob");
+  ompi_info::mca_types.push_back("odls");
   ompi_info::mca_types.push_back("ras");
-  ompi_info::mca_types.push_back("rds");
   ompi_info::mca_types.push_back("rmaps");
-  ompi_info::mca_types.push_back("rmgr");
   ompi_info::mca_types.push_back("rml");
-  ompi_info::mca_types.push_back("pls");
-  ompi_info::mca_types.push_back("sds");
-  ompi_info::mca_types.push_back("soh");
-
+  ompi_info::mca_types.push_back("routed");
+  ompi_info::mca_types.push_back("plm");
+#if OPAL_ENABLE_FT == 1
+  ompi_info::mca_types.push_back("snapc");
+#endif
+  ompi_info::mca_types.push_back("filem");
+#endif
+    /* these are always included */
+    ompi_info::mca_types.push_back("errmgr");
+    ompi_info::mca_types.push_back("ess");
+    ompi_info::mca_types.push_back("grpcomm");
+    ompi_info::mca_types.push_back("shmem");
+    
   // Execute the desired action(s)
 
   if (opal_cmd_line_is_taken(cmd_line, "pretty")) {
@@ -217,7 +253,11 @@ int main(int argc, char *argv[])
     acted = true;
   }
   if (want_all || opal_cmd_line_is_taken(cmd_line, "arch")) {
-    do_arch(cmd_line);
+    do_arch();
+    acted = true;
+  }
+  if (want_all || opal_cmd_line_is_taken(cmd_line, "hostname")) {
+    do_hostname();
     acted = true;
   }
   if (want_all || opal_cmd_line_is_taken(cmd_line, "config")) {
@@ -233,8 +273,9 @@ int main(int argc, char *argv[])
 
   if (!acted) {
     ompi_info::show_ompi_version(ver_full);
-    ompi_info::show_path(path_prefix, OPAL_PREFIX);
-    ompi_info::do_arch(cmd_line);
+    ompi_info::show_path(path_prefix, opal_install_dirs.prefix);
+    ompi_info::do_arch();
+    ompi_info::do_hostname();
     ompi_info::do_config(false);
     ompi_info::open_components();
     for (ompi_info::type_vector_t::size_type i = 0; 
@@ -257,6 +298,8 @@ int main(int argc, char *argv[])
   ompi_info::close_components();
   OBJ_RELEASE(cmd_line);
   mca_base_close();
-  opal_class_finalize();
+
+  opal_finalize_util();
+
   return 0;
 }
