@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2004-2006 The University of Tennessee and The University
@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -29,8 +30,8 @@
  * to generate internal documentation (by default, it is skipped).
  */
 
-#ifndef OMPI_MCA_BASE_PARAM_INTERNAL_H
-#define OMPI_MCA_BASE_PARAM_INTERNAL_H
+#ifndef OPAL_MCA_BASE_PARAM_INTERNAL_H
+#define OPAL_MCA_BASE_PARAM_INTERNAL_H
 
 #include "opal_config.h"
 
@@ -63,7 +64,7 @@ typedef union {
  * default value.
  */
 struct mca_base_param_t {
-    /** Allow this to be an OMPI OBJ */
+    /** Allow this to be an OPAL OBJ */
     opal_object_t mbp_super;
 
     /** Enum indicating the type of the parameter (integer or string) */
@@ -77,6 +78,15 @@ struct mca_base_param_t {
     /** Full parameter name, in case it is not
        <type>_<component>_<param> */
     char *mbp_full_name;
+
+    /** List of synonym names for this parameter.  This *must* be a
+        pointer (vs. a plain opal_list_t) because we copy this whole
+        struct into a new param for permanent storage
+        (opal_vale_array_append_item()), and the internal pointers in
+        the opal_list_t will be invalid when that happens.  Hence, we
+        simply keep a pointer to an external opal_list_t.  Synonyms
+        are uncommon enough that this is not a big performance hit. */
+    opal_list_t *mbp_synonyms;
     
     /** Whether this is internal (not meant to be seen / modified by
         users) or not */
@@ -85,11 +95,15 @@ struct mca_base_param_t {
         was registered (e.g., when true, useful for reporting values,
         like the value of the GM library that was linked against) */
     bool mbp_read_only;
+    /** Whether this MCA parameter (*and* all of its synonyms) is
+        deprecated or not */
+    bool mbp_deprecated;
+    /** Whether the warning message for the deprecated MCA param has
+        been shown already or not */
+    bool mbp_deprecated_warning_shown;
     /** Help message associated with this parameter */
     char *mbp_help_msg;
 
-    /** Keyval value for MPI attribute parameters */
-    int mbp_keyval;
     /** Environment variable name */
     char *mbp_env_var_name;
 
@@ -100,6 +114,8 @@ struct mca_base_param_t {
     bool mbp_file_value_set;
     /** Value of the parameter found in a file */
     mca_base_param_storage_t mbp_file_value;
+    /** File the value came from */
+    char *mbp_source_file;
 
     /** Whether or not we have an override value */
     bool mbp_override_value_set;
@@ -127,13 +143,15 @@ OPAL_DECLSPEC OBJ_CLASS_DECLARATION(mca_base_param_t);
  * Structure for holding param names and values read in from files.
  */
 struct mca_base_param_file_value_t {
-    /** Allow this to be an OMPI OBJ */
+    /** Allow this to be an OPAL OBJ */
     opal_list_item_t super;
     
     /** Parameter name */
     char *mbpfv_param;
     /** Parameter value */
     char *mbpfv_value;
+    /** File it came from */
+    char *mbpfv_file;
 };
 /**
  * \internal
@@ -166,4 +184,4 @@ OPAL_DECLSPEC int mca_base_parse_paramfile(const char *paramfile);
 }
 #endif
     
-#endif /* OMPI_MCA_BASE_PARAM_INTERNAL_H */
+#endif /* OPAL_MCA_BASE_PARAM_INTERNAL_H */
