@@ -2,13 +2,14 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Cisco, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -23,11 +24,16 @@
 #include "opal/mca/base/base.h"
 #include "opal/mca/paffinity/paffinity.h"
 #include "opal/mca/paffinity/base/base.h"
-#include "opal/mca/paffinity/base/internal.h"
-
 
 int opal_paffinity_base_close(void)
 {
+    /* If there is a selected paffinity module, finalize it */
+
+    if (NULL != opal_paffinity_base_module &&
+        NULL != opal_paffinity_base_module->paff_module_finalize) {
+        opal_paffinity_base_module->paff_module_finalize();
+    }
+
     /* Close all components that are still open (this should only
        happen during ompi_info). */
 
