@@ -19,10 +19,12 @@
 #include "opal_config.h"
 
 #include <stdlib.h>
+#ifdef HAVE_STRING_H
 #include <string.h>
-#if HAVE_DIRNAME
+#endif  /* HAVE_STRING_H */
+#ifdef HAVE_LIBGEN_H
 #include <libgen.h>
-#endif  /* HAVE_DIRNAME */
+#endif  /* HAVE_LIBGEN_H */
 
 #include "opal/util/basename.h"
 #include "opal/util/os_path.h"
@@ -70,32 +72,9 @@ char *opal_basename(const char *filename)
     const char sep = OPAL_PATH_SEP[0];
 
     /* Check for the bozo case */
-
     if (NULL == filename) {
         return NULL;
     }
-
-    /* On Windows, automatically exclude a drive designator */
-
-#ifdef __WINDOWS__
-    if (strlen(filename) == 2 &&
-        isalpha(filename[0]) && ':' == filename[1]) {
-        return strdup(filename);
-    } else if (strlen(filename) == 3 &&
-        isalpha(filename[0]) && ':' == filename[1] && sep == filename[2]) {
-        return strdup(filename);
-    }
-
-    if (':' == filename[1] && isalpha(filename[0])) {
-        filename += 2;
-        if (sep == filename[0]) {
-            ++filename;
-        }
-    }
-#endif
-
-    /* Check for the bozo cases */
-
     if (0 == strlen(filename)) {
         return strdup("");
     }
@@ -103,8 +82,23 @@ char *opal_basename(const char *filename)
         return strdup(filename);
     }
 
-    /* Remove trailing sep's (note that we already know that strlen > 0) */
+    /* On Windows, automatically exclude the drive designator */
 
+#ifdef __WINDOWS__
+    if( isalpha(filename[0]) && (':' == filename[1]) ) {
+        if( strlen(filename) == 2 ) {
+            return strdup(filename);
+        } else if( strlen(filename) == 3 && (sep == filename[2]) ) {
+            return strdup(filename);
+        }
+        filename += 2;
+        if( sep == filename[0] ) {
+            ++filename;
+        }
+    }
+#endif
+
+    /* Remove trailing sep's (note that we already know that strlen > 0) */
     tmp = strdup(filename);
     for (i = strlen(tmp) - 1; i > 0; --i) {
         if (sep == tmp[i]) {
@@ -119,14 +113,13 @@ char *opal_basename(const char *filename)
     }
 
     /* Look for the final sep */
-
-    ret = strrchr(tmp, sep);
+    ret = opal_find_last_path_separator( tmp, strlen(tmp) );
     if (NULL == ret) {
         return tmp;
     }
     ret = strdup(ret + 1);
     free(tmp);
-    return opal_make_filename_os_friendly(ret);
+    return ret;
 }
 
 char* opal_dirname(const char* filename)
@@ -150,13 +143,21 @@ char* opal_dirname(const char* filename)
             }
             if( p != filename ) {
                 char* ret = (char*)malloc( p - filename + 1 );
+#ifdef HAVE_STRNCPY_S
                 strncpy_s( ret, (p - filename + 1), filename, p - filename );
+#else
+                strncpy(ret, filename, p - filename);
+#endif
                 ret[p - filename] = '\0';
                 return opal_make_filename_os_friendly(ret);
             }
             break;  /* return the duplicate of "." */
         }
     }
+#ifdef HAVE__STRDUP
     return _strdup(".");
+#else
+    return strdup(".");
+#endif
 #endif  /* defined(HAVE_DIRNAME) */
 }
