@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007-2008 Cisco, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -29,7 +30,7 @@
  * Streams can multiplex output to several kinds of outputs (one of
  * each):
  *
- * - the syslog
+ * - the syslog (if available)
  * - standard output
  * - standard error
  * - file
@@ -63,13 +64,21 @@
 
 #include "opal_config.h"
 
+#ifdef HAVE_STDARG_H
 #include <stdarg.h>
+#endif
+
+/*
+ * For C++, we should not need the file, anyhow.
+ * Circumentvent problems in C++ compilers.
+ */
+#if defined(HAVE_STDBOOL_H) && !(defined(c_plusplus) || defined(__cplusplus))
+#include <stdbool.h>
+#endif
 
 #include "opal/class/opal_object.h"
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
 
 /**
  * \class opal_output_stream_t 
@@ -130,10 +139,21 @@ struct opal_output_stream_t {
      *
      * When this field is non-NULL, it is prefixed to all lines of
      * output on the stream.  When this field is NULL, no prefix is
-     * added to each line of output in the stream.
+     * added to each line of output in the stream. The prefix is copied
+     * to an internal structure in the call to opal_output_open()!
      */
     char *lds_prefix;
     
+    /**
+     * String suffix added to all output on the stream.
+     *
+     * When this field is non-NULL, it is appended to all lines of
+     * output on the stream.  When this field is NULL, no suffix is
+     * added to each line of output in the stream. The suffix is copied
+     * to an internal structure in the call to opal_output_open()!
+     */
+    char *lds_suffix;
+
     /**
      * Indicates whether the output of the stream is
      * debugging/developer-only output or not.
@@ -204,6 +224,7 @@ struct opal_output_stream_t {
      * for details on what happens in this situation.
      */
     char *lds_file_suffix;
+
 };
 
     /**
@@ -341,7 +362,7 @@ struct opal_output_stream_t {
      * created, opal_output() will automatically create the file and
      * writing to it.
      */
-    OPAL_DECLSPEC void opal_output(int output_id, const char *format, ...);
+    OPAL_DECLSPEC void opal_output(int output_id, const char *format, ...) __opal_attribute_format__(__printf__, 2, 3);
     
     /**
      * Send output to a stream only if the passed verbosity level is
@@ -372,8 +393,37 @@ struct opal_output_stream_t {
      * @see opal_output_set_verbosity()
      */
     OPAL_DECLSPEC void opal_output_verbose(int verbose_level, int output_id, 
-                                           const char *format, ...);
-    
+                                           const char *format, ...) __opal_attribute_format__(__printf__, 3, 4);
+
+   /**
+    * Same as opal_output_verbose(), but takes a va_list form of varargs.
+    */
+    OPAL_DECLSPEC void opal_output_vverbose(int verbose_level, int output_id, 
+                                            const char *format, va_list ap);
+
+    /**    
+     * Send output to a string if the verbosity level is high enough.
+     *
+     * @param output_id Stream id returned from opal_output_open().
+     * @param level Target verbosity level.
+     * @param format printf-style format string.
+     * @param varargs printf-style varargs list to fill the string
+     * specified by the format parameter.
+     *
+     * Exactly the same as opal_output_verbose(), except the output it
+     * sent to a string instead of to the stream.  If the verbose
+     * level is not high enough, NULL is returned.  The caller is
+     * responsible for free()'ing the returned string.
+     */
+    OPAL_DECLSPEC char *opal_output_string(int verbose_level, int output_id, 
+                                           const char *format, ...) __opal_attribute_format__(__printf__, 3, 4);
+
+   /**
+    * Same as opal_output_string, but accepts a va_list form of varargs.
+    */
+    OPAL_DECLSPEC char *opal_output_vstring(int verbose_level, int output_id, 
+                                            const char *format, va_list ap);
+
     /**
      * Set the verbosity level for a stream.
      *
@@ -385,6 +435,14 @@ struct opal_output_stream_t {
      */
     OPAL_DECLSPEC void opal_output_set_verbosity(int output_id, int level);
 
+    /**
+     * Get the verbosity level for a stream
+     *
+     * @param output_id Stream id returned from opal_output_open()
+     * @returns Verbosity of stream
+     */
+    OPAL_DECLSPEC int opal_output_get_verbosity(int output_id);
+
     /**
      * Set characteristics for output files.
      *
@@ -478,9 +536,7 @@ struct opal_output_stream_t {
  */
 OPAL_DECLSPEC OBJ_CLASS_DECLARATION(opal_output_stream_t);
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
 
 #endif /* OPAL_OUTPUT_H_ */
 
