@@ -18,14 +18,16 @@
 
 
 #include "orte_config.h"
-#include "orte/orte_constants.h"
+#include "orte/constants.h"
 
 #include "opal/mca/mca.h"
 #include "opal/mca/base/base.h"
 #include "opal/mca/base/mca_base_param.h"
-#include "opal/util/output.h"
+#include "orte/util/show_help.h"
 #include "opal/util/trace.h"
 
+#include "orte/util/show_help.h"
+
 #include "orte/mca/errmgr/base/base.h"
 #include "orte/mca/errmgr/base/errmgr_private.h"
 
@@ -47,20 +49,17 @@
  */
 int orte_errmgr_base_output = -1;
 /*
- * we must define a default module so that the error logging
- * functions can be available as early as possible
+ * define a default module that all application procs
+ * can use without having to open the framework. The
+ * decision on whether or not to open the framework is
+ * made in orte_init
  */
-orte_errmgr_base_module_t orte_errmgr_default = {
-    orte_errmgr_base_log,
+orte_errmgr_base_module_t orte_errmgr = {
     orte_errmgr_base_proc_aborted_not_avail,
     orte_errmgr_base_incomplete_start_not_avail,
-    orte_errmgr_base_error_detected,
-    orte_errmgr_base_register_job_not_avail,
-    orte_errmgr_base_abort,
-    orte_errmgr_base_abort_procs_request_not_avail
+    orte_errmgr_base_register_cb_not_avail,
+    orte_errmgr_base_error_abort
 };
-/* start out with a default module */
-orte_errmgr_base_module_t orte_errmgr;
 
 bool orte_errmgr_base_selected = false;
 opal_list_t orte_errmgr_base_components_available;
@@ -73,26 +72,12 @@ bool orte_errmgr_initialized = false;
  */
 int orte_errmgr_base_open(void)
 {
-    int param, value;
-
     OPAL_TRACE(5);
     
     if (!orte_errmgr_initialized) { /* ensure we only do this once */
       
-        /* Debugging / verbose output */
-        
-        param = mca_base_param_reg_int_name("errmgr", "base_verbose",
-                                            "Verbosity level for the errmgr framework",
-                                            false, false, 0, &value);
-        if (value != 0) {
-            orte_errmgr_base_output = opal_output_open(NULL);
-        } else {
-            orte_errmgr_base_output = -1;
-        }
+        orte_errmgr_base_output = opal_output_open(NULL);
 
-        /* set the default module */
-        orte_errmgr = orte_errmgr_default;
-        
         /* Open up all available components */
     
         if (ORTE_SUCCESS != 
