@@ -20,12 +20,13 @@
 #include "orte_config.h"
 
 #include "opal/mca/base/base.h"
+#include "opal/util/output.h"
 #include "opal/mca/base/mca_base_param.h"
 #include "opal/event/event.h"
 
-#include "orte/util/show_help.h"
 #include "orte/util/proc_info.h"
 #include "orte/mca/rml/rml.h"
+#include "orte/mca/rml/rml_types.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/runtime/orte_globals.h"
 
