@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2010      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -24,19 +25,32 @@
 #define _MCA_OOB_TCP_ADDR_H_
 
 #include "orte_config.h"
+#include "orte/types.h"
+
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
-#include "orte/dss/dss.h"
-#include "opal/class/opal_object.h"
-#include "orte/mca/ns/ns_types.h"
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+#include "opal/class/opal_object.h"
+
+#include "opal/dss/dss.h"
+
+BEGIN_C_DECLS
+
+#define MCA_OOB_TCP_ADDR_UNCLASSIFIED	0  /* we don't know anything */
+#define MCA_OOB_TCP_ADDR_MATCHED	1  /* peer has IP on the same LAN */
+#define MCA_OOB_TCP_ADDR_IPV6		2  /* peer has an IPv6 address */
+#define MCA_OOB_TCP_ADDR_IPV4public	4  /* peer has public IPv4 address */
+
+#define MCA_OOB_TCP_ADDR_TYPE_AFINET   0x01
+#define MCA_OOB_TCP_ADDR_TYPE_AFINET6  0x02
+
 /**
  * Address info published to registry
  */
@@ -46,42 +60,26 @@ struct mca_oob_tcp_addr_t {
     orte_std_cntr_t addr_count;               
     orte_std_cntr_t addr_next;
     orte_std_cntr_t addr_alloc;
-    bool addr_matched;
-    struct sockaddr_in* addr_inet;    
+    orte_std_cntr_t addr_matched;/* status of already tried address classes */
+    struct sockaddr_storage *addr_inet; /* yes, we want storage here, so the indexes work out... */
 };
 typedef struct mca_oob_tcp_addr_t mca_oob_tcp_addr_t;
 
 OBJ_CLASS_DECLARATION(mca_oob_tcp_addr_t);
 
-#define MCA_OOB_TCP_ADDR_TYPE_AFINET   0x01
-
-/**
- * Unpack the contact information posted by the peer.
- */
-
-mca_oob_tcp_addr_t* mca_oob_tcp_addr_unpack(orte_buffer_t*);
-
-/**
- * Pack this hosts addressing info into a buffer for posting
- * into the registry.
- */
-
-int mca_oob_tcp_addr_pack(orte_buffer_t*);
-
 /**
  *
  */
 
-int mca_oob_tcp_addr_insert(mca_oob_tcp_addr_t*, const struct sockaddr_in*);
+int mca_oob_tcp_addr_insert(mca_oob_tcp_addr_t*, const struct sockaddr*);
 
 /**
  * 
  */
+ 
+int mca_oob_tcp_addr_get_next(mca_oob_tcp_addr_t*, struct sockaddr*);
 
-int mca_oob_tcp_addr_get_next(mca_oob_tcp_addr_t*, struct sockaddr_in*);
+END_C_DECLS
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
 #endif 
 
