@@ -26,15 +26,11 @@
 #include "orte/mca/ras/ras.h"
 #include "orte/mca/ras/base/base.h"
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
 
-    ORTE_DECLSPEC extern orte_ras_base_component_t mca_ras_slurm_component;
-    ORTE_DECLSPEC extern orte_ras_base_module_t orte_ras_slurm_module;
+ORTE_DECLSPEC extern orte_ras_base_component_t mca_ras_slurm_component;
+ORTE_DECLSPEC extern orte_ras_base_module_t orte_ras_slurm_module;
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
 
 #endif
