@@ -81,6 +81,7 @@ ORTE_DECLSPEC extern bool orte_in_parallel_debugger;
    simple (and throw an error if someone does try to use them) */
 struct orte_job_t;
 typedef struct orte_job_t orte_job_t;
+typedef uint16_t orte_mapping_policy_t;
 
 #else
 
@@ -137,7 +138,7 @@ typedef struct {
     /** Parent object */
     opal_object_t super;
     /** Unique index when multiple apps per job */
-    int8_t idx;
+    int32_t idx;
     /** Absolute pathname of argv[0] */
     char   *app;
     /** Number of copies of this process that are to be launched */
@@ -308,7 +309,7 @@ typedef struct {
     /* app_context array for this job */
     opal_pointer_array_t *apps;
     /* number of app_contexts in the array */
-    orte_std_cntr_t num_apps;
+    int32_t num_apps;
     /* flags to control the launch of this job - see above
      * for description of supported flags
      */
@@ -382,7 +383,7 @@ struct orte_proc_t {
     /* exit code */
     orte_exit_code_t exit_code;
     /* the app_context that generated this proc */
-    int8_t app_idx;
+    int32_t app_idx;
     /* a cpu list, if specified by the user */
     char *slot_list;
     /* pointer to the node where this proc is executing */
@@ -526,6 +527,9 @@ ORTE_DECLSPEC extern uint8_t orte_default_num_cores_per_socket;
 ORTE_DECLSPEC extern char *orte_default_cpu_set;
 ORTE_DECLSPEC extern char *orte_default_hostfile;
 ORTE_DECLSPEC extern char *orte_rankfile;
+#ifdef __WINDOWS__
+ORTE_DECLSPEC extern char *orte_ccp_headnode;
+#endif 
 
 /* default rank assigment and binding policy */
 ORTE_DECLSPEC extern orte_mapping_policy_t orte_default_mapping_policy;
