@@ -1602,7 +1602,19 @@ static int parse_locals(int argc, char* argv[])
     if (NULL != env) {
         size1 = opal_argv_count(env);
         for (j = 0; j < size1; ++j) {
-            putenv(env[j]);
+            /* Use-after-Free error possible here.  putenv does not copy
+             * the string passed to it, and instead stores only the pointer.
+             * env[j] may be freed later, in which case the pointer
+             * in environ will now be left dangling into a deallocated
+             * region.
+             * So we make a copy of the variable.
+             */
+            char *s = strdup(env[j]);
+
+            if (NULL == s) {
+                return OPAL_ERR_OUT_OF_RESOURCE;
+            }
+            putenv(s);
         }
     }
 
@@ -2174,19 +2186,6 @@ static int parse_appfile(char *filename, char ***env)
             if (made_app) {
                 app->idx = app_num;
                 ++app_num;
-                /* The v1.3/v1.4 series does not support more than 127
-                   app contexts.  v1.5 and beyond does support more
-                   than 127 contexts, but back-porting the fix would
-                   be a gigantic pain.  See
-                   https://svn.open-mpi.org/trac/ompi/ticket/2430 for
-                   more details. */
-                if (app_num > 127) {
-                    orte_show_help("help-orterun.txt", "too many app contexts",
-                                   true, orte_cmd_basename);
-                    fclose(fp);
-                    orte_finalize();
-                    exit(1);
-                }
                 opal_pointer_array_add(jdata->apps, app);
                 ++jdata->num_apps;
             }
